/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonTexture.h"
#include "math.h"

ButtonTexture::ButtonTexture() : 
	m_Data(NULL),

	useAlphaMap(false),

	memoryTopDC(0),
	memoryTopBitmap(0),

	memoryBottomDC(0),
	memoryBottomBitmap(0),

	memoryEntryDC(0),
	memoryEntryBitmap(0),

	memorySelEntryDC(0),
	memorySelEntryBitmap(0),

	memoryFolderDC(0),
	memoryFolderBitmap(0),

	memorySelFolderDC(0),
	memorySelFolderBitmap(0), 

	memoryInfoDC(0),
	memoryInfoBitmap(0), 

	memorySeparatorDC(0),
	memorySeparatorBitmap(0), 

	memoryArrowDC(0),
	memoryArrowBitmap(0), 

	memorySelArrowDC(0),
	memorySelArrowBitmap(0), 

	//Requested -> Must be initialized
	imageEntryHeight(20),
	imageFolderHeight(20),
	imageInfoHeight(20),
	imageSeparatorHeight(2),

	imageArrowHeight(0),
	imageArrowWidth(0),
	imageSelArrowHeight(0),
	imageSelArrowWidth(0),
	
	arrowSpacing(4),
	arrowOnLeft(false)
{
}

ButtonTexture::~ButtonTexture()
{
	if ( m_Data != NULL )
		delete m_Data;

	TopGradient.clear();
	BottomGradient.clear();
	EntryGradient.clear();
	SelEntryGradient.clear();
	FolderGradient.clear();
	SelFolderGradient.clear();
	InfoGradient.clear();
	SeparatorGradient.clear();

	if(memoryTopBitmap)
	{
		memoryTopBitmap = (HBITMAP) SelectObject(memoryTopDC, memoryTopBitmap);
		DeleteObject(memoryTopBitmap);
	}
	if(memoryTopDC)
		DeleteDC(memoryTopDC);

	if(memoryBottomBitmap)
	{
		memoryBottomBitmap = (HBITMAP) SelectObject(memoryBottomDC, memoryBottomBitmap);
		DeleteObject(memoryBottomBitmap);
	}
	if(memoryBottomDC)
		DeleteDC(memoryBottomDC);

	if(memoryEntryBitmap)
	{
		memoryEntryBitmap = (HBITMAP) SelectObject(memoryEntryDC, memoryEntryBitmap);
		DeleteObject(memoryEntryBitmap);
	}
	if(memoryEntryDC)
		DeleteDC(memoryEntryDC);

	if(memorySelEntryBitmap)
	{
		memorySelEntryBitmap = (HBITMAP) SelectObject(memorySelEntryDC, memorySelEntryBitmap);
		DeleteObject(memorySelEntryBitmap);
	}
	if(memorySelEntryDC)
		DeleteDC(memorySelEntryDC);

	if(memoryFolderBitmap)
	{
		memoryFolderBitmap = (HBITMAP) SelectObject(memoryFolderDC, memoryFolderBitmap);
		DeleteObject(memoryFolderBitmap);
	}
	if(memoryFolderDC)
		DeleteDC(memoryFolderDC);

	if(memorySelFolderBitmap)
	{
		memorySelFolderBitmap = (HBITMAP) SelectObject(memorySelFolderDC, memorySelFolderBitmap);
		DeleteObject(memorySelFolderBitmap);
	}
	if(memorySelFolderDC)
		DeleteDC(memorySelFolderDC);

	if(memoryInfoBitmap)
	{
		memoryInfoBitmap = (HBITMAP) SelectObject(memoryInfoDC, memoryInfoBitmap);
		DeleteObject(memoryInfoBitmap);
	}
	if(memoryInfoDC)
		DeleteDC(memoryInfoDC);

	if(memorySeparatorBitmap)
	{
		memorySeparatorBitmap = (HBITMAP) SelectObject(memorySeparatorDC, memorySeparatorBitmap);
		DeleteObject(memorySeparatorBitmap);
	}
	if(memorySeparatorDC)
		DeleteDC(memorySeparatorDC);

	if(memoryArrowBitmap)
	{
		memoryArrowBitmap = (HBITMAP) SelectObject(memoryArrowDC, memoryArrowBitmap);
		DeleteObject(memoryArrowBitmap);
	}
	if(memoryArrowDC)
		DeleteDC(memoryArrowDC);

	if(memorySelArrowBitmap)
	{
		memorySelArrowBitmap = (HBITMAP) SelectObject(memorySelArrowDC, memorySelArrowBitmap);
		DeleteObject(memorySelArrowBitmap);
	}
	if(memorySelArrowDC)
		DeleteDC(memorySelArrowDC);
}

bool ButtonTexture::useAlphaTransparency(int buttonType)
{
	if ( buttonType == -1 && top.alphaTransparency != 255 )
		return true;
	else if ( buttonType == -2 && bottom.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 1 && entry.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 2 && selentry.alphaTransparency != 255 )
		return true;
	else if (buttonType == 3 && folder.alphaTransparency != 255 )
		return true;
	else if (buttonType == 4 && selfolder.alphaTransparency != 255 )
		return true;
	else if (buttonType == 5 && info.alphaTransparency != 255 )
		return true;
	else if (buttonType == 10 && separator.alphaTransparency != 255 )
		return true;

	return false;
}

void ButtonTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	bgcolor = RGB(255,255,255);
	lightcolor = GetSysColor(COLOR_3DLIGHT);
	darkcolor = GetSysColor(COLOR_3DSHADOW);
		
	if ( cfg.empty() )
		return;

	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);
		
	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = GetSysColor(COLOR_3DLIGHT);
		darkcolor = GetSysColor(COLOR_3DSHADOW);
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = lightcolor;
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void ButtonTexture::parseGradientCFG(string cfg, int imageType)
{
	if ( cfg.empty() )
		return;

	int color;
	COLORREF insertColor;
	char token[4096];
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == -1)
		{
			TopGradient.insert(TopGradient.end(), insertColor );
		}
		else if (imageType == -2)
		{
			BottomGradient.insert(BottomGradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			EntryGradient.insert(EntryGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			SelEntryGradient.insert(SelEntryGradient.end(), insertColor );
		}
		else if (imageType == 3)
		{
			FolderGradient.insert(FolderGradient.end(), insertColor );
		}
		else if (imageType == 4)
		{
			SelFolderGradient.insert(SelFolderGradient.end(), insertColor );
		}
		else if (imageType == 5)
		{
			InfoGradient.insert(InfoGradient.end(), insertColor );
		}
		else if (imageType == 10)
		{
			SeparatorGradient.insert(SeparatorGradient.end(), insertColor );
		}
	}
}

void ButtonTexture::parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel )
{
	char left[32], right[32], top[32], bottom[32];
	char *buffers[] = { left, right, top, bottom };

	leftBevel = 0;
	rightBevel = 0;
	topBevel = 0;
	bottomBevel = 0;

	int numTokens = LCTokenize(cfg.c_str(), buffers, 4, NULL);
		
	if(numTokens == 1)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = leftBevel;
		bottomBevel = leftBevel;
	}
	else if (numTokens == 2)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = atoi(right);
		bottomBevel = topBevel;
	}
	else if (numTokens == 4)
	{
		leftBevel = atoi(left);
		rightBevel = atoi(right);
		topBevel = atoi(top);
		bottomBevel = atoi(bottom);
	}
}

void ButtonTexture::subconfigure(const string &uniquekey, const string &groupkey, const string &prefixkey,  BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings)
{
	NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
	};

	NameValuePair borderValues[] = 
	{
		{ ".none", 0 },
		{ "none", 0 },
		{ "raised", 1 },
		{ "sunken", 2 },
		{ "etched", 3 },
		{ "bump", 4 },
		{ 0, 0 }
	};

	NameValuePair gradientmodeConstants[] =
	{
		{ ".none", GRADIENT_NONE },
		{ ".horizontal", GRADIENT_HORIZONTAL },
		{ ".vertical", GRADIENT_VERTICAL },
		{ ".radial", GRADIENT_RADIAL },
		{ ".diagonal", GRADIENT_DIAGONAL },
		{ ".fdiagonal", GRADIENT_FDIAGONAL },
		{ ".bdiagonal", GRADIENT_BDIAGONAL },
		{ 0, 0 }
	};

	NameValuePair gradienttransformationConstants[] =
	{
		{ ".none", TRANSFORMATION_NONE },
		{ ".caricature", TRANSFORMATION_CHARICATURE },
		{ ".fisheye", TRANSFORMATION_FISHEYE },
		{ ".swirled", TRANSFORMATION_SWIRLED },
		{ ".cylinder", TRANSFORMATION_CYLINDER },
		{ ".shift", TRANSFORMATION_SHIFT },
		{ 0, 0 }
	};

	workSettings.image = Getx2RCString(uniquekey, groupkey, (prefixkey + "Image"), defaultSettings.image);

	workSettings.mode = Getx2RCNamedValue(uniquekey, groupkey, (prefixkey + "ImageMode"), modeConstants, defaultSettings.mode);

	workSettings.button_hueColor = Getx2RCColor(uniquekey, groupkey, (prefixkey + "HueColor"), defaultSettings.button_hueColor);
	workSettings.button_hueIntensity = Getx2RCInt(uniquekey, groupkey, (prefixkey + "HueIntensity"), defaultSettings.button_hueIntensity, 0, 100);
	workSettings.button_luminanceIntensity = Getx2RCInt(uniquekey, groupkey, (prefixkey + "LuminanceIntensity"), defaultSettings.button_luminanceIntensity, -100, 100);
	workSettings.button_saturationIntensity = Getx2RCInt(uniquekey, groupkey, (prefixkey + "SaturationIntensity"), defaultSettings.button_saturationIntensity, 0, 200);

	workSettings.button_mixColor = Getx2RCColor(uniquekey, groupkey, (prefixkey + "MixColor"), defaultSettings.button_mixColor);
	workSettings.button_mixIntensity = Getx2RCInt(uniquekey, groupkey, (prefixkey + "MixIntensity"), defaultSettings.button_mixIntensity, 0, 100);

	workSettings.leftEdge = Getx2RCInt(uniquekey, groupkey, (prefixkey + "ImageLeftEdge"), defaultSettings.leftEdge, 0);
	workSettings.topEdge = Getx2RCInt(uniquekey, groupkey, (prefixkey + "ImageTopEdge"), defaultSettings.topEdge, 0);
	workSettings.rightEdge = Getx2RCInt(uniquekey, groupkey, (prefixkey + "ImageRightEdge"), defaultSettings.rightEdge, 0);
	workSettings.bottomEdge = Getx2RCInt(uniquekey, groupkey, (prefixkey + "ImageBottomEdge"), defaultSettings.bottomEdge, 0);

	workSettings.solidcolors = Getx2RCLine(uniquekey, groupkey, (prefixkey + "SolidColors"), defaultSettings.solidcolors);
	workSettings.gradientcolors = Getx2RCLine(uniquekey, groupkey, (prefixkey + "SolidGradientColors"), defaultSettings.gradientcolors);

	workSettings.bevelsize = Getx2RCLine(uniquekey, groupkey, (prefixkey + "SolidBevelSize"), defaultSettings.bevelsize);

	workSettings.verticalGradient = Getx2RCBool(uniquekey, groupkey, (prefixkey + "SolidGradientVertical"));
	workSettings.repeatedGradient = Getx2RCInt(uniquekey, groupkey, (prefixkey + "SolidGradientRepeated"), defaultSettings.repeatedGradient, 0);

	workSettings.gradientType = Getx2RCNamedValue(uniquekey, groupkey, (prefixkey + "SolidGradientType"), gradientmodeConstants, defaultSettings.gradientType);
	workSettings.transformationType = Getx2RCNamedValue(uniquekey, groupkey, (prefixkey + "SolidTransformationType"), gradienttransformationConstants, defaultSettings.transformationType);

	workSettings.alphaTransparency = Getx2RCInt(uniquekey, groupkey, (prefixkey + "AlphaTransparency"), defaultSettings.alphaTransparency, 0, 255);
	workSettings.borderMethod = Getx2RCNamedValue(uniquekey, groupkey, (prefixkey + "BorderMethod"), borderValues, defaultSettings.borderMethod);
	workSettings.trueTransparent = Getx2RCBool(uniquekey, groupkey, (prefixkey + "TrueTransparent"), defaultSettings.trueTransparent);
}

void ButtonTexture::configure(const string &uniquekey, const string &groupkey, bool alphaMap)
{
	defaultentry.image = "";

	defaultentry.mode = MULTIBLT_STRETCH;

	defaultentry.button_hueColor = RGB(255, 255, 255);
	defaultentry.button_hueIntensity = 0;
	defaultentry.button_luminanceIntensity = 0;
	defaultentry.button_saturationIntensity = 100;

	defaultentry.button_mixColor = RGB(255, 255, 255);
	defaultentry.button_mixIntensity = 0;

	defaultentry.leftEdge = 0;
	defaultentry.topEdge = 0;
	defaultentry.rightEdge = 0;
	defaultentry.bottomEdge = 0;


	defaultentry.solidcolors = "";
	defaultentry.gradientcolors = "";
	
	defaultentry.bevelsize = "0";

	defaultentry.verticalGradient = false;
	defaultentry.repeatedGradient = 0;

	defaultentry.gradientType = -1;
	defaultentry.transformationType = -1;

	defaultentry.alphaTransparency = 255;
	defaultentry.borderMethod = 0;
	defaultentry.trueTransparent = false;

	useAlphaMap = alphaMap;

	activeFolderUseFolderDefault = Getx2RCBool(uniquekey, groupkey, "ActiveFolderUseFolderDefault");

	subconfigure(uniquekey, groupkey, "Title", top, defaultentry);

	subconfigure(uniquekey, groupkey, "Bottom", bottom, defaultentry);

	subconfigure(uniquekey, groupkey, "Entry", entry, defaultentry);

	subconfigure(uniquekey, groupkey, "ActiveEntry", selentry, entry);

	subconfigure(uniquekey, groupkey, "Folder", folder, entry);

	subconfigure(uniquekey, groupkey, "ActiveFolder", selfolder, (activeFolderUseFolderDefault ? folder : selentry));

	subconfigure(uniquekey, groupkey, "Info", info, defaultentry);

	subconfigure(uniquekey, groupkey, "Separator", separator, defaultentry);


	usetopSolidColors = Getx2RCBool(uniquekey, groupkey, "UseTitleSolidColors");

	if (usetopSolidColors)
	{
		if ( !top.solidcolors.empty() )
		{
			parseCFG(top.solidcolors, top.backgroundColor, top.lightBevelColor, top.darkBevelColor);
		}
		else
		{
			top.backgroundColor = GetSysColor(COLOR_BTNFACE);
			top.lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			top.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		TopGradient.insert(TopGradient.end(), top.backgroundColor );

		parseBevelCFG(top.bevelsize, top.bevelSizeLeft, top.bevelSizeRight, top.bevelSizeTop, top.bevelSizeBottom);

		parseGradientCFG(top.gradientcolors, -1);
	}
	else
	{
		if (!top.image.empty())
		{
			memoryTopBitmap = LoadLSImage(top.image.c_str(), 0);

			if (memoryTopBitmap)
			{
				GetLSBitmapSize(memoryTopBitmap, &imageTopWidth, &imageTopHeight);

				memoryTopDC = CreateCompatibleDC(NULL);
				SelectObject(memoryTopDC, memoryTopBitmap);	
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryTopBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memoryTopBitmap, top.button_hueColor, top.button_hueIntensity, top.button_luminanceIntensity, top.button_saturationIntensity, top.button_mixColor, top.button_mixIntensity);
				}
				else if ( top.button_hueIntensity != 0 || top.button_luminanceIntensity != 0 || top.button_saturationIntensity != 100 )
				{
					paintEffects(memoryTopDC, imageTopWidth, imageTopHeight, top.button_hueColor, top.button_hueIntensity, top.button_luminanceIntensity, top.button_saturationIntensity, top.button_mixColor, top.button_mixIntensity);
				}
			}
		}
	}

	usebottomSolidColors = Getx2RCBool(uniquekey, groupkey, "UseBottomSolidColors");

	if (usebottomSolidColors)
	{
		if ( !bottom.solidcolors.empty() )
		{
			parseCFG(bottom.solidcolors, bottom.backgroundColor, bottom.lightBevelColor, bottom.darkBevelColor);
		}
		else
		{
			bottom.backgroundColor = GetSysColor(COLOR_BTNFACE);
			bottom.lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			bottom.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		BottomGradient.insert(BottomGradient.end(), bottom.backgroundColor );

		parseBevelCFG(bottom.bevelsize, bottom.bevelSizeLeft, bottom.bevelSizeRight, bottom.bevelSizeTop, bottom.bevelSizeBottom);

		parseGradientCFG(bottom.gradientcolors, -2);
	}
	else
	{
		if (!bottom.image.empty())
		{
			memoryBottomBitmap = LoadLSImage(bottom.image.c_str(), 0);

			if (memoryBottomBitmap)
			{
				GetLSBitmapSize(memoryBottomBitmap, &imageBottomWidth, &imageBottomHeight);

				memoryBottomDC = CreateCompatibleDC(NULL);
				SelectObject(memoryBottomDC, memoryBottomBitmap);
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryBottomBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memoryBottomBitmap, bottom.button_hueColor, bottom.button_hueIntensity, bottom.button_luminanceIntensity, bottom.button_saturationIntensity, bottom.button_mixColor, bottom.button_mixIntensity);
				}
				else if ( bottom.button_hueIntensity != 0 || bottom.button_luminanceIntensity != 0 || bottom.button_saturationIntensity != 100 )
				{
					paintEffects(memoryBottomDC, imageBottomWidth, imageBottomHeight, bottom.button_hueColor, bottom.button_hueIntensity, bottom.button_luminanceIntensity, bottom.button_saturationIntensity, bottom.button_mixColor, bottom.button_mixIntensity);
				}
			}
		}
	}


	useentrySolidColors = Getx2RCBool(uniquekey, groupkey, "UseEntrySolidColors");

	if (useentrySolidColors)
	{
		if ( !entry.solidcolors.empty() )
		{
			parseCFG(entry.solidcolors, entry.backgroundColor, entry.lightBevelColor, entry.darkBevelColor);
		}
		else
		{
			entry.backgroundColor = GetSysColor(COLOR_BTNFACE);
			entry.lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			entry.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		EntryGradient.insert(EntryGradient.end(), entry.backgroundColor );

		parseBevelCFG(entry.bevelsize, entry.bevelSizeLeft, entry.bevelSizeRight, entry.bevelSizeTop, entry.bevelSizeBottom);

		parseGradientCFG(entry.gradientcolors, 1);
	}
	else
	{
		if (!entry.image.empty())
		{
			memoryEntryBitmap = LoadLSImage(entry.image.c_str(), 0);

			if (memoryEntryBitmap)
			{
				GetLSBitmapSize(memoryEntryBitmap, &imageEntryWidth, &imageEntryHeight);

				memoryEntryDC = CreateCompatibleDC(NULL);
				SelectObject(memoryEntryDC, memoryEntryBitmap);
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryEntryBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memoryEntryBitmap, entry.button_hueColor, entry.button_hueIntensity, entry.button_luminanceIntensity, entry.button_saturationIntensity, entry.button_mixColor, entry.button_mixIntensity);
				}
				else if ( entry.button_hueIntensity != 0 || entry.button_luminanceIntensity != 0 || entry.button_saturationIntensity != 100 )
				{
					paintEffects(memoryEntryDC, imageEntryWidth, imageEntryHeight, entry.button_hueColor, entry.button_hueIntensity, entry.button_luminanceIntensity, entry.button_saturationIntensity, entry.button_mixColor, entry.button_mixIntensity);
				}
			}
		}
	}

	useselentrySolidColors = Getx2RCBool(uniquekey, groupkey, "UseActiveEntrySolidColors", useentrySolidColors);

	if (useselentrySolidColors)
	{
		if ( !selentry.solidcolors.empty() )
		{
			parseCFG(selentry.solidcolors, selentry.backgroundColor, selentry.lightBevelColor, selentry.darkBevelColor);
		}
		else
		{
			selentry.backgroundColor = GetSysColor(COLOR_BTNHIGHLIGHT);
			selentry.lightBevelColor = GetSysColor(COLOR_3DHILIGHT);
			selentry.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		SelEntryGradient.insert(SelEntryGradient.end(), selentry.backgroundColor );

		parseBevelCFG(selentry.bevelsize, selentry.bevelSizeLeft, selentry.bevelSizeRight, selentry.bevelSizeTop, selentry.bevelSizeBottom);

		parseGradientCFG(selentry.gradientcolors, 2);
	}
	else
	{
		if (!selentry.image.empty())
		{
			memorySelEntryBitmap = LoadLSImage(selentry.image.c_str(), 0);

			if (memorySelEntryBitmap)
			{
				GetLSBitmapSize(memorySelEntryBitmap, &imageSelEntryWidth, &imageSelEntryHeight);

				memorySelEntryDC = CreateCompatibleDC(NULL);
				SelectObject(memorySelEntryDC, memorySelEntryBitmap);
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memorySelEntryBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memorySelEntryBitmap, selentry.button_hueColor, selentry.button_hueIntensity, selentry.button_luminanceIntensity, selentry.button_saturationIntensity, selentry.button_mixColor, selentry.button_mixIntensity);
				}
				else if ( selentry.button_hueIntensity != 0 || selentry.button_luminanceIntensity != 0 || selentry.button_saturationIntensity != 100 )
				{
					paintEffects(memorySelEntryDC, imageSelEntryWidth, imageSelEntryHeight, selentry.button_hueColor, selentry.button_hueIntensity, selentry.button_luminanceIntensity, selentry.button_saturationIntensity, selentry.button_mixColor, selentry.button_mixIntensity);
				}
			}
		}
	}

	usefolderSolidColors = Getx2RCBool(uniquekey, groupkey, "UseFolderSolidColors", useentrySolidColors);

	if (usefolderSolidColors)
	{
		if ( !folder.solidcolors.empty() )
		{
			parseCFG(folder.solidcolors, folder.backgroundColor, folder.lightBevelColor, folder.darkBevelColor);
		}
		else
		{
			folder.backgroundColor = GetSysColor(COLOR_BTNFACE);
			folder.lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			folder.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		FolderGradient.insert(FolderGradient.end(), folder.backgroundColor );

		parseBevelCFG(folder.bevelsize, folder.bevelSizeLeft, folder.bevelSizeRight, folder.bevelSizeTop, folder.bevelSizeBottom);

		parseGradientCFG(folder.gradientcolors, 3);
	}
	else
	{
		if (!folder.image.empty())
		{
			memoryFolderBitmap = LoadLSImage(folder.image.c_str(), 0);

			if (memoryFolderBitmap)
			{
				GetLSBitmapSize(memoryFolderBitmap, &imageFolderWidth, &imageFolderHeight);

				memoryFolderDC = CreateCompatibleDC(NULL);
				SelectObject(memoryFolderDC, memoryFolderBitmap);
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryFolderBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memoryFolderBitmap, folder.button_hueColor, folder.button_hueIntensity, folder.button_luminanceIntensity, folder.button_saturationIntensity, folder.button_mixColor, folder.button_mixIntensity);
				}
				else if ( folder.button_hueIntensity != 0 || folder.button_luminanceIntensity != 0 || folder.button_saturationIntensity != 100 )
				{
					paintEffects(memoryFolderDC, imageFolderWidth, imageFolderHeight, folder.button_hueColor, folder.button_hueIntensity, folder.button_luminanceIntensity, folder.button_saturationIntensity, folder.button_mixColor, folder.button_mixIntensity);
				}
			}
		}
	}

	useselfolderSolidColors = Getx2RCBool(uniquekey, groupkey, "UseActiveFolderSolidColors", (activeFolderUseFolderDefault ? usefolderSolidColors : useselentrySolidColors));

	if (useselfolderSolidColors)
	{
		if ( !selfolder.solidcolors.empty() )
		{
			parseCFG(selfolder.solidcolors, selfolder.backgroundColor, selfolder.lightBevelColor, selfolder.darkBevelColor);
		}
		else
		{
			if ( activeFolderUseFolderDefault )
			{
				selfolder.backgroundColor = GetSysColor(COLOR_BTNFACE);
				selfolder.lightBevelColor = GetSysColor(COLOR_3DLIGHT);
				selfolder.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
			}
			else
			{
				selfolder.backgroundColor = GetSysColor(COLOR_BTNHIGHLIGHT);
				selfolder.lightBevelColor = GetSysColor(COLOR_3DHILIGHT);
				selfolder.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
			}
		}
		//BG Insert in Gradient List
		SelFolderGradient.insert(SelFolderGradient.end(), selfolder.backgroundColor );

		parseBevelCFG(selfolder.bevelsize, selfolder.bevelSizeLeft, selfolder.bevelSizeRight, selfolder.bevelSizeTop, selfolder.bevelSizeBottom);
	
		parseGradientCFG(selfolder.gradientcolors, 4);
	}
	else
	{
		if (!selfolder.image.empty())
		{
			memorySelFolderBitmap = LoadLSImage(selfolder.image.c_str(), 0);

			if (memorySelFolderBitmap)
			{ 
				GetLSBitmapSize(memorySelFolderBitmap, &imageSelFolderWidth, &imageSelFolderHeight);

				memorySelFolderDC = CreateCompatibleDC(NULL);
				SelectObject(memorySelFolderDC, memorySelFolderBitmap);	
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memorySelFolderBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memorySelFolderBitmap, selfolder.button_hueColor, selfolder.button_hueIntensity, selfolder.button_luminanceIntensity, selfolder.button_saturationIntensity, selfolder.button_mixColor, selfolder.button_mixIntensity);
				}
				else if ( selfolder.button_hueIntensity != 0 || selfolder.button_luminanceIntensity != 0 || selfolder.button_saturationIntensity != 100 )
				{
					paintEffects(memorySelFolderDC, imageSelFolderWidth, imageSelFolderHeight, selfolder.button_hueColor, selfolder.button_hueIntensity, selfolder.button_luminanceIntensity, selfolder.button_saturationIntensity, selfolder.button_mixColor, selfolder.button_mixIntensity);
				}
			}
		}
	}

	useinfoSolidColors = Getx2RCBool(uniquekey, groupkey, "UseInfoSolidColors");

	if (useinfoSolidColors)
	{
		parseCFG(info.solidcolors, info.backgroundColor, info.lightBevelColor, info.darkBevelColor);
		//BG Insert in Gradient List
		InfoGradient.insert(InfoGradient.end(), info.backgroundColor );

		parseBevelCFG(info.bevelsize, info.bevelSizeLeft, info.bevelSizeRight, info.bevelSizeTop, info.bevelSizeBottom);
	
		parseGradientCFG(info.gradientcolors, 5);
	}
	else
	{
		if (!info.image.empty())
		{
			memoryInfoBitmap = LoadLSImage(info.image.c_str(), 0);

			if (memoryInfoBitmap)
			{
				GetLSBitmapSize(memoryInfoBitmap, &imageInfoWidth, &imageInfoHeight);

				memoryInfoDC = CreateCompatibleDC(NULL);
				SelectObject(memoryInfoDC, memoryInfoBitmap);
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryInfoBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memoryInfoBitmap, info.button_hueColor, info.button_hueIntensity, info.button_luminanceIntensity, info.button_saturationIntensity, info.button_mixColor, info.button_mixIntensity);
				}
				else if ( info.button_hueIntensity != 0 || info.button_luminanceIntensity != 0 || info.button_saturationIntensity != 100 )
				{
					paintEffects(memoryInfoDC, imageInfoWidth, imageInfoHeight, info.button_hueColor, info.button_hueIntensity, info.button_luminanceIntensity, info.button_saturationIntensity, info.button_mixColor, info.button_mixIntensity);
				}
			}
		}
	}

	//Special Textures
	//----------------
	useseparatorSolidColors = Getx2RCBool(uniquekey, groupkey, "UseSeparatorSolidColors");

	if (useseparatorSolidColors)
	{
		parseCFG(separator.solidcolors, separator.backgroundColor, separator.lightBevelColor, separator.darkBevelColor);
		//BG Insert in Gradient List
		SeparatorGradient.insert(SeparatorGradient.end(), separator.backgroundColor );

		parseBevelCFG(separator.bevelsize, separator.bevelSizeLeft, separator.bevelSizeRight, separator.bevelSizeTop, separator.bevelSizeBottom);
	
		parseGradientCFG(separator.gradientcolors, 10);
	}
	else
	{
		if (!separator.image.empty())
		{
			memorySeparatorBitmap = LoadLSImage(separator.image.c_str(), 0);

			if (memorySeparatorBitmap)
			{ 
				GetLSBitmapSize(memorySeparatorBitmap, &imageSeparatorWidth, &imageSeparatorHeight);

				memorySeparatorDC = CreateCompatibleDC(NULL);
				SelectObject(memorySeparatorDC, memorySeparatorBitmap);	
				
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memorySeparatorBitmap, sizeof(BITMAP), &bm);

					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
						PrepareAlphaChannel(memorySeparatorBitmap, separator.button_hueColor, separator.button_hueIntensity, separator.button_luminanceIntensity, separator.button_saturationIntensity, separator.button_mixColor, separator.button_mixIntensity);
				}
				else if ( separator.button_hueIntensity != 0 || separator.button_luminanceIntensity != 0 || separator.button_saturationIntensity != 100 )
				{
					paintEffects(memorySeparatorDC, imageSeparatorWidth, imageSeparatorHeight, separator.button_hueColor, separator.button_hueIntensity, separator.button_luminanceIntensity, separator.button_saturationIntensity, separator.button_mixColor, separator.button_mixIntensity);
				}
			}
		}
	}

	string imageFile = "";
	//Folder Arrows (Normal and Active)
	imageFile = Getx2RCString(uniquekey, groupkey, "ArrowImage", "");
	if (!imageFile.empty())
	{
		memoryArrowBitmap = LoadLSImage(imageFile.c_str(), 0);

		if (memoryArrowBitmap)
		{
			GetLSBitmapSize(memoryArrowBitmap, &imageArrowWidth, &imageArrowHeight);

			arrowSpacing = Getx2RCInt(uniquekey, groupkey, "ArrowOffset", 4, 0);
			arrowOnLeft = Getx2RCBool(uniquekey, groupkey, "ArrowOnLeft");

			memoryArrowDC = CreateCompatibleDC(NULL);
			SelectObject(memoryArrowDC, memoryArrowBitmap);	
			
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryArrowBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryArrowBitmap, RGB(255,255,255), 0, 0, 100, RGB(255,255,255), 0);
			}
		}
	}

	imageFile = Getx2RCString(uniquekey, groupkey, "ActiveArrowImage", imageFile);
	if (!imageFile.empty())
	{
		memorySelArrowBitmap = LoadLSImage(imageFile.c_str(), 0);

		if (memorySelArrowBitmap)
		{
			GetLSBitmapSize(memorySelArrowBitmap, &imageSelArrowWidth, &imageSelArrowHeight);

			selarrowSpacing = Getx2RCInt(uniquekey, groupkey, "ActiveArrowOffset", arrowSpacing, 0);
			selarrowOnLeft = Getx2RCBool(uniquekey, groupkey, "ActiveArrowOnLeft", arrowOnLeft);

			memorySelArrowDC = CreateCompatibleDC(NULL);
			SelectObject(memorySelArrowDC, memorySelArrowBitmap);
			
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memorySelArrowBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memorySelArrowBitmap, RGB(255,255,255), 0, 0, 100, RGB(255,255,255), 0 );
			}
		}
	}	
}

void ButtonTexture::apply(HDC hDC, int x, int y, int width, int height, int buttonType)
{
	if ( width <= 0 || height <= 0 )
		return;

	if ( useAlphaTransparency(buttonType) || useAlphaMap )
	{
		applyOverlay(hDC, x, y, width, height, buttonType);
		return;
	}

	RECT r;
	r.left = x;
	r.top = y;
	r.right = r.left + width;
	r.bottom = r.top + height;

	int usedBorderMethod = 0;

	int bevelWidth = 0;
	int bevelHeight = 0;

	SetStretchBltMode(hDC, STRETCH_DELETESCANS);

	if (buttonType == -1 && usetopSolidColors)
	{
		usedBorderMethod = top.borderMethod;
	
		bevelWidth = top.bevelSizeLeft+top.bevelSizeRight;
		bevelHeight = top.bevelSizeTop+top.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, top.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-top.bevelSizeRight;
			r.bottom = r.bottom-top.bevelSizeBottom;
			::SetBkColor(hDC, top.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+top.bevelSizeLeft;
			r.top = r.top+top.bevelSizeTop;

			if ( TopGradient.size() > 1 )
			{
				if ( top.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, TopGradient, r, buttonType);
				else
					DrawGradient(hDC, r, TopGradient[0], TopGradient[1], top.gradientType, top.transformationType);
			}
			else
			{
				::SetBkColor(hDC, top.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( TopGradient.size() > 1 )
			{
				if ( top.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, TopGradient, r, buttonType);
				else
					DrawGradient(hDC, r, TopGradient[0], TopGradient[1], top.gradientType, top.transformationType);
			}
			else
			{
				::SetBkColor(hDC, top.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (buttonType == -2 && usebottomSolidColors)
	{
		usedBorderMethod = bottom.borderMethod;
	
		bevelWidth = bottom.bevelSizeLeft+bottom.bevelSizeRight;
		bevelHeight = bottom.bevelSizeTop+bottom.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, bottom.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-bottom.bevelSizeRight;
			r.bottom = r.bottom-bottom.bevelSizeBottom;
			::SetBkColor(hDC, bottom.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+bottom.bevelSizeLeft;
			r.top = r.top+bottom.bevelSizeTop;

			if ( BottomGradient.size() > 1 )
			{
				if ( bottom.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, BottomGradient, r, buttonType);
				else
					DrawGradient(hDC, r, BottomGradient[0], BottomGradient[1], bottom.gradientType, bottom.transformationType);
			}
			else
			{
				::SetBkColor(hDC, bottom.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( BottomGradient.size() > 1 )
			{
				if ( bottom.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, BottomGradient, r, buttonType);
				else
					DrawGradient(hDC, r, BottomGradient[0], BottomGradient[1], bottom.gradientType, bottom.transformationType);
			}
			else
			{
				::SetBkColor(hDC, bottom.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (buttonType == 1 && useentrySolidColors)
	{
		usedBorderMethod = entry.borderMethod;
	
		bevelWidth = entry.bevelSizeLeft+entry.bevelSizeRight;
		bevelHeight = entry.bevelSizeTop+entry.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, entry.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-entry.bevelSizeRight;
			r.bottom = r.bottom-entry.bevelSizeBottom;
			::SetBkColor(hDC, entry.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+entry.bevelSizeLeft;
			r.top = r.top+entry.bevelSizeTop;

			if ( EntryGradient.size() > 1 )
			{
				if ( entry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, EntryGradient, r, buttonType);
				else
					DrawGradient(hDC, r, EntryGradient[0], EntryGradient[1], entry.gradientType, entry.transformationType);
			}
			else
			{
				::SetBkColor(hDC, entry.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( EntryGradient.size() > 1 )
			{
				if ( entry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, EntryGradient, r, buttonType);
				else
					DrawGradient(hDC, r, EntryGradient[0], EntryGradient[1], entry.gradientType, entry.transformationType);
			}
			else
			{
				::SetBkColor(hDC, entry.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 2 && useselentrySolidColors )
	{
		usedBorderMethod = selentry.borderMethod;

		bevelWidth = selentry.bevelSizeLeft+selentry.bevelSizeRight;
		bevelHeight = selentry.bevelSizeTop+selentry.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, selentry.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-selentry.bevelSizeRight;
			r.bottom = r.bottom-selentry.bevelSizeBottom;
			::SetBkColor(hDC, selentry.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+selentry.bevelSizeLeft;
			r.top = r.top+selentry.bevelSizeTop;

			if ( SelEntryGradient.size() > 1 )
			{
				if ( selentry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, SelEntryGradient, r, buttonType);
				else
					DrawGradient(hDC, r, SelEntryGradient[0], SelEntryGradient[1], selentry.gradientType, selentry.transformationType);
			}
			else
			{
				::SetBkColor(hDC, selentry.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SelEntryGradient.size() > 1 )
			{
				if ( selentry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, SelEntryGradient, r, buttonType);
				else
					DrawGradient(hDC, r, SelEntryGradient[0], SelEntryGradient[1], selentry.gradientType, selentry.transformationType);
			}
			else
			{
				::SetBkColor(hDC, selentry.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 3 && usefolderSolidColors )
	{
		usedBorderMethod = folder.borderMethod;

		bevelWidth = folder.bevelSizeLeft+folder.bevelSizeRight;
		bevelHeight = folder.bevelSizeTop+folder.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, folder.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-folder.bevelSizeRight;
			r.bottom = r.bottom-folder.bevelSizeBottom;
			::SetBkColor(hDC, folder.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+folder.bevelSizeLeft;
			r.top = r.top+folder.bevelSizeTop;

			if ( FolderGradient.size() > 1 )
			{
				if ( folder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, FolderGradient, r, buttonType);
				else
					DrawGradient(hDC, r, FolderGradient[0], FolderGradient[1], folder.gradientType, folder.transformationType);
			}
			else
			{
				::SetBkColor(hDC, folder.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FolderGradient.size() > 1 )
			{
				if ( folder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, FolderGradient, r, buttonType);
				else
					DrawGradient(hDC, r, FolderGradient[0], FolderGradient[1], folder.gradientType, folder.transformationType);
			}
			else
			{
				::SetBkColor(hDC, folder.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 4 && useselfolderSolidColors )
	{
		usedBorderMethod = selfolder.borderMethod;

		bevelWidth = selfolder.bevelSizeLeft+selfolder.bevelSizeRight;
		bevelHeight = selfolder.bevelSizeTop+selfolder.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, selfolder.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-selfolder.bevelSizeRight;
			r.bottom = r.bottom-selfolder.bevelSizeBottom;
			::SetBkColor(hDC, selfolder.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+selfolder.bevelSizeLeft;
			r.top = r.top+selfolder.bevelSizeTop;

			if ( SelFolderGradient.size() > 1 )
			{
				if ( selfolder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, SelFolderGradient, r, buttonType);
				else
					DrawGradient(hDC, r, SelFolderGradient[0], SelFolderGradient[1], selfolder.gradientType, selfolder.transformationType);
			}
			else
			{
				::SetBkColor(hDC, selfolder.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SelFolderGradient.size() > 1 )
			{
				if ( selfolder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, SelFolderGradient, r, buttonType);
				else
					DrawGradient(hDC, r, SelFolderGradient[0], SelFolderGradient[1], selfolder.gradientType, selfolder.transformationType);
			}
			else
			{
				::SetBkColor(hDC, selfolder.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 5 && useinfoSolidColors )
	{
		usedBorderMethod = info.borderMethod;

		bevelWidth = info.bevelSizeLeft+info.bevelSizeRight;
		bevelHeight = info.bevelSizeTop+info.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, info.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-info.bevelSizeRight;
			r.bottom = r.bottom-info.bevelSizeBottom;
			::SetBkColor(hDC, info.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+info.bevelSizeLeft;
			r.top = r.top+info.bevelSizeTop;

			if ( InfoGradient.size() > 1 )
			{
				if ( info.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, InfoGradient, r, buttonType);
				else
					DrawGradient(hDC, r, InfoGradient[0], InfoGradient[1], info.gradientType, info.transformationType);
			}
			else
			{
				::SetBkColor(hDC, info.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( InfoGradient.size() > 1 )
			{
				if ( info.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, InfoGradient, r, buttonType);
				else
					DrawGradient(hDC, r, InfoGradient[0], InfoGradient[1], info.gradientType, info.transformationType);
			}
			else
			{
				::SetBkColor(hDC, info.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 10 && useseparatorSolidColors )
	{
		usedBorderMethod = separator.borderMethod;

		bevelWidth = separator.bevelSizeLeft+separator.bevelSizeRight;
		bevelHeight = separator.bevelSizeTop+separator.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, separator.darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-separator.bevelSizeRight;
			r.bottom = r.bottom-separator.bevelSizeBottom;
			::SetBkColor(hDC, separator.lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+separator.bevelSizeLeft;
			r.top = r.top+separator.bevelSizeTop;

			if ( SeparatorGradient.size() > 1 )
			{
				if ( separator.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, SeparatorGradient, r, buttonType);
				else
					DrawGradient(hDC, r, SeparatorGradient[0], SeparatorGradient[1], separator.gradientType, separator.transformationType);
			}
			else
			{
				::SetBkColor(hDC, separator.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SeparatorGradient.size() > 1 )
			{
				if ( separator.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(hDC, SeparatorGradient, r, buttonType);
				else
					DrawGradient(hDC, r, SeparatorGradient[0], SeparatorGradient[1], separator.gradientType, separator.transformationType);
			}
			else
			{
				::SetBkColor(hDC, separator.backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if ( buttonType == -1 && memoryTopBitmap == 0 )
			return;

		if ( buttonType == -2 && memoryBottomBitmap == 0 )
			return;

		if ( buttonType == 31 && memoryArrowBitmap == 0 )
			return;
		if ( buttonType == 41 && memorySelArrowBitmap == 0 )
			return;

		if ( buttonType == 10 && memorySeparatorBitmap == 0 )
			return;

		if ( buttonType == 5 && memoryInfoBitmap == 0 )
			return;
		if ( buttonType == 4 && memorySelFolderBitmap == 0 )
			buttonType = (activeFolderUseFolderDefault ? 3 : 2);		
		if ( buttonType == 3 && memoryFolderBitmap == 0 )
			buttonType = 1;	
		if ( buttonType == 2 && memorySelEntryBitmap == 0 )
			buttonType = 1;
		
		if (buttonType == 1 && memoryEntryBitmap == 0)
			return;

		bool useTrueTransparent = false;

		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

		if ( buttonType == 31 || buttonType == 41 )
		{
			r.left = 0;
			r.top = 0;
			r.right = width;
			r.bottom = height;

			::SetBkColor(bufferDC, RGB(255,0,255));
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
		}
	
		if (buttonType == -1)
		{
			usedBorderMethod = top.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryTopDC, 0, 0, imageTopWidth, imageTopHeight,
					top.leftEdge, top.topEdge, top.rightEdge, top.bottomEdge,
					top.mode, SRCCOPY);

			useTrueTransparent = top.trueTransparent;
		}
		else if (buttonType == -2)
		{
			usedBorderMethod = bottom.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryBottomDC, 0, 0, imageBottomWidth, imageBottomHeight,
					bottom.leftEdge, bottom.topEdge, bottom.rightEdge, bottom.bottomEdge,
					bottom.mode, SRCCOPY);

			useTrueTransparent = bottom.trueTransparent;
		}
		else if (buttonType == 1)
		{
			usedBorderMethod = entry.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryEntryDC, 0, 0, imageEntryWidth, imageEntryHeight,
					entry.leftEdge, entry.topEdge, entry.rightEdge, entry.bottomEdge,
					entry.mode, SRCCOPY);

			useTrueTransparent = entry.trueTransparent;
		}
		else if (buttonType == 2)
		{
			usedBorderMethod = selentry.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memorySelEntryDC, 0, 0, imageSelEntryWidth, imageSelEntryHeight,
					selentry.leftEdge, selentry.topEdge, selentry.rightEdge, selentry.bottomEdge,
					selentry.mode, SRCCOPY);

			useTrueTransparent = selentry.trueTransparent;

		}
		else if (buttonType == 3)
		{
			usedBorderMethod = folder.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFolderDC, 0, 0, imageFolderWidth, imageFolderHeight,
					folder.leftEdge, folder.topEdge, folder.rightEdge, folder.bottomEdge,
					folder.mode, SRCCOPY);

			useTrueTransparent = folder.trueTransparent;
		}
		else if (buttonType == 4)
		{
			usedBorderMethod = selfolder.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memorySelFolderDC, 0, 0, imageSelFolderWidth, imageSelFolderHeight,
					selfolder.leftEdge, selfolder.topEdge, selfolder.rightEdge, selfolder.bottomEdge,
					selfolder.mode, SRCCOPY);

			useTrueTransparent = selfolder.trueTransparent;
		}
		else if (buttonType == 5)
		{
			usedBorderMethod = info.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryInfoDC, 0, 0, imageInfoWidth, imageInfoHeight,
					info.leftEdge, info.topEdge, info.rightEdge, info.bottomEdge,
					info.mode, SRCCOPY);

			useTrueTransparent = info.trueTransparent;
		}
		else if (buttonType == 10)
		{
			usedBorderMethod = separator.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
						memorySeparatorDC, 0, 0, imageSeparatorWidth, imageSeparatorHeight,
						separator.leftEdge, separator.topEdge, separator.rightEdge, separator.bottomEdge,
						separator.mode, SRCCOPY);

			useTrueTransparent = separator.trueTransparent;
		}
		else if ( buttonType == 31 )
		{
			MultiBlt(bufferDC, (arrowOnLeft ? arrowSpacing : width-imageArrowWidth-arrowSpacing), (height-imageArrowHeight)/2, imageArrowWidth, imageArrowHeight,
						memoryArrowDC, 0, 0, imageArrowWidth, imageArrowHeight,
						0, 0, 0, 0,
						MULTIBLT_STRETCH, SRCCOPY);
		}
		else if ( buttonType == 41 )
		{
			MultiBlt(bufferDC, (selarrowOnLeft ? selarrowSpacing : width-imageSelArrowWidth-selarrowSpacing), (height-imageSelArrowHeight)/2, imageSelArrowWidth, imageSelArrowHeight,
						memorySelArrowDC, 0, 0, imageSelArrowWidth, imageSelArrowHeight,
						0, 0, 0, 0,
						MULTIBLT_STRETCH, SRCCOPY);
		}
	
		SetStretchBltMode(hDC, STRETCH_DELETESCANS);

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
		DeleteObject(oldBMP);
	}

	RECT completeFillRect = {x, y, x+width, y+height};
	if (usedBorderMethod == 1)
		DrawEdge(hDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 2)
		DrawEdge(hDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 3)
		DrawEdge(hDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 4)
		DrawEdge(hDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
}

void ButtonTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int buttonType)
{
	if ( width <= 0 || height <= 0 )
		return;

	RECT r;
	r.left = 0;
	r.top = 0;
	r.right = width;
	r.bottom = height;

	int usedAlphaTransparency = 255;
	int usedBorderMethod = 0;
	bool useTrueTransparent = false;

	bool solidColorsApplied = false;

	int bevelWidth = 0;
	int bevelHeight = 0;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (buttonType == -1 && usetopSolidColors)
	{
		solidColorsApplied = true;

		usedAlphaTransparency = top.alphaTransparency;
		usedBorderMethod = top.borderMethod;
		useTrueTransparent = top.trueTransparent;
	
		bevelWidth = top.bevelSizeLeft+top.bevelSizeRight;
		bevelHeight = top.bevelSizeTop+top.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, top.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-top.bevelSizeRight;
			r.bottom = r.bottom-top.bevelSizeBottom;
			::SetBkColor(bufferDC, top.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+top.bevelSizeLeft;
			r.top = r.top+top.bevelSizeTop;

			if ( TopGradient.size() > 1 )
			{
				if ( top.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, TopGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, TopGradient[0], TopGradient[1], top.gradientType, top.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, top.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( TopGradient.size() > 1 )
			{
				if ( top.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, TopGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, TopGradient[0], TopGradient[1], top.gradientType, top.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, top.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (buttonType == -2 && usebottomSolidColors)
	{
		solidColorsApplied = true;

		usedAlphaTransparency = bottom.alphaTransparency;
		usedBorderMethod = bottom.borderMethod;
		useTrueTransparent = bottom.trueTransparent;
	
		bevelWidth = bottom.bevelSizeLeft+bottom.bevelSizeRight;
		bevelHeight = bottom.bevelSizeTop+bottom.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, bottom.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-bottom.bevelSizeRight;
			r.bottom = r.bottom-bottom.bevelSizeBottom;
			::SetBkColor(bufferDC, bottom.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+bottom.bevelSizeLeft;
			r.top = r.top+bottom.bevelSizeTop;

			if ( BottomGradient.size() > 1 )
			{
				if ( bottom.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, BottomGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, BottomGradient[0], BottomGradient[1], bottom.gradientType, bottom.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, bottom.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( BottomGradient.size() > 1 )
			{
				if ( bottom.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, BottomGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, BottomGradient[0], BottomGradient[1], bottom.gradientType, bottom.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, bottom.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (buttonType == 1 && useentrySolidColors)
	{
		solidColorsApplied = true;

		usedAlphaTransparency = entry.alphaTransparency;
		usedBorderMethod = entry.borderMethod;
		useTrueTransparent = entry.trueTransparent;
	
		bevelWidth = entry.bevelSizeLeft+entry.bevelSizeRight;
		bevelHeight = entry.bevelSizeTop+entry.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, entry.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-entry.bevelSizeRight;
			r.bottom = r.bottom-entry.bevelSizeBottom;
			::SetBkColor(bufferDC, entry.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+entry.bevelSizeLeft;
			r.top = r.top+entry.bevelSizeTop;

			if ( EntryGradient.size() > 1 )
			{
				if ( entry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, EntryGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, EntryGradient[0], EntryGradient[1], entry.gradientType, entry.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, entry.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( EntryGradient.size() > 1 )
			{
				if ( entry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, EntryGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, EntryGradient[0], EntryGradient[1], entry.gradientType, entry.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, entry.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 2 && useselentrySolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = selentry.alphaTransparency;
		usedBorderMethod = selentry.borderMethod;
		useTrueTransparent = selentry.trueTransparent;

		bevelWidth = selentry.bevelSizeLeft+selentry.bevelSizeRight;
		bevelHeight = selentry.bevelSizeTop+selentry.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, selentry.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-selentry.bevelSizeRight;
			r.bottom = r.bottom-selentry.bevelSizeBottom;
			::SetBkColor(bufferDC, selentry.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+selentry.bevelSizeLeft;
			r.top = r.top+selentry.bevelSizeTop;

			if ( SelEntryGradient.size() > 1 )
			{
				if ( selentry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, SelEntryGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, SelEntryGradient[0], SelEntryGradient[1], selentry.gradientType, selentry.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, selentry.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SelEntryGradient.size() > 1 )
			{
				if ( selentry.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, SelEntryGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, SelEntryGradient[0], SelEntryGradient[1], selentry.gradientType, selentry.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, selentry.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 3 && usefolderSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = folder.alphaTransparency;
		usedBorderMethod = folder.borderMethod;
		useTrueTransparent = folder.trueTransparent;

		bevelWidth = folder.bevelSizeLeft+folder.bevelSizeRight;
		bevelHeight = folder.bevelSizeTop+folder.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, folder.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-folder.bevelSizeRight;
			r.bottom = r.bottom-folder.bevelSizeBottom;
			::SetBkColor(bufferDC, folder.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+folder.bevelSizeLeft;
			r.top = r.top+folder.bevelSizeTop;

			if ( FolderGradient.size() > 1 )
			{
				if ( folder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, FolderGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, FolderGradient[0], FolderGradient[1], folder.gradientType, folder.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, folder.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FolderGradient.size() > 1 )
			{
				if ( folder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, FolderGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, FolderGradient[0], FolderGradient[1], folder.gradientType, folder.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, folder.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 4 && useselfolderSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = selfolder.alphaTransparency;
		usedBorderMethod = selfolder.borderMethod;
		useTrueTransparent = selfolder.trueTransparent;

		bevelWidth = selfolder.bevelSizeLeft+selfolder.bevelSizeRight;
		bevelHeight = selfolder.bevelSizeTop+selfolder.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, selfolder.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-selfolder.bevelSizeRight;
			r.bottom = r.bottom-selfolder.bevelSizeBottom;
			::SetBkColor(bufferDC, selfolder.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+selfolder.bevelSizeLeft;
			r.top = r.top+selfolder.bevelSizeTop;

			if ( SelFolderGradient.size() > 1 )
			{
				if ( selfolder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, SelFolderGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, SelFolderGradient[0], SelFolderGradient[1], selfolder.gradientType, selfolder.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, selfolder.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SelFolderGradient.size() > 1 )
			{
				if ( selfolder.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, SelFolderGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, SelFolderGradient[0], SelFolderGradient[1], selfolder.gradientType, selfolder.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, selfolder.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 5 && useinfoSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = info.alphaTransparency;
		usedBorderMethod = info.borderMethod;
		useTrueTransparent = info.trueTransparent;

		bevelWidth = info.bevelSizeLeft+info.bevelSizeRight;
		bevelHeight = info.bevelSizeTop+info.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, info.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-info.bevelSizeRight;
			r.bottom = r.bottom-info.bevelSizeBottom;
			::SetBkColor(bufferDC, info.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+info.bevelSizeLeft;
			r.top = r.top+info.bevelSizeTop;

			if ( InfoGradient.size() > 1 )
			{
				if ( info.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, InfoGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, InfoGradient[0], InfoGradient[1], info.gradientType, info.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, info.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( InfoGradient.size() > 1 )
			{
				if ( info.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, InfoGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, InfoGradient[0], InfoGradient[1], info.gradientType, info.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, info.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 10 && useseparatorSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = separator.alphaTransparency;
		usedBorderMethod = separator.borderMethod;
		useTrueTransparent = separator.trueTransparent;

		bevelWidth = separator.bevelSizeLeft+separator.bevelSizeRight;
		bevelHeight = separator.bevelSizeTop+separator.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, separator.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-separator.bevelSizeRight;
			r.bottom = r.bottom-separator.bevelSizeBottom;
			::SetBkColor(bufferDC, separator.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+separator.bevelSizeLeft;
			r.top = r.top+separator.bevelSizeTop;

			if ( SeparatorGradient.size() > 1 )
			{
				if ( separator.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, SeparatorGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, SeparatorGradient[0], SeparatorGradient[1], separator.gradientType, separator.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, separator.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SeparatorGradient.size() > 1 )
			{
				if ( separator.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, SeparatorGradient, r, buttonType);
				else
					DrawGradient(bufferDC, r, SeparatorGradient[0], SeparatorGradient[1], separator.gradientType, separator.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, separator.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if ( buttonType == -1 && memoryTopBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if ( buttonType == -2 && memoryBottomBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if ( buttonType == 31 && memoryArrowBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		if ( buttonType == 41 && memorySelArrowBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if ( buttonType == 10 && memorySeparatorBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if ( buttonType == 5 && memoryInfoBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		if ( buttonType == 4 && memorySelFolderBitmap == 0 )
			buttonType = (activeFolderUseFolderDefault ? 3 : 2);		
		if ( buttonType == 3 && memoryFolderBitmap == 0 )
			buttonType = 1;	
		if ( buttonType == 2 && memorySelEntryBitmap == 0 )
			buttonType = 1;
		
		if (buttonType == 1 && memoryEntryBitmap == 0)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if ( buttonType == 31 || buttonType == 41 )
		{
			r.left = 0;
			r.top = 0;
			r.right = width;
			r.bottom = height;

			::SetBkColor(bufferDC, RGB(255,0,255));
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
		}
	
		if (buttonType == -1)
		{
			usedAlphaTransparency = top.alphaTransparency;
			usedBorderMethod = top.borderMethod;
			useTrueTransparent = top.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryTopDC, 0, 0, imageTopWidth, imageTopHeight,
					top.leftEdge, top.topEdge, top.rightEdge, top.bottomEdge,
					top.mode, SRCCOPY);
		}
		else if (buttonType == -2)
		{
			usedAlphaTransparency = bottom.alphaTransparency;
			usedBorderMethod = bottom.borderMethod;
			useTrueTransparent = bottom.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryBottomDC, 0, 0, imageBottomWidth, imageBottomHeight,
					bottom.leftEdge, bottom.topEdge, bottom.rightEdge, bottom.bottomEdge,
					bottom.mode, SRCCOPY);
		}
		else if (buttonType == 1)
		{
			usedAlphaTransparency = entry.alphaTransparency;
			usedBorderMethod = entry.borderMethod;
			useTrueTransparent = entry.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryEntryDC, 0, 0, imageEntryWidth, imageEntryHeight,
					entry.leftEdge, entry.topEdge, entry.rightEdge, entry.bottomEdge,
					entry.mode, SRCCOPY);
		}
		else if (buttonType == 2)
		{
			usedAlphaTransparency = selentry.alphaTransparency;
			usedBorderMethod = selentry.borderMethod;
			useTrueTransparent = selentry.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memorySelEntryDC, 0, 0, imageSelEntryWidth, imageSelEntryHeight,
					selentry.leftEdge, selentry.topEdge, selentry.rightEdge, selentry.bottomEdge,
					selentry.mode, SRCCOPY);
		}
		else if (buttonType == 3)
		{
			usedAlphaTransparency = folder.alphaTransparency;
			usedBorderMethod = folder.borderMethod;
			useTrueTransparent = folder.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFolderDC, 0, 0, imageFolderWidth, imageFolderHeight,
					folder.leftEdge, folder.topEdge, folder.rightEdge, folder.bottomEdge,
					folder.mode, SRCCOPY);
		}
		else if (buttonType == 4)
		{
			usedAlphaTransparency = selfolder.alphaTransparency;
			usedBorderMethod = selfolder.borderMethod;
			useTrueTransparent = selfolder.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memorySelFolderDC, 0, 0, imageSelFolderWidth, imageSelFolderHeight,
					selfolder.leftEdge, selfolder.topEdge, selfolder.rightEdge, selfolder.bottomEdge,
					selfolder.mode, SRCCOPY);	
		}
		else if (buttonType == 5)
		{
			usedAlphaTransparency = info.alphaTransparency;
			usedBorderMethod = info.borderMethod;
			useTrueTransparent = info.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryInfoDC, 0, 0, imageInfoWidth, imageInfoHeight,
					info.leftEdge, info.topEdge, info.rightEdge, info.bottomEdge,
					info.mode, SRCCOPY);
		}
		else if (buttonType == 10)
		{
			usedAlphaTransparency = separator.alphaTransparency;
			usedBorderMethod = separator.borderMethod;
			useTrueTransparent = separator.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
						memorySeparatorDC, 0, 0, imageSeparatorWidth, imageSeparatorHeight,
						separator.leftEdge, separator.topEdge, separator.rightEdge, separator.bottomEdge,
						separator.mode, SRCCOPY);
		}
		else if ( buttonType == 31 )
		{
			MultiBlt(bufferDC, (arrowOnLeft ? arrowSpacing : width-imageArrowWidth-arrowSpacing), (height-imageArrowHeight)/2, imageArrowWidth, imageArrowHeight,
						memoryArrowDC, 0, 0, imageArrowWidth, imageArrowHeight,
						0, 0, 0, 0,
						MULTIBLT_STRETCH, SRCCOPY);
		}
		else if ( buttonType == 41 )
		{
			MultiBlt(bufferDC, (selarrowOnLeft ? selarrowSpacing : width-imageSelArrowWidth-selarrowSpacing), (height-imageSelArrowHeight)/2, imageSelArrowWidth, imageSelArrowHeight,
						memorySelArrowDC, 0, 0, imageSelArrowWidth, imageSelArrowHeight,
						0, 0, 0, 0,
						MULTIBLT_STRETCH, SRCCOPY);
		}
	}

	RECT completeFillRect = {0, 0, width, height};

	if (usedBorderMethod == 1)
		DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 2)
		DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 3)
		DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 4)
		DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);

	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int j = 0; j < iSize; ++j)
	{
		//Remove Magic Pink
		if ( (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255) )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;
		}
		else if ( !useAlphaMap || pPixel[3] == 0 || solidColorsApplied )
		{
			pPixel[3] = 255;
		}

		pPixel+= 4;
	}
	
	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, usedAlphaTransparency, (useTrueTransparent ? 0 : AC_SRC_ALPHA) };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void ButtonTexture::paintEffects(HDC destHDC, int width, int height, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( button_luminanceIntensity != 0 || button_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( button_hueIntensity != 0 )
	{
		r = GetRValue(button_hueColor);
		g = GetGValue(button_hueColor);
		b = GetBValue(button_hueColor);

		if ( r == b && b == g )
		{
			button_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(r, g, b, hueh, hues, huel);
			needHSLConversion = true;
		}
	}

	if ( button_mixIntensity != 0 )
	{
		button_mixIntensity = (int) min(255, button_mixIntensity*2.55);
		needMixColor = true;
	}

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
		
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				if ( needHSLConversion )
				{
					RGB2HSL(r, g, b, h, s, l);

					if(button_hueIntensity != 0)
						h = (BYTE) max(0, min(240, (h*(100-button_hueIntensity)+hueh*button_hueIntensity)/100));
		
					if(button_luminanceIntensity != 0)
						l = (BYTE) max(0, min(240, ((l*100)+(120*button_luminanceIntensity))/100));

					if(button_saturationIntensity != 100)
					{
						if ( button_saturationIntensity > 100 )
							s = (BYTE) max(0, min(240, (240*(button_saturationIntensity-100))/100));				
						else
							s = (BYTE) max(0, min(240, (s*button_saturationIntensity)/100));
					}
				
					HSL2RGB(r, g, b, h, s, l);
				}

				if ( needMixColor )
				{
					// HueMix effect
					r = (BYTE)((r * (255 - button_mixIntensity) + GetRValue(button_mixColor) * button_mixIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - button_mixIntensity) + GetGValue(button_mixColor) * button_mixIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - button_mixIntensity) + GetBValue(button_mixColor) * button_mixIntensity + 255) >> 8);
				}
			}

			SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void ButtonTexture::solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, int buttonType)
{
	bool vertical = false;
	int repeated = 0;

	if ( buttonType == -1 )
	{
		vertical = top.verticalGradient;
		repeated = top.repeatedGradient;
	}
	else if ( buttonType == -2 )
	{
		vertical = bottom.verticalGradient;
		repeated = bottom.repeatedGradient;
	}
	else if ( buttonType == 1 )
	{
		vertical = entry.verticalGradient;
		repeated = entry.repeatedGradient;
	}
	else if( buttonType == 2 )
	{
		vertical = selentry.verticalGradient;
		repeated = selentry.repeatedGradient;
	}
	else if( buttonType == 3 )
	{
		vertical = folder.verticalGradient;
		repeated = folder.repeatedGradient;
	}
	else if( buttonType == 4 )
	{
		vertical = selfolder.verticalGradient;
		repeated = selfolder.repeatedGradient;
	}
	else if( buttonType == 5 )
	{
		vertical = info.verticalGradient;
		repeated = info.repeatedGradient;
	}
	else if( buttonType == 10 )
	{
		vertical = separator.verticalGradient;
		repeated = separator.repeatedGradient;
	}
	else
	{
		return;
	}

	if ( repeated == 0 )
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = workGradient.size();

		if (vertical)
		{
			gradSteps = max(0, (nHeight-colorCount)/(colorCount-1));
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = max(0, (nWidth-colorCount)/(colorCount-1));
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
		{
			if (cit == workGradient.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, one);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos+i, r.right, min(lastGradientPos+i+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos+i, r.top, min(lastGradientPos+i+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (vertical)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		int nSize = 0;
		
		if (vertical)
			nSize = nHeight;
		else
			nSize = nWidth; 

		while ( counter < nSize )
		{
			for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
			{
				color = *cit;

				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+repeated, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+repeated, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, color);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				if ( counter == nSize )
					break;
			}
		}
	}
}

void ButtonTexture::PrepareAlphaChannel(HBITMAP &bmp, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( button_luminanceIntensity != 0 || button_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( button_hueIntensity != 0 )
	{
		r = GetRValue(button_hueColor);
		g = GetGValue(button_hueColor);
		b = GetBValue(button_hueColor);

		if ( r == b && b == g )
		{
			button_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
			needHSLConversion = true;
		}
	}

	if ( button_mixIntensity != 0 )
	{
		button_mixIntensity = (int) min(255, button_mixIntensity*2.55);
		needMixColor = true;
	}

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int i = 0; i < iSize; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		if ( needHSLConversion )
		{
			//AlphaMap Colorizing
			RGB2HSL(r, g, b, h, s, l);

			if(button_hueIntensity != 0)
				h = (BYTE) max(0, min(240, (h*(100-button_hueIntensity)+hueh*button_hueIntensity)/100));
		
			if(button_luminanceIntensity != 0)
				l = (BYTE) max(0, min(240, ((l*100)+(120*button_luminanceIntensity))/100));

			if(button_saturationIntensity != 100)
			{
				if ( button_saturationIntensity > 100 )
					s = (BYTE) max(0, min(240, (240*(button_saturationIntensity-100))/100));				
				else
					s = (BYTE) max(0, min(240, (s*button_saturationIntensity)/100));
			}

			HSL2RGB(r, g, b, h, s, l);
		}
		
		if ( needMixColor )
		{
			// HueMix effect
			r = (BYTE)((r * (255 - button_mixIntensity) + GetBValue(button_mixColor) * button_mixIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - button_mixIntensity) + GetGValue(button_mixColor) * button_mixIntensity + 255) >> 8);
			b = (BYTE)((b * (255 - button_mixIntensity) + GetRValue(button_mixColor) * button_mixIntensity + 255) >> 8);
		}

		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}

void ButtonTexture::DrawGradient( HDC hDC, RECT& rect, COLORREF startColor, COLORREF endColor, int gradientType, int transformationType)
{
	int i, j, y, x1, y1;
	double r, d, n1;

	// Calculate image size
	int width = rect.right - rect.left;
	int height = rect.bottom - rect.top;
	int size = height*width*4;
	double radius = sqrt( pow(height/2,2) + pow(width/2,2) );
	double k = double(rect.bottom-rect.top)/double(rect.right-rect.left);
	if ( gradientType == GRADIENT_BDIAGONAL )
		k = double(rect.bottom-rect.top)/double(rect.left-rect.right);
	if ( ( gradientType == GRADIENT_FDIAGONAL ) || ( gradientType == GRADIENT_BDIAGONAL ) )
		gradientType = GRADIENT_DIAGONAL;
	double n = 0.0;
	double k1 = -1/k;

	// Create data buffer
	m_Data = new unsigned char[size];
	
	// Create gradient
	for ( i=0; i<height; i++ )
	{
		for ( j=0; j<width; j++ )
		{
			switch (gradientType)
			{
				// Horizontal gradient
				case GRADIENT_HORIZONTAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)j/(double)width)*GetBValue(endColor) + (1-(double)j/(double)width)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)j/(double)width)*GetGValue(endColor) + (1-(double)j/(double)width)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)j/(double)width)*GetRValue(endColor) + (1-(double)j/(double)width)*GetRValue(startColor));
				}
				break;

				// Vertical gradient
				case GRADIENT_VERTICAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)i/(double)height)*GetBValue(endColor) + (1-(double)i/(double)height)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)i/(double)height)*GetGValue(endColor) + (1-(double)i/(double)height)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)i/(double)height)*GetRValue(endColor) + (1-(double)i/(double)height)*GetRValue(startColor));
				}
				break;

				// Radial gradient
				case GRADIENT_RADIAL:
				{
					r = sqrt( pow(double(i-height/2)/radius,2) + pow(double(j-width/2)/radius,2) );
					if ( r <= 1.0 )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)(r*GetBValue(endColor) + (1-r)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)(r*GetGValue(endColor) + (1-r)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)(r*GetRValue(endColor) + (1-r)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = GetBValue(endColor);
						m_Data[i*width*4 + j*4+1] = GetGValue(endColor);
						m_Data[i*width*4 + j*4+2] = GetRValue(endColor);
					}
				}
				break;

				// Diagonal gradient
				case GRADIENT_DIAGONAL:
				{
					y = int(k*(j-width/2) + n);
					n1 = int((i-height/2) - k1*(j-width/2));
					x1 = int((n1-n)/(k-k1));
					y1 = int(k1*x1 + n1);
					d = sqrt( pow((j-width/2)-x1,2) + pow((i-height/2)-y1,2) ) / (2*radius);
					if ( (i-height/2) > y )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(endColor) + (0.5+d)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(endColor) + (0.5+d)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(endColor) + (0.5+d)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(startColor) + (0.5+d)*GetBValue(endColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(startColor) + (0.5+d)*GetGValue(endColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(startColor) + (0.5+d)*GetRValue(endColor));
					}
				}
				break;
			}
		}
	}

	if ( transformationType != TRANSFORMATION_NONE )
		ApplyTransformation( width, height, transformationType );

	// Create temporary bitmap
	HBITMAP hBitmap = (HBITMAP)CreateBitmap( width, height, 1, 32, m_Data );

	// Create temporary device context
	HDC hMemDC = CreateCompatibleDC(hDC);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject( hMemDC, hBitmap );

	// Blit gradient bitmap
	BitBlt( hDC, rect.left, rect.top, width, height, hMemDC, 0, 0, SRCCOPY );

	// Free memory
	SelectObject( hMemDC, hOldBitmap );
	DeleteDC(hMemDC);
	DeleteObject(hBitmap);
	delete m_Data;
	m_Data = NULL;
}

void ButtonTexture::ApplyTransformation(int width, int height, int transformationType)
{
	int X, Y;
	double r, fi;
	double R = sqrt( pow(width/4,2) + pow(height/4,2) );
	double FI = atan( double(height)/double(width) );

	int size = height*width*4;
	unsigned char* newBuffer = new unsigned char[size];
	memcpy( newBuffer, m_Data, size );

	for ( int i=0; i<height; i++ )
	{
		for ( int j=0; j<width; j++ )
		{
			switch (transformationType)
			{
				case TRANSFORMATION_CHARICATURE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(sqrt(r*R*2)*cos(fi));
					Y = int(fabs(sqrt(r*R*2)*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_FISHEYE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int((pow(r,2)/R)*cos(fi));
					Y = int(fabs((pow(r,2)/R)*sin(fi))) % height;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SWIRLED:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi+r/8)));
					Y = int(fabs(r*sin(fi+r/8)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_CYLINDER:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = abs((j-width/2) + int(fi+r/10)%32 - 8);
					Y = i;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SHIFT:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi)));
					Y = int(fabs(r*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;
			}
		}
	}

	memcpy( m_Data, newBuffer, size );
	delete newBuffer;
}