/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Utilities.h"

extern UINT (WINAPI *pPrivateExtractIcons)(LPCTSTR,INT,INT,INT,HICON*,UINT*,UINT,UINT);

#include <shlobj.h>
#include <shlwapi.h>

#define INVALID_ICON(x) ((NULL == (x)) || (1 == (int)(x)) || (-1 == (int)(x)))

void PrefixedLSSetVariable(string prefix, string var, string value)
{
	if ( !var.empty() )
		prefix.append(var);

	LSSetVariable(prefix.c_str(), value.c_str());
}

vector<string> split(string str, string sep)
{
	vector<string> substrs;
	int start = 0;
	int end = str.find(sep, start);
	
	while(end != string::npos)
	{
		substrs.push_back(str.substr(start, end - start - sep.length() + 1));
		start = end + sep.length();
		end = str.find(sep, start);
	}
	
	substrs.push_back(str.substr(start));
	return substrs;
}

string QueryPixelDepth()
{
	CString Response = "";
	DEVMODE DevMode;
	DevMode.dmSize=sizeof(DEVMODE);
	
	if (EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&DevMode))
	{
		Response.Format("%ld", DevMode.dmBitsPerPel);
	}
	else
	{
		Response = "8";
	}
	
	return (string)Response;
}

bool IsOS(const int &os)
{	
	static int osversion = 0;

	static bool bInitialized = false;

	if (!bInitialized)
	{
		OSVERSIONINFO verInfo;
		verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx(&verInfo);
		
		switch(verInfo.dwPlatformId)
		{
			case VER_PLATFORM_WIN32_WINDOWS:

				osversion |= OS_9X;
			
				if(verInfo.dwMinorVersion >= 90)
				{
					osversion |= OS_ME;
				}
				else if(verInfo.dwMinorVersion >= 10)
				{
					osversion |= OS_98;
				}
				else
				{
					osversion |= OS_95;
				}
				break;
			
			case VER_PLATFORM_WIN32_NT:
			
				osversion |= OS_NT;
			
				if(verInfo.dwMajorVersion >= 5)
				{
					osversion |= OS_2KXP;

					if (verInfo.dwMinorVersion >= 1)
					{
						osversion |= OS_XP;
					}
					else
					{
						osversion |= OS_2K;
					}
				}
				else
				{
					osversion |= OS_NT4;
				}
				break;
			default:;
		}
		bInitialized = true;
	}

	return (osversion & os) ? true : false;
}

// retrieve handle of the desktop window
HWND GetLitestepDesktop()
{
	HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);

	if(hWnd != 0)
		return hWnd;

	return GetDesktopWindow();
}

// is the given window a toplevel application window?
bool IsAppWindow(const HWND &hWnd)
{
	if(!IsWindowVisible(hWnd))
		return false;

	if(GetWindowLong(hWnd, GWL_USERDATA) == MAGIC_DWORD)
		return false;

	long exStyle = GetWindowLong(hWnd, GWL_EXSTYLE);

	if(exStyle & WS_EX_APPWINDOW)
		return true;
	
	if(GetWindow(hWnd, GW_OWNER) != 0)
		return false;

	if(exStyle & WS_EX_TOOLWINDOW)
		return false;

	return true;
}

static int ParseIconLocation(LPSTR szIconFile)
{
	LPSTR pszComma = strrchr(szIconFile, ',');

	if (!pszComma)
		return -1;

	*pszComma = 0;
	return atoi(pszComma + 1);
}

string TrimAndStripQuote(string input, bool removeslash)
{
	string result = "";
	CString worker = input.c_str();

	worker.TrimLeft();
	worker.TrimRight();
	while (worker.Left(1) == "\"")
		worker = worker.Right(worker.GetLength()-1);
	while (worker.Right(1) == "\"")
		worker = worker.Left(worker.GetLength()-1);

	if (removeslash && worker.Right(1) == "\\")
		worker = worker.Left(worker.GetLength()-1);

	result = worker;

	return result;
}

HICON GetIconFromPathEx(LPCSTR pszPath, bool bBigIcon)
{
	char szIconFile[MAX_LINE_LENGTH];
	int nIconIndex = -1;
	LPCSTR pszExt;

    if (strlen(pszPath) > 0 && pszPath[0])
    {
        if ( _stricmp(pszPath, ".none") == NULL )
		{
			return NULL;
		}
		else if ( _stricmp(pszPath, "*") == NULL )
		{
			SHFILEINFO shfi;
        
			SHGetFileInfo("*", FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof(SHFILEINFO),
							SHGFI_USEFILEATTRIBUTES | SHGFI_ICON | (!bBigIcon ? SHGFI_SMALLICON : SHGFI_LARGEICON) );

			return shfi.hIcon;
		}
        
        strncpy(szIconFile, pszPath, MAX_LINE_LENGTH);
        nIconIndex = ParseIconLocation(szIconFile);
        
        pszExt = strrchr(szIconFile, '.');
            
        if (pszExt && _stricmp(pszExt, ".ico") == 0)
        {
            return (HICON) LoadImage(NULL, szIconFile, IMAGE_ICON,
                (bBigIcon ? 32 : 16), (bBigIcon ? 32 : 16), LR_LOADFROMFILE);
        }
		else if (pszExt && _stricmp(pszExt, ".lnk") == 0)
		{
			HICON hIcon = NULL;
			// Special case for link files
			// If the icon is set for the link, retrieve it
			SHFILEINFO shfi;
			SHGetFileInfo(szIconFile, 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICONLOCATION);
			// Extract the icon
			if ( pPrivateExtractIcons != NULL )
				pPrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, (bBigIcon ? 32 : 16), (bBigIcon ? 32 : 16), &hIcon, NULL, 1, NULL);

			// If the icon was not explicitly set in the linkfile, we'll use the icon from the target file
			if(hIcon == NULL)
			{
				HRESULT hRes;				
				char szInfo[1024];
				IShellLink* psl;

				// Create the ShellLink object
				hRes = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*)&psl);
				if(SUCCEEDED(hRes))
				{
					IPersistFile* ppf;
					// Bind the ShellLink object to the Persistent File
					hRes = psl->QueryInterface(IID_IPersistFile, (LPVOID *) &ppf);
					if(SUCCEEDED(hRes))
					{
						WORD wsz[MAX_LINE_LENGTH];

						MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, szIconFile, -1, wsz, MAX_LINE_LENGTH);
						hRes = ppf->Load(wsz, 0);
						
						if(SUCCEEDED(hRes))
						{
							// Retrieve the target of the link file
							psl->GetPath(szInfo, 1024, NULL, SLGP_UNCPRIORITY);
						}
					}
				}
				psl->Release();

				// Extract the icon from the link target
				hIcon = GetIconFromPathEx(szInfo, bBigIcon);

				if (hIcon == NULL)
				{
					SHFILEINFO shfi;
					LPCSTR pszFile = szIconFile;
					UINT uFlags = NULL;
                
					if (strchr(szIconFile, '*'))
					{
						PathStripPath(szIconFile);
						uFlags |= SHGFI_USEFILEATTRIBUTES;
					}
					else if (!PathFileExists(szIconFile))
						return NULL;
                
					SHGetFileInfo(pszFile, 0, &shfi, sizeof(SHFILEINFO),
						SHGFI_ICON | (!bBigIcon ? SHGFI_SMALLICON : SHGFI_LARGEICON) | uFlags);
            
					return shfi.hIcon;
				}
			}

			return hIcon;
		}
        else if (nIconIndex >= 0)
        {
            HICON hIcon = NULL;
                
            ExtractIconEx(szIconFile, nIconIndex,
                (!bBigIcon ? NULL : &hIcon),
                (!bBigIcon ? &hIcon : NULL), 1);
                
            return hIcon;
        }
        else
        {
            SHFILEINFO shfi;
            LPCSTR pszFile = szIconFile;
            UINT uFlags = NULL;
                
            if (strchr(szIconFile, '*'))
            {
                PathStripPath(szIconFile);
                uFlags |= SHGFI_USEFILEATTRIBUTES;
            }
            else if (!PathFileExists(szIconFile))
			{
				return NULL;
			}
                
            SHGetFileInfo(pszFile, 0, &shfi, sizeof(SHFILEINFO),
                SHGFI_ICON | (!bBigIcon ? SHGFI_SMALLICON : SHGFI_LARGEICON) | uFlags);
            
			return shfi.hIcon;
        }
    }
    else
	{
		return NULL;
	}
}

HICON GetTaskIcon(HWND hTask, bool bBigIcon)
{
	int nTimeOut = 1000;

	HICON hIcon = NULL;

	if(bBigIcon)
	{
		if(SendMessageTimeout(	 hTask
								,WM_GETICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon) )
			return hIcon;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICON);
		if(!INVALID_ICON(hIcon))
			return hIcon;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon))
			return hIcon;

		if(SendMessageTimeout(	 hTask
								,WM_GETICON
								,ICON_SMALL
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon))
			return hIcon;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICONSM);
		if(!INVALID_ICON(hIcon))
			return hIcon;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_SMALL
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon))
			return hIcon;	
	}
	else
	{
		if(SendMessageTimeout(	 hTask
								,WM_GETICON
								,ICON_SMALL
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon))
			return hIcon;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICONSM);
		if(!INVALID_ICON(hIcon))
			return hIcon;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_SMALL
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon))
			return hIcon;
		
		if(SendMessageTimeout(	 hTask
								,WM_GETICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon) )
			return hIcon;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICON);
		if(!INVALID_ICON(hIcon))
			return hIcon;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon))
			return hIcon;
	}

	//All Failed
	return LoadIcon(NULL, MAKEINTRESOURCE(IDI_APPLICATION));
}

// modify window style
void ModifyStyle(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_STYLE, (GetWindowLong(hWnd, GWL_STYLE) & ~removeStyle) | addStyle);
}

void ModifyStyleEx(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_EXSTYLE, (GetWindowLong(hWnd, GWL_EXSTYLE) & ~removeStyle) | addStyle);
}

// trim leading and trailing whitespace
string trim(const string &toTrim)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

int SplitString(const string &process, string &one, string &two, string &three, string &four, string &five, string &six, string &seven)
{
	int count = 0;
	one = "";
	two = "";
	three = "";
	four = "";
	five = "";
	six = "";
	seven = "";
	char cone[256] = {0};
	char ctwo[256] = {0};
	char cthree[256] = {0};
	char cfour[256] = {0};
	char cfive[256] = {0};
	char csix[256] = {0};
	char cseven[256] = {0};
	char *cbuffers[] = {cone, ctwo, cthree, cfour, cfive, csix, cseven};
		
	int numTokens = LCTokenize( process.c_str(), cbuffers, 7, NULL);
	if( numTokens >= 1 )
	{
		one = cone;
		count++;
		
		if ( numTokens >= 2 )
		{
			two = ctwo;
			count++;
		}
		
		if ( numTokens >= 3 )
		{
			three = cthree;
			count++;
		}
		
		if ( numTokens >= 4 )
		{
			four = cfour;
			count++;
		}

		if ( numTokens >= 5 )
		{
			five = cfive;
			count++;
		}

		if ( numTokens >= 6 )
		{
			six = csix;
			count++;
		}

		if ( numTokens >= 7 )
		{
			seven = cseven;
			count++;
		}
	}

	return count;
}

StringList GetxRCLineList(const string &uniqueKey)
{
	StringList resultList;
	LPVOID f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, uniqueKey.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], configline[MAX_LINE_LENGTH];
			char *buffers[] = {temp};
			
			if(LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
			{
				resultList.insert(resultList.end(), configline);
			}
		}
		
		LCClose(f);
	}

	return resultList;
}

void GetxRCList(char *text, StringList &theList, string cfgpath)
{
	LPVOID f = NULL;
	if ( cfgpath.empty() )
		f = LCOpen(NULL);
	else
		f = LCOpen(cfgpath.c_str());

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, text, lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], configline[MAX_LINE_LENGTH];
			char *buffers[] = {temp};
			
			if(LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
			{
				theList.insert(theList.end(), configline);
			}
		}
		
		LCClose(f);
	}
}

// retrieve a boolean value from the step.rc
bool GetxRCBool(const string &uniqueKey, const string &baseName, bool defaultVal)
{
	string key = uniqueKey + baseName;
	return GetRCBoolDef(key.c_str(), defaultVal) ? true : false;
}

// retrieve a boolean value from the step.rc
bool Getx2RCBool(const string &uniqueKey, const string &groupKey, const string &baseName, bool defaultVal)
{
	if ( groupKey.empty() )
		return GetxRCBool(uniqueKey, baseName, defaultVal);

	string nextlabelGroup = GetxRCString(groupKey, "AddToGroup", "");

	if ( !nextlabelGroup.empty() )
		return GetxRCBool(uniqueKey, baseName, GetxRCBool(groupKey, baseName, Getx2RCBool(nextlabelGroup, GetxRCString(nextlabelGroup, "AddToGroup", ""), baseName, defaultVal)));
	else
		return GetxRCBool(uniqueKey, baseName, GetxRCBool(groupKey, baseName, defaultVal));
}

// retrieve a string value from step.rc
string GetxRCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;
	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCString(key.c_str(), buffer, defaultVal.c_str(), MAX_LINE_LENGTH);
	return string(buffer);
}

// retrieve a string value from step.rc
string Getx2RCString(const string &uniqueKey, const string &groupKey, const string &baseName, const string &defaultVal)
{
	if ( groupKey.empty() )
		return GetxRCString(uniqueKey, baseName, defaultVal);

	string nextlabelGroup = GetxRCString(groupKey, "AddToGroup", "");

	if ( !nextlabelGroup.empty() )
		return GetxRCString(uniqueKey, baseName, GetxRCString(groupKey, baseName, Getx2RCString(nextlabelGroup, GetxRCString(nextlabelGroup, "AddToGroup", ""), baseName, defaultVal)));
	else
		return GetxRCString(uniqueKey, baseName, GetxRCString(groupKey, baseName, defaultVal));
}

// retrieve a line from step.rc
string GetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;
	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCLine(key.c_str(), buffer, MAX_LINE_LENGTH, defaultVal.c_str());

	return (string) buffer;
}

// retrieve a line from step.rc
string Getx2RCLine(const string &uniqueKey, const string &groupKey, const string &baseName, const string &defaultVal)
{
	if ( groupKey.empty() )
		return GetxRCLine(uniqueKey, baseName, defaultVal);

	string nextlabelGroup = GetxRCString(groupKey, "AddToGroup", "");

	if ( !nextlabelGroup.empty() )
		return GetxRCLine(uniqueKey, baseName, GetxRCLine(groupKey, baseName, Getx2RCLine(nextlabelGroup, GetxRCString(nextlabelGroup, "AddToGroup", ""), baseName, defaultVal)));
	else
		return GetxRCLine(uniqueKey, baseName, GetxRCLine(groupKey, baseName, defaultVal));
}

// retrieve a named value from step.rc
int GetxRCNamedValue(const string &uniqueKey, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	string name = GetxRCString(uniqueKey, baseName, "");
	const NameValuePair *current = nameValuePairs;

	if(name.length() > 0)
	{
		while(current->name)
		{
			if( _stricmp(current->name, name.c_str()) == NULL )
				return current->value;

			current++;
		}
	}

	return defaultVal;
}

int Getx2RCNamedValue(const string &uniqueKey, const string &groupKey, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	if ( groupKey.empty() )
		return GetxRCNamedValue(uniqueKey, baseName, nameValuePairs, defaultVal);

	string nextlabelGroup = GetxRCString(groupKey, "AddToGroup", "");

	if ( !nextlabelGroup.empty() )
		return GetxRCNamedValue(uniqueKey, baseName, nameValuePairs, GetxRCNamedValue(groupKey, baseName, nameValuePairs, Getx2RCNamedValue(nextlabelGroup, GetxRCString(nextlabelGroup, "AddToGroup", ""), baseName, nameValuePairs, defaultVal)));
	else
		return GetxRCNamedValue(uniqueKey, baseName, nameValuePairs, GetxRCNamedValue(groupKey, baseName, nameValuePairs, defaultVal));
}

// retrieve a color value from step.rc
int GetxRCColor(const string &uniqueKey, const string &baseName, int defaultVal)
{
	string key = uniqueKey + baseName;
	while ( strstr( key.c_str(), "#" ) != NULL )
		key.replace(key.find("#"), 1, "", 0);
	return GetRCColor(key.c_str(), defaultVal);
}

// retrieve a color value from step.rc
int Getx2RCColor(const string &uniqueKey, const string &groupKey, const string &baseName, int defaultVal)
{
	if ( groupKey.empty() )
		return GetxRCColor(uniqueKey, baseName, defaultVal);

	string nextlabelGroup = GetxRCString(groupKey, "AddToGroup", "");

	if ( !nextlabelGroup.empty() )
		return GetxRCColor(uniqueKey, baseName, GetxRCColor(groupKey, baseName, Getx2RCColor(nextlabelGroup, GetxRCString(nextlabelGroup, "AddToGroup", ""), baseName, defaultVal)));
	else
		return GetxRCColor(uniqueKey, baseName, GetxRCColor(groupKey, baseName, defaultVal));
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int GetxRCInt(const string &uniqueKey, const string &baseName, int defaultVal, int minVal, int maxVal)
{
	string key = uniqueKey + baseName;
	int answer = GetRCInt(key.c_str(), defaultVal);
	return max(min(answer, maxVal), minVal);
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int Getx2RCInt(const string &uniqueKey, const string &groupKey, const string &baseName, int defaultVal, int minVal, int maxVal)
{
	if ( groupKey.empty() )
		return GetxRCInt(uniqueKey, baseName, defaultVal, minVal, maxVal);

	string nextlabelGroup = GetxRCString(groupKey, "AddToGroup", "");

	if ( !nextlabelGroup.empty() )
		return GetxRCInt(uniqueKey, baseName, GetxRCInt(groupKey, baseName, Getx2RCInt(nextlabelGroup, GetxRCString(nextlabelGroup, "AddToGroup", ""), baseName, defaultVal, minVal, maxVal), minVal, maxVal), minVal, maxVal);
	else
		return GetxRCInt(uniqueKey, baseName, GetxRCInt(groupKey, baseName, defaultVal, minVal, maxVal), minVal, maxVal);
}

// parse a coordinate specification
int xParseCoord(const string &aString, int defaultVal, int maxSize, int minPos)
{
	string strVal = trim(aString);

	if(strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool realnegative = false;
	bool center = false;
	UINT i = 0;
	int value = 0;

	if(strVal[i] == '-')
	{
		negative = true;
		i++;
	}
	else if(strVal[i] == '~')
	{
		realnegative = true;
		i++;
	}
	else if(strVal[i] == '+')
	{
		i++;
	}

	while(i < strVal.length())
	{
		if(strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if(strVal[i] == 'c' || strVal[i] == 'C')
				center = true;

			break;
		}

		i++;
	}

	if(center)
	{
		if (realnegative)
			negative = true;

		if(negative)
			value = (maxSize / 2) - value;
		else
			value = (maxSize / 2) + value;
	}
	else if(realnegative)
	{
		value = -value;
	}
	else if(negative)
	{
		value = maxSize - value;
	}

	value = minPos + value;

	return value;
}

// parse a dimension
int xParseDimen(const string &aString, int defaultVal, int maxVal)
{
	string strVal = trim(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool percentage = false;
	bool extendedpercentage = false;
	UINT i = 0;
	int value = 0;
	int extendedvalue = 0;

	if ( strstr(strVal.c_str(), "@") != NULL )
		i= i+2;

	if (strVal[i] == '-')
	{
		negative = true;
		i++;
	}

	while (i < strVal.length())
	{
		if (strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if (strVal[i] == '%')
				percentage = true;

			if (strVal[i+1] == '+' )
			{
				extendedpercentage = true;
				i++;

				while (i < strVal.length())
				{
					if (strVal[i] >= '0' && strVal[i] <= '9')
					{
						extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
					}
					i++;
				}
			}
			else if (strVal[i+1] == '-' )
			{
				extendedpercentage = true;
				i++;

				while (i < strVal.length())
				{
					if (strVal[i] >= '0' && strVal[i] <= '9')
					{
						extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
					}
					i++;
				}
				extendedvalue = extendedvalue*(-1);
			}

			break;
		}

		i++;
	}

	if (percentage)
	{
		value = (value * maxVal) / 100;
		if (extendedpercentage)
			value = value + extendedvalue;
	}

	if (negative)
		value = maxVal - value;

	return value;
}

// tile an image horizontally and vertically
void TileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		for(int x = 0; x < cxDest; x += cxSrc)
		{
			BitBlt(hdcDest,
				xDest + x,
				yDest + y,
				min(cxDest - x, cxSrc),
				min(cyDest - y, cySrc),
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}
	}
}

// tile an image horizontally
void HorizontalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int x = 0; x < cxDest; x += cxSrc)
	{
		StretchBlt(hdcDest,
			xDest + x,
			yDest,
			min(cxDest - x, cxSrc),
			cyDest,
			hdcSrc,
			xSrc,
			ySrc,
			min(cxDest - x, cxSrc),
			cySrc,
			rasterOp);
	}
}

// tile an image vertically
void VerticalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		StretchBlt(hdcDest,
			xDest,
			yDest + y,
			cxDest,
			min(cyDest - y, cySrc),
			hdcSrc,
			xSrc,
			ySrc,
			cxSrc,
			min(cyDest - y, cySrc),
			rasterOp);
	}
}

// multi-blt!
void MultiBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, int cxLeft, int cyTop, int cxRight, int cyBottom, int mode, DWORD rasterOp)
{
	if(cyTop > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				yDest,
				cxLeft,
				cyTop,
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				yDest,
				cxRight,
				cyTop,
				hdcSrc,
				cxSrc - cxRight,
				ySrc,
				rasterOp);
		}
	}

	if(cxLeft > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(mode == MULTIBLT_STRETCH)
	{
		StretchBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILE)
	{
		TileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEHORIZONTAL)
	{
		HorizontalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEVERTICAL)
	{
		VerticalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}

	if(cxRight > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(cyBottom > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				cyDest - cyBottom,
				cxLeft,
				cyBottom,
				hdcSrc,
				xSrc,
				cySrc - cyBottom,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				cyDest - cyBottom,
				cxRight,
				cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				cySrc - cyBottom,
				rasterOp);
		}
	}
}

string IntToString(int input)
{
	char buffer[32];
	_itoa( input, buffer, 10 );
	return (string) buffer;
}

void RGB2HSL(BYTE& R, BYTE& G, BYTE& B, BYTE& H, BYTE& S, BYTE& L)
{
	R = min(max(0,R),255);
	G = min(max(0,G),255);
	B = min(max(0,B),255);
	BYTE minval = min(min(R,G),B);
	BYTE maxval = max(max(R,G),B);
	DWORD msum = maxval+minval;
	L = (BYTE)(0.5+(msum*240/510.));

	if (maxval == minval)
	{
		S = 0;
		H = 160;
	}
	else
	{
		float delta = (float)maxval-minval;

		S = (BYTE)(0.5+(240*(delta/((msum<=255) ? msum : (510-msum)))));

		float h;
        
		if (R == maxval)
			h = 40*((G-B)/delta);
		else if (G == maxval)
			h = 40*(2+(B-R)/delta);
		else if (B == maxval)
			h = 40*(4+(R-G)/delta);

		if (h > 240)
			h -= 240;

		H = (BYTE)(0.5+h);
	}
}

void HSL2RGB(BYTE& R, BYTE& G, BYTE& B, BYTE& _H, BYTE& _S, BYTE& _L)
{
	_H = min(max(0,_H),239);
	_S = min(max(0,_S),240);

	_L = min(max(0,_L),240);
	
	float H = _H*360/240.0f;
	float S = _S/240.0f;
	float L = _L/240.0f;

	if (S)
	{
		float rm2;

		if (_L<=120)
			rm2 = L+L*S;  
		else
			rm2 = L+S-L*S;
		float rm1 = 2*L-rm2;   
		
		R = ToRGB(rm1, rm2, H+120);   
		G = ToRGB(rm1, rm2, H);
		B = ToRGB(rm1, rm2, H-120);
	}
	else 
	{
		R = G = B = (BYTE)(0.5+L*255);
	}
}

BYTE ToRGB(float rm1, float rm2, float rh)
{
    if (rh > 360)
		rh -= 360;
	else if (rh < 0) 
		rh += 360;

    if (rh < 60)
		rm1 = rm1+(rm2-rm1)*rh/60;   
	else if (rh < 180)
		rm1 = rm2;
	else if (rh < 240)
		rm1 = rm1+(rm2-rm1)*(240-rh)/60;      

	return (BYTE)(0.5+rm1*255);
}

