/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Popup.h"

#include "VerInfo.h"

#include "PopupTexture.h"
#include "ButtonTexture.h"
#include "ButtonIcon.h"
#include "ButtonFont.h"

#include "GhostPopup.h"
#include "GhostButton.h"

PopupTexture *defaultpopuptexture = NULL;
ButtonTexture *defaultbuttontexture = NULL;
ButtonIcon *defaultbuttonicon = NULL;
ButtonFont *defaultbuttonfont = NULL;

GhostPopupList *defaultghostpopuplist = NULL;
GhostButtonList *defaultghostbuttonlist = NULL;

//DropShadow under XP
#ifndef CS_DROPSHADOW
	#define CS_DROPSHADOW 0x00020000
#endif

HINSTANCE hInstance;
HWND messageHandler;

UINT (WINAPI *pPrivateExtractIcons)(LPCTSTR,INT,INT,INT,HICON*,UINT*,UINT,UINT) = NULL;

//VOID (WINAPI *pSwitchToThisWindow)(HWND,BOOL) = NULL;

PopupList popupList;
PopupList visiblepopupList;

int lsMessages[] = {
	LM_GETREVID,
	LM_REFRESH,
	0
};

int screenX;
int	screenY;
int screenWidth;
int	screenHeight;

int infoExport = 1;

bool internaltoggle = false;

int bitsperpixel = atoi( QueryPixelDepth().c_str() );

string dragndropSource = "-1";
string dragndropDestination = "";

void StartUp();
void PreParseStepRC(StringList &theList);
string PrepareAnchor(POINT &pt, const char *pszArguments);
void PreCalcPosition(int &x, int &y, int width, int height, LPCSTR anchor);

void BangRescanPopup(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangPopup(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangDynamicPopup(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangLoadPopupCFG(HWND hwndCaller, const char *pszName, const char *pszArguments);

void BangTasks(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangControlPanel(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangMyComputer(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangNetwork(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangPrinters(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangRecentDocuments(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangRecycleBin(HWND hwndCaller, const char *pszName, const char *pszArguments);

void BangReposition(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangPin(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangUnPin(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangTogglePin(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangToggleShade(HWND hwndCaller, const char *pszName, const char *pszArguments);
void BangToggle(HWND hwndCaller, const char *pszName, const char *pszArguments);

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case LM_GETREVID:
		{
			UINT uLength;
			StringCchPrintf((char*)lParam, 64, "%s.dll: %s", V_NAME, V_VERSION);
			
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;

			lParam = NULL;
			return 0;
		}

		case LM_REFRESH:
		{	
			LSExecute(NULL, "!popuprescan", SW_SHOWNORMAL);

			return 0;
		}

		case WM_TIMER:
		{
			KillTimer(messageHandler, NULL);
			StartUp();
		}
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

// Why are we advoiding the crt startup code? The start up code is not large
// enough to cause bloat (except possibly a long system call, and I suspect that
// would need done eventually anyway) (default _DllMainCRTStartup is in
// (MSVCDIR\crt\src\dllcrt0.c) The elimination of the call to _CRT_INIT is 
// also causing warnings with MSVC7
BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);
	return TRUE;
}

int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *lsPath)
{
	WNDCLASSEX wc;

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS | CS_DBLCLKS;
	wc.lpfnWndProc = Popup::windowProcedure;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = sizeof(Popup *);
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = LoadCursor(0, IDC_ARROW);
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xPopup";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	if(IsOS(OS_XP))
	{
		wc.cbSize = sizeof(WNDCLASSEX);
		wc.style = CS_GLOBALCLASS | CS_DBLCLKS | CS_DROPSHADOW;
		wc.lpfnWndProc = Popup::windowProcedure;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = sizeof(Popup *);
		wc.hInstance = hInstance;
		wc.hbrBackground = 0;
		wc.hCursor = LoadCursor(0, IDC_ARROW);
		wc.hIcon = 0;
		wc.lpszMenuName = 0;
		wc.lpszClassName = "xPopupXP";
		wc.hIconSm = 0;

		RegisterClassEx(&wc);
	}

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = 0;
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xPopupMessageHandler";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	messageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		"xPopupMessageHandler",
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInstance,
		0);

	if (!messageHandler)
		return 1;
	
	SendMessage(GetLitestepWnd(),
		LM_REGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	::hInstance = hInstance;

	AddBangCommandEx("!Popup", BangPopup);
	AddBangCommandEx("!PopupDynamicFolder", BangDynamicPopup);
	AddBangCommandEx("!PopupLoadCFG", BangLoadPopupCFG);
	AddBangCommandEx("!PopupRescan", BangRescanPopup);

	AddBangCommandEx("!PopupTasks", BangTasks);
	AddBangCommandEx("!PopupControlPanel", BangControlPanel);
    AddBangCommandEx("!PopupMyComputer", BangMyComputer);
    AddBangCommandEx("!PopupNetwork", BangNetwork);
    AddBangCommandEx("!PopupPrinters", BangPrinters);
    AddBangCommandEx("!PopupRecentDocuments", BangRecentDocuments);
    AddBangCommandEx("!PopupRecycleBin", BangRecycleBin);

	if ( !IsOS( OS_95 ) && !IsOS( OS_NT4 ) )
	{
		screenX = GetxRCInt("xPopup", "ScreenX", GetSystemMetrics(SM_XVIRTUALSCREEN));
		screenY = GetxRCInt("xPopup", "ScreenY", GetSystemMetrics(SM_YVIRTUALSCREEN));

		screenWidth = GetxRCInt("xPopup", "ScreenWidth", GetSystemMetrics(SM_CXVIRTUALSCREEN), 0);
		screenHeight = GetxRCInt("xPopup", "ScreenHeight", GetSystemMetrics(SM_CYVIRTUALSCREEN), 0);
	}
	else
	{
		screenX = GetxRCInt("xPopup", "ScreenX", 0);
		screenY = GetxRCInt("xPopup", "ScreenY", 0);

		screenWidth = GetxRCInt("xPopup", "ScreenWidth", GetSystemMetrics(SM_CXSCREEN), 0);
		screenHeight = GetxRCInt("xPopup", "ScreenHeight", GetSystemMetrics(SM_CYSCREEN), 0);
	}

	if (!pPrivateExtractIcons)
	{
		pPrivateExtractIcons = (UINT (WINAPI *)(LPCTSTR,INT,INT,INT,HICON*,UINT*,UINT,UINT)) GetProcAddress(GetModuleHandle(_T("USER32.DLL")), "PrivateExtractIconsA");
	}

	//if ( !pSwitchToThisWindow )
	//{
	//	pSwitchToThisWindow = (VOID (WINAPI *)(HWND, BOOL)) GetProcAddress( GetModuleHandle(_T("USER32.DLL")), "SwitchToThisWindow");  
	//}

	bool tmpuseAlphaMap = false;
	if ( IsOS(OS_2KXP) && !Getx2RCBool("xpopup", "", "UseSolidColors") && !Getx2RCString("xpopup", "", "Image", "").empty() )
		tmpuseAlphaMap = Getx2RCBool("xpopup", "", "AlphaMap");

	//Create "xPopup" Default Painting Classes
	defaultpopuptexture = new PopupTexture();
	defaultpopuptexture->configure("xpopup", "", tmpuseAlphaMap);
	
	defaultbuttontexture = new ButtonTexture();
	defaultbuttontexture->configure("xpopup", "", tmpuseAlphaMap);
	
	defaultbuttonicon = new ButtonIcon();
	defaultbuttonicon->configure("xpopup", "");
	
	defaultbuttonfont = new ButtonFont();
	defaultbuttonfont->configure("xpopup", "");

	//Create Default OverlayPopup Painting
	//Initialize Basic Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";

	defaultghostpopuplist = new GhostPopupList();
	defaultghostbuttonlist = new GhostButtonList();

	StringList OverlayPaintingList = GetxRCLineList( ("*xPopupOverlayPopup") );
	
	if ( !OverlayPaintingList.empty() )
	{
		for(StringListIterator sit = OverlayPaintingList.begin(); sit != OverlayPaintingList.end(); sit++)
		{
			numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
			
			if ( numTokens >= 5 )
			{
				GhostPopup* newpopup = new GhostPopup();
			
				newpopup->orgx = one;
				newpopup->orgy = two;

				newpopup->orgwidth = three;
				newpopup->orgheight = four;
		
				if ( numTokens >= 6 )
				{
					if ( IsOS( OS_2KXP ) )
					{
						newpopup->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
				
						if ( numTokens == 7 )
						{
							int tmpalpha = atoi( seven.c_str() );
							if ( tmpalpha < 0 || tmpalpha > 255 )
								newpopup->alphatransparency = 255;
							else
								newpopup->alphatransparency = tmpalpha;
						}
					}
				}
		
				newpopup->popuptexture = new PopupTexture();
				newpopup->popuptexture->configure(five, "", newpopup->alphamap);

				if ( newpopup->popuptexture != 0  )
					defaultghostpopuplist->insert(defaultghostpopuplist->end(), newpopup);
				else
					delete newpopup;
			}
		}
	}

	OverlayPaintingList.clear();
	//Create Default OverlayButton Painting
	OverlayPaintingList = GetxRCLineList( ("*xPopupOverlayEntry") );
	
	if ( !OverlayPaintingList.empty() )
	{
		for(StringListIterator sit = OverlayPaintingList.begin(); sit != OverlayPaintingList.end(); sit++)
		{
			//Use Split Helper Strings
			numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
			
			if ( numTokens >= 5 )
			{
				GhostButton* newbutton = new GhostButton();
			
				newbutton->orgx = one;
				newbutton->orgy = two;

				newbutton->orgwidth = three;
				newbutton->orgheight = four;
			
				if ( numTokens >= 6 )
				{
					if ( IsOS( OS_2KXP ) )
					{
						newbutton->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
					}
				}

				newbutton->buttontexture = new ButtonTexture();
				newbutton->buttontexture->configure(five, "", newbutton->alphamap);

				if ( newbutton->buttontexture != 0 )
					defaultghostbuttonlist->insert(defaultghostbuttonlist->end(), newbutton);
				else
					delete newbutton;
			}
		}
	}

	//Parse Step
	SetTimer(messageHandler, NULL, 250, 0);

	return 0;
}

void StartUp()
{
	StringList pinnedList;
	pinnedList.clear();

	//Use pinnedList as empty dummy list
	PreParseStepRC(pinnedList);

	GetxRCList( "*xPopupStartPinned", pinnedList );

	char szFirst[MAX_LINE_LENGTH] = { 0 };
	char szSecond[MAX_LINE_LENGTH] = { 0 };
	char szThird[MAX_LINE_LENGTH] = { 0 };
	char szFourth[MAX_LINE_LENGTH] = { 0 };

	char *buffer[] = { szFirst, szSecond , szThird, szFourth };

	int x = 0;
	int y = 0;
	for (StringListIterator sit = pinnedList.begin(); sit != pinnedList.end(); ++sit)
	{
		LCTokenize((*sit).c_str(), buffer, 4, NULL);

		x = xParseCoord(szSecond, 0, screenWidth, screenX);
		y = xParseCoord(szThird, 0, screenHeight, screenY);
		for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
		{
			if (_stricmp((*plit)->BangToCall.c_str(), szFirst) == NULL)
			{
				//Calculate Position due to arguments
				PreCalcPosition(x, y, (*plit)->getWidth(), (*plit)->getHeight(), szFourth);

				(*plit)->reposition(3, x, y, (*plit)->getWidth(), (*plit)->getHeight());
				(*plit)->repaint(true);
				(*plit)->PinnedMode = true;

				if ( (*plit)->pinnedPopupNotOnTop )
					(*plit)->setAlwaysOnTop(false);
				else
					(*plit)->setAlwaysOnTop(true);

				(*plit)->show();
				break;
			}
		}		
	}
	pinnedList.clear();
}

void PreParseStepRC(StringList &theList)
{
	StringList settingsList;
	bool loadCFG = false;

	if ( !theList.empty() )
	{
		settingsList = theList;
		loadCFG = true;
	}
	else
	{
		GetxRCList( "*Popup", settingsList );
	}

	char szFirst[MAX_LINE_LENGTH] = { 0 };
	char szSecond[MAX_LINE_LENGTH] = { 0 };
	char szThird[MAX_LINE_LENGTH] = { 0 };
	char szFourth[MAX_LINE_LENGTH] = { 0 };

	char *buffer[] = { szFirst, szSecond , szThird, szFourth };

	//Initialize
	Popup* addsettingPopup = NULL;
	bool input = false;
	StringList sendToPopup;
	int nTokens = 0;
	string strAddBang = "";

	for (StringListIterator sit = settingsList.begin(); sit != settingsList.end(); ++sit)
	{
		nTokens = LCTokenize((*sit).c_str(), buffer, 3, szFourth);

		if(nTokens >= 2 && (_stricmp(szFirst, "!New") == NULL || _stricmp(szSecond, "!New") == NULL))
		{	
			//Search for Popup Start with !New and Create New Popup
			Popup *popup = NULL;
			if (_stricmp(szFirst, "!New") == NULL)
			{
				if ( _stricmp(szThird, "") == NULL )
					popup = new Popup("xpopup", "", defaultpopuptexture, defaultbuttontexture, defaultbuttonicon, defaultbuttonfont, defaultghostpopuplist, defaultghostbuttonlist);
				else
					popup = new Popup(szThird, "");

				popup->BangToCall = szSecond;

				//Remove if already defined
				if ( loadCFG )
				{
					for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
					{
						if (_stricmp((*plit)->BangToCall.c_str(), popup->BangToCall.c_str()) == NULL)
						{
							(*plit)->hide();
							delete *plit;
							popupList.erase(plit);
							break;
						}
					}
				}
			
				AddBangCommandEx(szSecond, BangPopup);

				//Reposition Bang
				strAddBang = szSecond;
				strAddBang.append("Reposition");
				AddBangCommandEx(strAddBang.c_str(), BangReposition);

				//Pin Bang == Show
				strAddBang = szSecond;
				strAddBang.append("Pin");
				AddBangCommandEx(strAddBang.c_str(), BangPin);

				//UnPin Bang == Hide
				strAddBang = szSecond;
				strAddBang.append("UnPin");
				AddBangCommandEx(strAddBang.c_str(), BangUnPin);

				//Toggle PinnedMode == Toggle
				strAddBang = szSecond;
				strAddBang.append("TogglePin");
				AddBangCommandEx(strAddBang.c_str(), BangTogglePin);

				//Toggle ShadeMode == Toggle
				strAddBang = szSecond;
				strAddBang.append("ToggleShade");
				AddBangCommandEx(strAddBang.c_str(), BangToggleShade);

				//Toggle Visibility == Toggle
				strAddBang = szSecond;
				strAddBang.append("Toggle");
				AddBangCommandEx(strAddBang.c_str(), BangToggle);
			}
			else if (_stricmp(szSecond, "!New") == NULL)
			{
				if ( _stricmp(szFourth, "") == NULL )
					popup = new Popup("xpopup", szFirst, defaultpopuptexture, defaultbuttontexture, defaultbuttonicon, defaultbuttonfont, defaultghostpopuplist, defaultghostbuttonlist);
				else
					popup = new Popup(szFourth, szFirst);

				popup->BangToCall = szThird;

				//Remove if already defined
				if ( loadCFG )
				{
					for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
					{
						if (_stricmp((*plit)->BangToCall.c_str(), popup->BangToCall.c_str()) == NULL)
						{
							(*plit)->hide();
							delete *plit;
							popupList.erase(plit);
							break;
						}
					}
				}

				AddBangCommandEx(szThird, BangPopup);

				//Reposition Bang
				strAddBang = szThird;
				strAddBang.append("Reposition");
				AddBangCommandEx(strAddBang.c_str(), BangReposition);

				//Pin Bang == Show
				strAddBang = szThird;
				strAddBang.append("Pin");
				AddBangCommandEx(strAddBang.c_str(), BangPin);

				//UnPin Bang == Hide
				strAddBang = szThird;
				strAddBang.append("UnPin");
				AddBangCommandEx(strAddBang.c_str(), BangUnPin);

				//Toggle PinnedMode == Toggle
				strAddBang = szThird;
				strAddBang.append("TogglePin");
				AddBangCommandEx(strAddBang.c_str(), BangTogglePin);

				//Toggle ShadeMode == Toggle
				strAddBang = szThird;
				strAddBang.append("ToggleShade");
				AddBangCommandEx(strAddBang.c_str(), BangToggleShade);

				//Toggle Visibility == Toggle
				strAddBang = szThird;
				strAddBang.append("Toggle");
				AddBangCommandEx(strAddBang.c_str(), BangToggle);
			}
		
			if (popup)
			{
				popupList.insert(popupList.end(), popup);
				
				addsettingPopup = popup;

				//Allow following settings to be tracked
				input = true;
			}
		}
		else if (nTokens >= 1 && (_stricmp(szFirst, "~New") == NULL || _stricmp(szSecond, "~New") == NULL))
		{
			//Close Popup Definition, give Settings and Load
			if (addsettingPopup)
			{
				addsettingPopup->load(hInstance);
				addsettingPopup->popupcontentList = sendToPopup;
				addsettingPopup->ParseStepRC();
			}
			addsettingPopup = NULL;
			input = false;
			sendToPopup.clear();
		}
		else if (input)
		{
			//Add to specific Popup Settings list
			if (strstr(szFirst, ".icon=") == NULL && strstr(szFirst, ".Icon=") == NULL && strstr(szFirst, ".ICON=") == NULL)
			{
				sendToPopup.insert(sendToPopup.end(), (".icon=\".noexist\" " + *sit));
			}
			else
			{
				sendToPopup.insert(sendToPopup.end(), *sit);
			}
		}
	}

	sendToPopup.clear();
}

string PrepareAnchor(POINT &pt, const char *pszArguments)
{
	char x[32] = {0};
	char y[32] = {0};
	char anchor1[64] = {0};
	char anchor2[64] = {0};
	string combinedanchor = "topleft";
	char *posbuffers[] = {x, y, anchor1, anchor2};
	
	int numTokens = LCTokenize( pszArguments, posbuffers, 4, NULL);

	if ( numTokens > 0 )
	{
		if ( _stricmp(x, "topleft") == NULL || _stricmp(x, "topcenter") == NULL || _stricmp(x, "topright") == NULL
				|| _stricmp(x, "centerleft") == NULL || _stricmp(x, "centercenter") == NULL || _stricmp(x, "centerright") == NULL
				|| _stricmp(x, "bottomleft") == NULL || _stricmp(x, "bottomcenter") == NULL || _stricmp(x, "bottomright") == NULL )
		{
			combinedanchor = x;
			return combinedanchor;
		}
		else if ( _stricmp(x, "bottom") == NULL || _stricmp(x, "top") == NULL || _stricmp(x, "center") == NULL )
		{
			combinedanchor = x;
			if ( _stricmp(y, "left") == NULL || _stricmp(y, "right") == NULL || _stricmp(y, "center") == NULL )
				combinedanchor.append(y);
			else
				combinedanchor.append("left");

			return combinedanchor;
		}
		else if ( _stricmp(x, "left") == NULL || _stricmp(x, "right") == NULL )
		{
			if ( _stricmp(y, "bottom") == NULL || _stricmp(y, "top") == NULL || _stricmp(y, "center") == NULL )
				combinedanchor = y;
			else
				combinedanchor = "top";
			
			combinedanchor.append(x);

			return combinedanchor;
		}
		else
		{
			pt.x = xParseCoord(x, pt.x, screenWidth, screenX);
			pt.y = xParseCoord(y, pt.y, screenHeight, screenY);

			if ( _stricmp(anchor1, "topleft") == NULL || _stricmp(anchor1, "topcenter") == NULL || _stricmp(anchor1, "topright") == NULL
				|| _stricmp(anchor1, "centerleft") == NULL || _stricmp(anchor1, "centercenter") == NULL || _stricmp(anchor1, "centerright") == NULL
				|| _stricmp(anchor1, "bottomleft") == NULL || _stricmp(anchor1, "bottomcenter") == NULL || _stricmp(anchor1, "bottomright") == NULL )
			{
				combinedanchor = anchor1;
				return combinedanchor;
			}
			else if ( _stricmp(anchor1, "bottom") == NULL || _stricmp(anchor1, "top") == NULL || _stricmp(anchor1, "center") == NULL )
			{
				combinedanchor = anchor1;
				if ( _stricmp(anchor2, "left") == NULL || _stricmp(anchor2, "right") == NULL || _stricmp(anchor2, "center") == NULL )
					combinedanchor.append(y);
				else
					combinedanchor.append("left");

				return combinedanchor;
			}
			else if ( _stricmp(anchor1, "left") == NULL || _stricmp(anchor1, "right") == NULL )
			{
				if ( _stricmp(anchor2, "bottom") == NULL || _stricmp(anchor2, "top") == NULL || _stricmp(anchor2, "center") == NULL )
					combinedanchor = anchor2;
				else
					combinedanchor = "top";
				
				combinedanchor.append(anchor1);

				return combinedanchor;
			}
		}
	}

	return combinedanchor;
}

void PreCalcPosition(int &x, int &y, int width, int height, LPCSTR anchor)
{
	// TopLeft is Default
	if (_stricmp(anchor, "topleft") == NULL)
	{
		if (x+width > screenWidth)
			x = screenWidth-width;

		if (y+height > screenHeight)
			y = screenHeight-height;
	}
	else if (_stricmp(anchor, "topcenter") == NULL)
	{
		if (x-width/2 < screenX)
			x = screenX;
		else if (x+width/2 > screenWidth)
			x = screenWidth-width;
		else
			x = x-width/2;

		if (y+height > screenHeight)
			y = screenHeight-height;
	}
	else if (_stricmp(anchor, "topright") == NULL)
	{
		if (x-width < screenX)
			x = screenX;
		else
			x = x-width;

		if (y+height > screenHeight)
			y = screenHeight-height;
	}
	else if (_stricmp(anchor, "centerleft") == NULL)
	{
		if (x+width > screenWidth)
			x = screenWidth-width;

		if (y-height/2 < screenY)
			y = screenY;
		else if (y+height/2 > screenHeight)
			y = screenHeight-height;
		else 
			y = y-height/2;
	}
	else if (_stricmp(anchor, "centercenter") == NULL)
	{
		if (x-width/2 < screenX)
			x = screenX;
		else if (x+width/2 > screenWidth)
			x = screenWidth-width;
		else
			x = x-width/2;

		if (y-height/2 < screenY)
			y = screenY;
		else if (y+height/2 > screenHeight)
			y = screenHeight-height;
		else 
			y = y-height/2;
	}
	else if (_stricmp(anchor, "centerright") == NULL)
	{
		if (x-width < screenX)
			x = screenX;
		else
			x = x-width;

		if (y-height/2 < screenY)
			y = screenY;
		else if (y+height/2 > screenHeight)
			y = screenHeight-height;
		else 
			y = y-height/2;
	}
	else if(_stricmp(anchor, "bottomleft") == NULL)
	{
		if (x+width > screenWidth)
			x = screenWidth-width;

		if (y-height < screenY)
			y = screenY;
		else
			y = y-height;
	}
	else if (_stricmp(anchor, "bottomcenter") == NULL)
	{
		if (x-width/2 < screenX)
			x = screenX;
		else if (x+width/2 > screenWidth)
			x = screenWidth-width;
		else
			x = x-width/2;

		if (y-height < screenY)
			y = screenY;
		else
			y = y-height;
	}
	else if(_stricmp(anchor, "bottomright") == NULL)
	{
		if (x-width < screenX)
			x = screenX;
		else
			x = x-width;

		if (y-height < screenY)
			y = screenY;
		else
			y = y-height;
	}
	// TopLeft is Default
	else
	{
		if (x+width > screenWidth)
			x = screenWidth-width;

		if (y+height > screenHeight)
			y = screenHeight-height;
	}
}

void quitModule(HINSTANCE hInstance)
{
	visiblepopupList.clear();

	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		(*plit)->hide();
		delete *plit;
		*plit = NULL;
	}

	popupList.clear();

	RemoveBangCommand("!Popup");
	RemoveBangCommand("!PopupDynamicFolder");
	RemoveBangCommand("!PopupLoadCFG");
	RemoveBangCommand("!PopupRescan");

	RemoveBangCommand("!PopupTasks");
	RemoveBangCommand("!PopupControlPanel");
    RemoveBangCommand("!PopupMyComputer");
    RemoveBangCommand("!PopupNetwork");
    RemoveBangCommand("!PopupPrinters");
    RemoveBangCommand("!PopupRecentDocuments");
    RemoveBangCommand("!PopupRecycleBin");

	if (defaultpopuptexture)
		delete defaultpopuptexture;
	if (defaultbuttontexture)
		delete defaultbuttontexture;
	if (defaultbuttonicon)
		delete defaultbuttonicon;
	if (defaultbuttonfont)
		delete defaultbuttonfont;

	for(GhostPopupListIterator gtlit = defaultghostpopuplist->begin(); gtlit != defaultghostpopuplist->end(); gtlit++)
	{
		delete *gtlit;
		*gtlit = NULL;
	}

	defaultghostpopuplist->clear();
	delete defaultghostpopuplist;

	for(GhostButtonListIterator glit = defaultghostbuttonlist->begin(); glit != defaultghostbuttonlist->end(); glit++)
	{
		delete *glit;
		*glit = NULL;
	}

	defaultghostbuttonlist->clear();
	delete defaultghostbuttonlist;

	SendMessage(GetLitestepWnd(),
		LM_UNREGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	DestroyWindow(messageHandler);

	UnregisterClass("xPopup", hInstance);
	if(IsOS(OS_XP))
		UnregisterClass("xPopupXP", hInstance);
	UnregisterClass("xPopupMessageHandler", hInstance);
}

void BangPopup(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	POINT cursorPos;
	GetCursorPos(&cursorPos);

	CString anchor = PrepareAnchor(cursorPos, pszArguments).c_str();
	anchor.MakeLower();

	bool oneshown = false;
	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		if ( !oneshown && (_stricmp( (*plit)->BangToCall.c_str(), pszName) == NULL || _stricmp( pszName, "!popup") == NULL) )
		{
			//Hide, if already visible
			if ( (*plit)->isVisible() && internaltoggle )
			{
				(*plit)->hide();
				oneshown = true;
				continue;
			}

			if (cursorPos.y+(*plit)->getHeight()/2 > screenHeight)
			{
				anchor.Replace("top", "bottom");
				if (anchor.Find("center") == 0)
				{
					anchor = anchor.Right(anchor.GetLength()-6);
					anchor.Insert(0, "bottom");
				}
			}
			else if (cursorPos.y+(*plit)->getHeight() > screenHeight && cursorPos.y+(*plit)->getHeight()/2 <= screenHeight)
			{
				anchor.Replace("top", "center");
			}
									
			if (cursorPos.x+(*plit)->getWidth()/2 > screenWidth)
			{
				anchor.Replace("left", "right");
				if (anchor.Find("center", 1) == 3 || anchor.Find("center", 1) == 6)
				{
					anchor = anchor.Left(anchor.Find("center", 1));
					anchor += "right";
				}
			}
			else if (cursorPos.x+(*plit)->getWidth() > screenWidth && cursorPos.x+(*plit)->getWidth()/2 <= screenWidth)
			{
				anchor.Replace("left", "center");
			}

			//Calculate Position due to arguments
			int tmpx = cursorPos.x;
			int tmpy = cursorPos.y;
			PreCalcPosition(tmpx, tmpy, (*plit)->getWidth(), (*plit)->getHeight(), anchor);
			cursorPos.x = tmpx;
			cursorPos.y = tmpy;

			KillTimer((*plit)->gethWnd(), 1);
			(*plit)->reposition(3, cursorPos.x, cursorPos.y, (*plit)->getWidth(), (*plit)->getHeight());
			(*plit)->repaint(true);
			if ((*plit)->PinnedMode)
			{
				(*plit)->PinnedMode = false;
				(*plit)->setAlwaysOnTop(true);
			}
	
			(*plit)->show();
			
			oneshown = true;
		}
		else if ( (*plit)->isVisible() )
		{
			(*plit)->hide();
		}
	}

	internaltoggle = false;
}

void BangDynamicPopup(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	char path[MAX_LINE_LENGTH] = {0};
	char rest[MAX_LINE_LENGTH] = {0};
	char *posbuffers[] = {path};
	
	CString anchor = "";
	
	LCTokenize( pszArguments, posbuffers, 1, rest);

	POINT cursorPos;
	GetCursorPos(&cursorPos);

	if ( _stricmp(pszName, "controlpanel") == NULL || _stricmp(pszName, "mycomputer") == NULL ||
			_stricmp(pszName, "network") == NULL || _stricmp(pszName, "printers") == NULL ||
			_stricmp(pszName, "recentdocuments") == NULL || _stricmp(pszName, "recyclebin") == NULL ||
			_stricmp(pszName, "tasks") == NULL )
	{
		anchor = PrepareAnchor(cursorPos, pszArguments).c_str();
	}
	else
	{
		anchor = PrepareAnchor(cursorPos, rest).c_str();
	}

	anchor.MakeLower();

	Popup* popup = NULL;
	int alreadyCreated = 0;
	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		if ( alreadyCreated == 0 && _stricmp( (*plit)->BangToCall.c_str(), path ) == NULL )
		{
			alreadyCreated = 1;
			popup = *plit;
		}
		else if ( alreadyCreated == 0 && _stricmp( (*plit)->BangToCall.c_str(), pszName ) == NULL )
		{
			alreadyCreated = 2;
			popup = *plit;
		}
		else if ( (*plit)->isVisible() )
		{
			(*plit)->hide();
		}
	}
	if (alreadyCreated == 0)
	{
		popup = new Popup("xpopup", "", defaultpopuptexture, defaultbuttontexture, defaultbuttonicon, defaultbuttonfont, defaultghostpopuplist, defaultghostbuttonlist);

		popupList.insert(popupList.end(), popup);
		
		if ( _stricmp(pszName, "controlpanel") == NULL || _stricmp(pszName, "mycomputer") == NULL ||
				_stricmp(pszName, "network") == NULL	|| _stricmp(pszName, "printers") == NULL ||
				_stricmp(pszName, "recentdocuments") == NULL || _stricmp(pszName, "recyclebin") == NULL ||
				_stricmp(pszName, "tasks") == NULL )
			popup->BangToCall = pszName;
		else
			popup->BangToCall = path;
			
		popup->load(hInstance);
	}
	
	if (popup != NULL)
	{
		if ( _stricmp(pszName, "controlpanel") == NULL || _stricmp(pszName, "mycomputer") == NULL ||
				_stricmp(pszName, "network") == NULL	|| _stricmp(pszName, "printers") == NULL ||
				_stricmp(pszName, "recentdocuments") == NULL || _stricmp(pszName, "recyclebin") == NULL ||
				_stricmp(pszName, "tasks") == NULL )
		{
			int clsid = 0;

			if ( _stricmp(pszName, "controlpanel") == NULL )
			{
				clsid = CSIDL_CONTROLS;
				popup->captionname = "ControlPanel";
			}
			else if ( _stricmp(pszName, "mycomputer") == NULL )
			{
				clsid = CSIDL_DRIVES;
				popup->captionname = "MyComputer";
			}
			else if ( _stricmp(pszName, "network") == NULL )
			{
				clsid = CSIDL_NETWORK;
				popup->captionname = "Network";
			}
			else if ( _stricmp(pszName, "printers") == NULL )
			{
				clsid = CSIDL_PRINTERS;
				popup->captionname = "Printers";
			}
			else if ( _stricmp(pszName, "recentdocuments") == NULL )
			{
				clsid = CSIDL_RECENT;
				popup->captionname = "RecentDocuments";
			}
			else if ( _stricmp(pszName, "recyclebin") == NULL )
			{
				clsid = CSIDL_BITBUCKET;
				popup->captionname = "RecycleBin";
			}
			else if ( _stricmp(pszName, "tasks") == NULL )
			{
				clsid = -1;
				popup->captionname = "Tasks";
			}

			if (clsid == 0)
				return;

			if ( clsid == -1 )
				popup->ScanTasks();
			else
				popup->ScanShellFolder(clsid);
		}
		else
		{
			CString easyWorker = path;
			if (easyWorker.Right(1) == '\\')
				easyWorker = easyWorker.Left(easyWorker.GetLength()-1);
			easyWorker = easyWorker.Right(easyWorker.GetLength()-easyWorker.ReverseFind('\\')-1);
			popup->captionname = (string) easyWorker;
			popup->ScanSubMenu((string) path, -2);
		}

		if (cursorPos.y+popup->getHeight()/2 > screenHeight)
		{
			anchor.Replace("top", "bottom");
			if (anchor.Find("center") == 0)
			{
				anchor = anchor.Right(anchor.GetLength()-6);
				anchor.Insert(0, "bottom");
			}
		}
		else if (cursorPos.y+popup->getHeight() > screenHeight && cursorPos.y+popup->getHeight()/2 <= screenHeight)
		{
			anchor.Replace("top", "center");
		}
								
		if (cursorPos.x+popup->getWidth()/2 > screenWidth)
		{
			anchor.Replace("left", "right");
			if (anchor.Find("center", 1) == 3 || anchor.Find("center", 1) == 6)
			{
				anchor = anchor.Left(anchor.Find("center", 1));
				anchor += "right";
			}
		}
		else if (cursorPos.x+popup->getWidth() > screenWidth && cursorPos.x+popup->getWidth()/2 <= screenWidth)
		{
			anchor.Replace("left", "center");
		}

		//Calculate Position due to arguments
		int tmpx = cursorPos.x;
		int tmpy = cursorPos.y;
		PreCalcPosition(tmpx, tmpy, popup->getWidth(), popup->getHeight(), anchor);
		cursorPos.x = tmpx;
		cursorPos.y = tmpy;

		popup->reposition(1, cursorPos.x, cursorPos.y, 0, 0);
		popup->show();
	}
}

void BangLoadPopupCFG(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	char prefix[64] = {0};
	char path[MAX_LINE_LENGTH] = {0};
	char *posbuffers[] = {prefix, path};
	
	LCTokenize( pszArguments, posbuffers, 2, NULL);

	StringList cfgList;
	cfgList.clear();
	GetxRCList( prefix, cfgList, path );

	if ( cfgList.size() >= 3 )
		PreParseStepRC(cfgList);
}

void BangTasks(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	BangDynamicPopup(hwndCaller, "Tasks", pszArguments);
}

void BangControlPanel(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	BangDynamicPopup(hwndCaller, "ControlPanel", pszArguments);
}

void BangMyComputer(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	BangDynamicPopup(hwndCaller, "MyComputer", pszArguments);
}

void BangNetwork(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	BangDynamicPopup(hwndCaller, "Network", pszArguments);
}

void BangPrinters(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	BangDynamicPopup(hwndCaller, "Printers", pszArguments);
}

void BangRecentDocuments(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	BangDynamicPopup(hwndCaller, "RecentDocuments", pszArguments);
}

void BangRecycleBin(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	BangDynamicPopup(hwndCaller, "RecycleBin", pszArguments);
}

void BangReposition(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	int x;
	int y;
	char A[16], B[16], C[16], D[16], E[32];
	char *buffers[] = {A, B, C, D, E};

	int tokens = LCTokenize(pszArguments, buffers, 5, NULL);

	if ( tokens >= 2 )
	{
		for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
		{
			if ( _stricmp( ((*plit)->BangToCall + "reposition").c_str(), pszName) == NULL )
			{	
				if ( _stricmp(A, "currentx") == NULL)
					x = (*plit)->getX();
				else
					x = xParseCoord(A, 0, screenWidth, screenX);
				if ( _stricmp(B, "currenty") == NULL)
					y = (*plit)->getY();
				else
					y = xParseCoord(B, 0, screenHeight, screenY);

				if (tokens == 2)
				{
					(*plit)->reposition(1, x, y, 0, 0);
				}
				else if (tokens == 3)
				{
					PreCalcPosition(x, y, (*plit)->getWidth(), (*plit)->getHeight(), C);

					(*plit)->reposition(1, x, y, 0, 0);
				}
				else if (tokens == 4)
				{
					(*plit)->reposition(3, x, y, atoi( C ), atoi( D ), 1);
					(*plit)->repaint(true);
				}
				else if (tokens == 5)
				{
					PreCalcPosition(x, y, atoi( C ), atoi( D ), E);
					
					(*plit)->reposition(3, x, y, atoi( C ), atoi( D ), 1);
					(*plit)->repaint(true);
				}

				break;
			}
		}
	}
}

void BangPin(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		if ( _stricmp( ((*plit)->BangToCall + "pin").c_str(), pszName) == NULL )
		{
			(*plit)->PinnedMode = true;
			if ( (*plit)->pinnedPopupNotOnTop )
				(*plit)->setAlwaysOnTop(false);
			else
				(*plit)->setAlwaysOnTop(true);
			(*plit)->show();
			break;
		}
	}
}

void BangUnPin(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		if ( _stricmp( ((*plit)->BangToCall + "unpin").c_str(), pszName) == NULL )
		{
			(*plit)->PinnedMode = false;
			(*plit)->setAlwaysOnTop(true);
			(*plit)->hide();
			break;
		}
	}
}

void BangTogglePin(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		if ( _stricmp( ((*plit)->BangToCall + "togglepin").c_str(), pszName) == NULL )
		{
			if ((*plit)->PinnedMode)
			{
				(*plit)->PinnedMode = false;
				(*plit)->setAlwaysOnTop(true);
				(*plit)->hide();
			}
			else
			{
				(*plit)->PinnedMode = true;
				if ( (*plit)->pinnedPopupNotOnTop )
					(*plit)->setAlwaysOnTop(false);
				else
					(*plit)->setAlwaysOnTop(true);
				(*plit)->show();
			}
			break;
		}
	}
}

void BangToggleShade(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		if ( _stricmp( ((*plit)->BangToCall + "toggleshade").c_str(), pszName) == NULL )
		{
			(*plit)->ToggleShade();

			break;
		}
	}
}

void BangToggle(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	CString convert = pszName;
	convert = convert.Left(convert.GetLength()-6);

	internaltoggle = true;

	BangPopup(hwndCaller, convert, pszArguments);
}

void BangRescanPopup(HWND hwndCaller, const char *pszName, const char *pszArguments)
{
	if ( _stricmp(pszArguments, "reload") == NULL )
		LSExecute(NULL, "!reload", SW_SHOWNORMAL);

	visiblepopupList.clear();

	for(PopupListIterator plit = popupList.begin(); plit != popupList.end(); plit++)
	{
		(*plit)->hide();
		delete *plit;
		*plit = NULL;
	}

	popupList.clear();

	if (defaultpopuptexture)
		delete defaultpopuptexture;
	if (defaultbuttontexture)
		delete defaultbuttontexture;
	if (defaultbuttonicon)
		delete defaultbuttonicon;
	if (defaultbuttonfont)
		delete defaultbuttonfont;

	for(GhostPopupListIterator gtlit = defaultghostpopuplist->begin(); gtlit != defaultghostpopuplist->end(); gtlit++)
	{
		delete *gtlit;
		*gtlit = NULL;
	}

	defaultghostpopuplist->clear();
	delete defaultghostpopuplist;

	for(GhostButtonListIterator glit = defaultghostbuttonlist->begin(); glit != defaultghostbuttonlist->end(); glit++)
	{
		delete *glit;
		*glit = NULL;
	}

	defaultghostbuttonlist->clear();
	delete defaultghostbuttonlist;

	bool tmpuseAlphaMap = false;
	if ( IsOS(OS_2KXP) && !Getx2RCBool("xpopup", "", "UseSolidColors") && !Getx2RCString("xpopup", "", "Image", "").empty() )
		tmpuseAlphaMap = Getx2RCBool("xpopup", "", "AlphaMap");

	//Create xPopup Default Painting Classes
	defaultpopuptexture = new PopupTexture();
	defaultpopuptexture->configure("xpopup", "", tmpuseAlphaMap);
	
	defaultbuttontexture = new ButtonTexture();
	defaultbuttontexture->configure("xpopup", "", tmpuseAlphaMap);
	
	defaultbuttonicon = new ButtonIcon();
	defaultbuttonicon->configure("xpopup", "");
	
	defaultbuttonfont = new ButtonFont();
	defaultbuttonfont->configure("xpopup", "");

	//Create Default OverlayPopup Painting
	//Initialize Basic Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";

	defaultghostpopuplist = new GhostPopupList();
	defaultghostbuttonlist = new GhostButtonList();

	StringList OverlayPaintingList = GetxRCLineList( ("*xPopupOverlayPopup") );
	
	if ( !OverlayPaintingList.empty() )
	{
		for(StringListIterator sit = OverlayPaintingList.begin(); sit != OverlayPaintingList.end(); sit++)
		{
			numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
			
			if ( numTokens >= 5 )
			{
				GhostPopup* newpopup = new GhostPopup();
			
				newpopup->orgx = one;
				newpopup->orgy = two;

				newpopup->orgwidth = three;
				newpopup->orgheight = four;
		
				if ( numTokens >= 6 )
				{
					if ( IsOS( OS_2KXP ) )
					{
						newpopup->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
				
						if ( numTokens == 7 )
						{
							int tmpalpha = atoi( seven.c_str() );
							if ( tmpalpha < 0 || tmpalpha > 255 )
								newpopup->alphatransparency = 255;
							else
								newpopup->alphatransparency = tmpalpha;
						}
					}
				}
		
				newpopup->popuptexture = new PopupTexture();
				newpopup->popuptexture->configure(five, "", newpopup->alphamap);

				if ( newpopup->popuptexture != 0  )
					defaultghostpopuplist->insert(defaultghostpopuplist->end(), newpopup);
				else
					delete newpopup;
			}
		}
	}

	OverlayPaintingList.clear();
	//Create Default OverlayButton Painting
	OverlayPaintingList = GetxRCLineList( ("*xPopupOverlayEntry") );
	
	if ( !OverlayPaintingList.empty() )
	{
		for(StringListIterator sit = OverlayPaintingList.begin(); sit != OverlayPaintingList.end(); sit++)
		{
			//Use Split Helper Strings
			numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
			
			if ( numTokens >= 5 )
			{
				GhostButton* newbutton = new GhostButton();
			
				newbutton->orgx = one;
				newbutton->orgy = two;

				newbutton->orgwidth = three;
				newbutton->orgheight = four;
			
				if ( numTokens >= 6 )
				{
					if ( IsOS( OS_2KXP ) )
					{
						newbutton->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
					}
				}

				newbutton->buttontexture = new ButtonTexture();
				newbutton->buttontexture->configure(five, "", newbutton->alphamap);

				if ( newbutton->buttontexture != 0 )
					defaultghostbuttonlist->insert(defaultghostbuttonlist->end(), newbutton);
				else
					delete newbutton;
			}
		}
	}

	StartUp();
}