/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Popup.h"

#include "ContextMenu.h"
#include "NotifyDirCheck.h"

#include "PopupTexture.h"
#include "ButtonTexture.h"
#include "ButtonIcon.h"
#include "ButtonFont.h"

#include "GhostButton.h"
#include "GhostPopup.h"

#include <shlwapi.h>
#include <algorithm>

#include <process.h>

//////////////////////////////////////////////////////////////////////
// Defines
//////////////////////////////////////////////////////////////////////

extern PopupList popupList;
extern PopupList visiblepopupList;

extern int screenX;
extern int screenY;
extern int screenWidth;
extern int screenHeight;

extern int infoExport;

extern int bitsperpixel;

extern string dragndropSource;
extern string dragndropDestination;

#define GET_X_LPARAM(lp) ((int)(short)LOWORD(lp))
#define GET_Y_LPARAM(lp) ((int)(short)HIWORD(lp))

#define WM_MOUSEWHEEL 0x020A

  #define WM_XBUTTONDOWN 0x020B
  #define WM_XBUTTONUP 0x020C
  //#define WM_XBUTTONDBLCLK 0x020D

  #define MK_XBUTTON1 0x0020
  #define MK_XBUTTON2 0x0040
  #define XBUTTON1 0x0001
  #define XBUTTON2 0x0002

//Alpha Transparency
typedef DWORD (WINAPI *PSLWA)(HWND, DWORD, BYTE, DWORD);
static PSLWA pSetLayeredWindowAttributes = NULL;

typedef BOOL (WINAPI *PULW)(HWND hwnd, HDC hdcDst, POINT *pptDst, SIZE *psize, HDC hdcSrc, POINT *pptSrc, COLORREF crKey, BLENDFUNCTION *pblend, DWORD dwFlags);
static PULW pUpdateLayeredWindow = NULL;

//extern VOID (WINAPI *pSwitchToThisWindow)(HWND,BOOL);

static bool alphaInitialized = false;

#define TIMER_MOUSETRACK	1
#define TIMER_ALPHAFADEIN	2
#define TIMER_ALPHAFADEOUT	3
#define TIMER_UPDATE		4
#define TIMER_ACTIVE		5
#define TIMER_HOVER			6
#define TIMER_CHORDING		7
#define TIMER_AUTOUNLOCK	8

#define TIMER_MULTISCROLLUP		10
#define TIMER_MULTISCROLLDOWN	11

extern PopupTexture *defaultpopuptexture;
extern ButtonTexture *defaultbuttontexture;
extern ButtonIcon *defaultbuttonicon;
extern ButtonFont *defaultbuttonfont;

extern GhostPopupList *defaultghostpopuplist;
extern GhostButtonList *defaultghostbuttonlist;

#define LM_HIDEPOPUP				9183
#define LM_BRINGTOFRONT				8891

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Popup::Popup(const string &name, string subname, PopupTexture *parentpopuptexture, ButtonTexture *parentbuttontexture, ButtonIcon *parentbuttonicon, ButtonFont *parentbuttonfont, GhostPopupList *parentghostpopuplist, GhostButtonList *parentghostbuttonlist, int oldsubFolderLevel, string groupName) :
	actualx(0),
	actualy(0),
	actualwidth(0),
	actualheight(0),

	hWnd(0),

	hInstance(0),

	resetBackground(false),
	lockKeyboard(false),

	tooltipMode(0),

	snappedCode(0),

	customPaintingClasses(false),

	isTaskPopup(false),
	hasHookedWindows(false),
	hasInfoEntry(false),

	hasDynamicContent(false),

	moveButtonDown(false),

	dropShadow(false),

	openToLeft(false),

	pParentPopup(NULL),
	pParentEntry(NULL),

	startentry(NULL),
	lastactiveentry(NULL),
	hoverentry(NULL),

	lastkeyentry(-1),

	PinnedMode(false),

	defaultDC(NULL),
	defaultBitmap(NULL),
	olddefaultBMP(NULL),

	isFading(false),
	fadeBackupDC(NULL),
	fadeBackupBitmap(NULL),
	oldfadeBackupBMP(NULL),

	alphaMap(false),
	fakealphaMap(false),

	subFolderLevel(-1),

	currentPart(0),
	maxPart(0),
	maxMenuBreak(0),

	dynamicPopup(false),
	rootScanPath(""),
	//rootFullPath(""),
	rootUniversalAction(""),

	showExtensions(false),

	//Bang which calls this Popup (only Top Level Popups have this)
	BangToCall(""),

	execSoundPlayed(false),

	UpdateAlpha(0),
	alphaFadeStep(4),
	alphaFadeDelay(10),

	isMultiScrollTimerRunning(0),

	mouseInside(false),
	mousePressed(false),
	mouseMoved(false),

	manualMove(true),
	
	visible(false),

	inChordingTimeOut(false),

	shaded(-1),

	alwaysOnTop(false),

	str_completeCheckStringBackup(""),

	tmpActionForTimer(""),
	tmpPidlForTimer(NULL)
{
	//Specifies the Cfg Prefix
	this->name = name;

	if ( groupName.empty() )
	{
		popupgroup = GetxRCString(name, "AddToGroup", "xPopup");
	}
	else
	{
		PrefixedLSSetVariable(name, "AddToGroup", groupName);
		popupgroup = groupName;
	}

	//Specifies the internal name (Caption/Title) plus "sub_" Prefix
	this->subname = subname;

		CString help = subname.c_str();
		help.Replace("sub_", "");

		CString help1 = help.Left(1);
		help1.MakeUpper();
	
	//Specifies the Caption/Title
	this->captionname = (help1 + help.Right(help.GetLength()-1));

	this->subFolderLevel = oldsubFolderLevel+1;

	oldmovetracker.x = 0;
	oldmovetracker.y = 0;

	//Only Create Painting Classes, if it's a TopLevel Popup
	//-> Nothing is sent to the "SubPopup"
	if (parentpopuptexture == NULL && parentbuttontexture == NULL && parentbuttonicon == NULL && parentbuttonfont == NULL && parentghostpopuplist == NULL && parentghostbuttonlist == NULL )
	{
		customPaintingClasses = true;

		bool tmpuseAlphaMap = false;

		if ( IsOS(OS_2KXP) && !Getx2RCBool(name, popupgroup, "UseSolidColors") && !Getx2RCString(name, popupgroup, "Image", "").empty() )
			tmpuseAlphaMap = Getx2RCBool(name, popupgroup, "AlphaMap");

		popuptexture = new PopupTexture();
		popuptexture->configure(name, popupgroup, tmpuseAlphaMap);
		
		buttontexture = new ButtonTexture();
		buttontexture->configure(name, popupgroup, tmpuseAlphaMap);
		
		buttonicon = new ButtonIcon();
		buttonicon->configure(name, popupgroup);
		
		buttonfont = new ButtonFont();
		buttonfont->configure(name, popupgroup);

		//Create Default OverlayPopup Painting
		//Initialize Basic Helper Strings
		int numTokens = 0;
		string one = "";
		string two = "";
		string three = "";
		string four = "";
		string five = "";
		string six = "";
		string seven = "";

		ghostpopuplist = new GhostPopupList();
		ghostbuttonlist = new GhostButtonList();

		StringList OverlayPaintingList = GetxRCLineList( ("*" + name + "OverlayPopup") );
		
		if ( !OverlayPaintingList.empty() )
		{
			for(StringListIterator sit = OverlayPaintingList.begin(); sit != OverlayPaintingList.end(); sit++)
			{
				numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
				
				if ( numTokens >= 5 )
				{
					GhostPopup* newpopup = new GhostPopup();
				
					newpopup->orgx = one;
					newpopup->orgy = two;

					newpopup->orgwidth = three;
					newpopup->orgheight = four;
			
					if ( numTokens >= 6 )
					{
						if ( IsOS( OS_2KXP ) )
						{
							newpopup->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
					
							if ( numTokens == 7 )
							{
								int tmpalpha = atoi( seven.c_str() );
								if ( tmpalpha < 0 || tmpalpha > 255 )
									newpopup->alphatransparency = 255;
								else
									newpopup->alphatransparency = tmpalpha;
							}
						}
					}
			
					newpopup->popuptexture = new PopupTexture();
					newpopup->popuptexture->configure(five, "", newpopup->alphamap);

					if ( newpopup->popuptexture != 0  )
						ghostpopuplist->insert(ghostpopuplist->end(), newpopup);
					else
						delete newpopup;
				}
			}
		}

		OverlayPaintingList.clear();
		//Create Default OverlayButton Painting
		OverlayPaintingList = GetxRCLineList( ("*" + name + "OverlayEntry") );
		
		if ( !OverlayPaintingList.empty() )
		{
			for(StringListIterator sit = OverlayPaintingList.begin(); sit != OverlayPaintingList.end(); sit++)
			{
				//Use Split Helper Strings
				numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
				
				if ( numTokens >= 5 )
				{
					GhostButton* newbutton = new GhostButton();
				
					newbutton->orgx = one;
					newbutton->orgy = two;

					newbutton->orgwidth = three;
					newbutton->orgheight = four;
				
					if ( numTokens >= 6 )
					{
						if ( IsOS( OS_2KXP ) )
						{	
							newbutton->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
						}
					}

					newbutton->buttontexture = new ButtonTexture();
					newbutton->buttontexture->configure(five, "", newbutton->alphamap);

					if ( newbutton->buttontexture != 0 )
						ghostbuttonlist->insert(ghostbuttonlist->end(), newbutton);
					else
						delete newbutton;
				}
			}
		}
	}
	else
	{
		popuptexture = parentpopuptexture;
		buttontexture = parentbuttontexture;
		buttonicon = parentbuttonicon;
		buttonfont = parentbuttonfont;

		ghostpopuplist = parentghostpopuplist;
		ghostbuttonlist = parentghostbuttonlist;
	}
}

Popup::~Popup()
{
	for(CheckDirectoryListIterator dcit = checkdirectorylist.begin(); dcit != checkdirectorylist.end(); dcit++)
	{
		(*dcit)->Stop();
		delete *dcit;
		*dcit = NULL;
	}

	checkdirectorylist.clear();

	if (hWnd && ToolTips)
	{
		updateHint(hWnd,"", false, true);
		DestroyWindow(ToolTips);
		ToolTips = 0;
	}

	if ( !BangToCall.empty() /*-> TopLevel Popup*/ )
	{
		// unregister LS messages
        int msgs[] = { LM_HIDEPOPUP, 0 };

		SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE,
            (WPARAM)hWnd, (LPARAM) msgs);

		RemoveBangCommand(BangToCall.c_str());
		
		string strRemoveBang = "";

		//Reposition Bang
		strRemoveBang = BangToCall;
		strRemoveBang.append("Reposition");
		RemoveBangCommand(strRemoveBang.c_str());
		
		//Pin Bang == Show
		strRemoveBang = BangToCall;
		strRemoveBang.append("Pin");
		RemoveBangCommand(strRemoveBang.c_str());

		//UnPin Bang == Hide
		strRemoveBang = BangToCall;
		strRemoveBang.append("UnPin");
		RemoveBangCommand(strRemoveBang.c_str());

		//Toggle PinnedMode == Toggle
		strRemoveBang = BangToCall;
		strRemoveBang.append("TogglePin");
		RemoveBangCommand(strRemoveBang.c_str());

		//Toggle ShadeMode == Toggle
		strRemoveBang = BangToCall;
		strRemoveBang.append("ToggleShade");
		RemoveBangCommand(strRemoveBang.c_str());

		//Toggle Visiblity == Toggle
		strRemoveBang = BangToCall;
		strRemoveBang.append("Toggle");
		RemoveBangCommand(strRemoveBang.c_str());
	}

	for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
	{
		(*plit)->hide();
		delete *plit;
		*plit = NULL;
	}

	subpopupList.clear();

	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		delete *eit;
		*eit = NULL;
	}

	entrylist.clear();

	tasklist.clear();

	// shutdown timer
	if(IsWindow(hWnd))
	{
		KillTimer(hWnd, TIMER_MOUSETRACK);
		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		KillTimer(hWnd, TIMER_UPDATE);
		KillTimer(hWnd, TIMER_ACTIVE);
		KillTimer(hWnd, TIMER_HOVER);
		KillTimer(hWnd, TIMER_CHORDING);
		KillTimer(hWnd, TIMER_AUTOUNLOCK);
		KillTimer(hWnd, TIMER_MULTISCROLLUP);
		KillTimer(hWnd, TIMER_MULTISCROLLDOWN);

		DestroyWindow(hWnd);
	}

	ResetBackground();

	if (customPaintingClasses) 
	{	
		if (popuptexture && popuptexture != defaultpopuptexture)
			delete popuptexture;
		if (buttontexture && buttontexture != defaultbuttontexture)
			delete buttontexture;
		if (buttonicon && buttonicon != defaultbuttonicon)
			delete buttonicon;
		if (buttonfont && buttonfont != defaultbuttonfont)
			delete buttonfont;

		if ( ghostpopuplist && ghostpopuplist != defaultghostpopuplist )
		{
			for(GhostPopupListIterator gtlit = ghostpopuplist->begin(); gtlit != ghostpopuplist->end(); gtlit++)
			{
				delete *gtlit;
				*gtlit = NULL;
			}

			ghostpopuplist->clear();
			delete ghostpopuplist;
		}

		if ( ghostbuttonlist && ghostbuttonlist != defaultghostbuttonlist )
		{
			for(GhostButtonListIterator glit = ghostbuttonlist->begin(); glit != ghostbuttonlist->end(); glit++)
			{
				delete *glit;
				*glit = NULL;
			}

			ghostbuttonlist->clear();
			delete ghostbuttonlist;
		}
	}
}

void Popup::load(HINSTANCE hInstance)
{
	this->hInstance = hInstance;
	
	if (IsWindow(hWnd))	// window already exists...
		return;

	if ( IsOS(OS_XP) && Getx2RCBool(name, popupgroup, "DropShadow") )
		dropShadow = true;

	hWnd = CreateWindowEx( WS_EX_TOOLWINDOW,
							( dropShadow ? "xPopupXP" : "xPopup" ),
							name.c_str(),
							WS_POPUP | WS_CLIPCHILDREN,
							actualx, actualy,		
							actualwidth, actualheight,
							GetLitestepWnd(), 
							0,
							hInstance,
							this);
		
	if (hWnd)
	{
		SetWindowLong(hWnd, GWL_USERDATA, MAGIC_DWORD);

		// register LS messages
		if ( !BangToCall.empty() )
		{
			int msgs[] = { LM_HIDEPOPUP, 0 };

			SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM)hWnd, (LPARAM) msgs);
		}

		reconfigure();
		
		setAlwaysOnTop(true);
	}
	else
	{
		delete this;
	}
}

void Popup::reconfigure()
{
	//Initialize Basic Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";	

	NameValuePair captionPositionValues[] = {
			{ ".none", 0 },
			{ ".top", 1 },
			{ ".bottom", 2 },
			{ ".custom", 3 },
			{ 0, 0 }
		};

	NameValuePair quicklaunchValues[] = {
			{ ".none", 0 },
			{ ".horizontal", 1 },
			{ ".vertical", 2 },
			{ 0, 0 }
		};

	NameValuePair tooltipValues[] = {
			{ ".none", 0 },
			{ ".all", 1 },
			{ ".defined", 2 },
			{ 0, 0 }
		};

	NameValuePair alphaFadeValues[] = {
			{ ".none", 0 },
			{ "false", 0 },
			{ ".allpopups", 1 },
			{ "true", 1 },
			{ ".mainpopups", 2 },
			{ 0, 0}
		};

	NameValuePair clickactionValues[] = {
			{ ".none", 0 },
			{ ".pin", 1 },
			{ ".unpin", 2 },
			{ ".togglepin", 3 },
			{ ".toggleshade", 4 },
			{ ".toggleontop", 5 },
			{ ".move", 6 },
			{ ".close", 7 },
			{ ".pageup", 8 },
			{ ".pagedown", 9 },
			{ 0, 0}
		};

	transparencyNeededAlways = buttontexture->getTrueTransparent();

	GetxRCList( (char*) ("*" + name + "CustomSubFolder").c_str(), customSubFolderList );

	//---------------------------
	//Get DefaultIcons
	//---------------------------
	DefaultIcon = Getx2RCString(name, popupgroup, "DefaultIcon", "");		
	DefaultFolderIcon = Getx2RCString(name, popupgroup, "FolderIcon", "");

	cacheLevel = Getx2RCInt(name, popupgroup, "CacheLevel", 0, 0, 5)-1;

	customWorkArea.left = screenX;
	customWorkArea.right = screenWidth;
	customWorkArea.top = screenY;
	customWorkArea.bottom = screenHeight;

	string workArea = Getx2RCLine(name, popupgroup, "WorkArea", "");
	if ( !workArea.empty() )
	{
		numTokens = SplitString(workArea, one, two, three, four, five, six, seven);

		if ( numTokens == 4 )
		{
			customWorkArea.left = xParseCoord(one, customWorkArea.left, screenWidth, screenX);
			customWorkArea.right = xParseCoord(two, customWorkArea.right, screenWidth, screenX);
			customWorkArea.top = xParseCoord(three, customWorkArea.top, screenHeight, screenY);
			customWorkArea.bottom = xParseCoord(four, customWorkArea.bottom, screenHeight, screenY);
		}
	}

	restrictToWorkArea = Getx2RCBool(name, popupgroup, "RestrictToWorkArea");

	showCaption = Getx2RCNamedValue(name, popupgroup, "ShowCaption", captionPositionValues, 1);
	showExtensions = Getx2RCBool(name, popupgroup, "ShowExtensions");
	if ( showExtensions )
		noshowExtensions = split(Getx2RCString(name, popupgroup, "ShowExtensions", ""), "|");

	pinnedPopupNotOnTop = Getx2RCBool(name, popupgroup, "PinnedPopupNotOnTop");
	HideAllowed = Getx2RCBool(name, popupgroup, "ManualUnPin", true);
	MoveAllowed = Getx2RCBool(name, popupgroup, "Moveable", true);
	ShadeAllowed = Getx2RCBool(name, popupgroup, "Shadeable", true);
	AutoPin = Getx2RCBool(name, popupgroup, "PinOnMove", true);

	folderOpenOnClick = Getx2RCBool(name, popupgroup, "FolderOpenOnClick");

	dragndropEnabled = Getx2RCBool(name, popupgroup, "EnableDragnDrop");

	openToLeft = Getx2RCBool(name, popupgroup, "OpenToLeft");

	noLoadingPopup = Getx2RCBool(name, popupgroup, "NoLoadingPopup");

	tooltipMode = Getx2RCNamedValue(name, popupgroup, "TooltipMode", tooltipValues, 0);

	if ( pParentPopup == NULL )
		quicklaunchMode = Getx2RCNamedValue(name, popupgroup, "QuicklaunchMode", quicklaunchValues, 0);
	else
		quicklaunchMode = pParentPopup->quicklaunchMode;
	quicklaunchItemSize = Getx2RCInt(name, popupgroup, "QuicklaunchItemSize", 32, 16, 64);
	quicklaunchWrapCount = Getx2RCInt(name, popupgroup, "QuicklaunchWrapCount", 100, 1);
	quicklaunchOpenTo = Getx2RCNamedValue(name, popupgroup, "QuicklaunchOpenTo", captionPositionValues, 1);

	openSound = Getx2RCString(name, popupgroup, "OpenSound", "");
	closeSound = Getx2RCString(name, popupgroup, "CloseSound", "");
	subSound = Getx2RCString(name, popupgroup, "SwitchSound", "");
	execSound = Getx2RCString(name, popupgroup, "ActionSound", "");

	dropCommand = Getx2RCLine(name, popupgroup, "OnDrop", "");
	openCommand = Getx2RCLine(name, popupgroup, "OnOpen", "");
	closeCommand = Getx2RCLine(name, popupgroup, "OnClose", "");
	shadeCommand = Getx2RCLine(name, popupgroup, "OnShade", "");
	unshadeCommand = Getx2RCLine(name, popupgroup, "OnUnShade", "");

	folderDoubleClickAction = Getx2RCLine(name, popupgroup, "OnFolderDoubleClick", "explorer.exe /e,");

	closeAfterAction = Getx2RCBool(name, popupgroup, "CloseAfterAction", true);

	titleLClickAction = Getx2RCNamedValue(name, popupgroup, "OnTitleLeftClick", clickactionValues, 6);
	titleMClickAction = Getx2RCNamedValue(name, popupgroup, "OnTitleMiddleClick", clickactionValues, 0);
	titleRClickAction = Getx2RCNamedValue(name, popupgroup, "OnTitleRightClick", clickactionValues, 4);

	bottomLClickAction = Getx2RCNamedValue(name, popupgroup, "OnBottomLeftClick", clickactionValues, 0);
	bottomMClickAction = Getx2RCNamedValue(name, popupgroup, "OnBottomMiddleClick", clickactionValues, 0);
	bottomRClickAction = Getx2RCNamedValue(name, popupgroup, "OnBottomRightClick", clickactionValues, 0);

	//File Actions
	if (dropCommand.empty())
		DragAcceptFiles(hWnd, false);
	else
		DragAcceptFiles(hWnd, true);

	leftBorder = Getx2RCInt(name, popupgroup, "LeftBorder", 0, 0);
	topBorder = Getx2RCInt(name, popupgroup, "TopBorder", 0, 0);
	rightBorder = Getx2RCInt(name, popupgroup, "RightBorder", 0, 0);
	bottomBorder = Getx2RCInt(name, popupgroup, "BottomBorder", 0, 0);

	overlapX = Getx2RCInt(name, popupgroup, "OverlapX", 0);
	overlapY = Getx2RCInt(name, popupgroup, "OverlapY", -topBorder);

	entryHeight = Getx2RCInt(name, popupgroup, "EntryHeight", buttontexture->getEntryHeight(), 0);
	folderHeight = Getx2RCInt(name, popupgroup, "FolderHeight", buttontexture->getFolderHeight(), 0);
	infoHeight = Getx2RCInt(name, popupgroup, "InfoHeight", entryHeight, 0);
	separatorHeight = Getx2RCInt(name, popupgroup, "SeparatorHeight", buttontexture->getSeparatorHeight(), 0);

	autoSeparator = Getx2RCInt(name, popupgroup, "AutoSeparator", 0, 0, 3);
	if ( autoSeparator == 0 )
	{
		if ( Getx2RCBool(name, popupgroup, "AutoSeparator") )
			autoSeparator = 1;
	}
	autoMenuBreak = Getx2RCBool(name, popupgroup, "AutoMenuBreak");

	maxWidth = Getx2RCInt(name, popupgroup, "MaxWidth", screenWidth, 0, (customWorkArea.right-customWorkArea.left)-100 );
	maxHeight = Getx2RCInt(name, popupgroup, "MaxHeight", screenHeight, 0, (customWorkArea.bottom-customWorkArea.top)-100 );
	minWidth = Getx2RCInt(name, popupgroup, "MinWidth", 0, 0, screenWidth);
	minHeight = Getx2RCInt(name, popupgroup, "MinHeight", max(topBorder+bottomBorder, popuptexture->getMinNeededHeight()), max(topBorder+bottomBorder, popuptexture->getMinNeededHeight()), screenHeight);
	orgminHeight = minHeight;

	maxEntryWidth = Getx2RCInt(name, popupgroup, "MaxEntryWidth", screenWidth, 0, screenWidth);
	minEntryWidth = Getx2RCInt(name, popupgroup, "MinEntryWidth", 0, 0, screenWidth);

	xSpacing = Getx2RCInt(name, popupgroup, "xSpacing", 0, 0);
	ySpacing = Getx2RCInt(name, popupgroup, "ySpacing", 0, 0);

	separatorySpacing = Getx2RCInt(name, popupgroup, "SeparatorySpacing", 0, -ySpacing);

	autoHide = Getx2RCBool(name, popupgroup, "AutoHide");

	hideDelay = Getx2RCInt(name, popupgroup, "HideDelay", 250, 100);
	trackDelay = Getx2RCInt(name, popupgroup, "TrackingDelay", 50, 10);
	openDelay = Getx2RCInt(name, popupgroup, "FolderOpenDelay", 50, 1);
	tasksUpdate = Getx2RCInt(name, popupgroup, "TasksUpdateInterval", 5000, 1000);
	multiscrollDelay = Getx2RCInt(name, popupgroup, "MultiPartScrollDelay", 2000, 250);

	if ( IsOS(OS_2KXP) )
	{
		alphaTransparency = Getx2RCInt(name, popupgroup, "AlphaTransparency", 255, 0, 255);

		if ( !Getx2RCBool(name, popupgroup, "UseSolidColors") && !Getx2RCString(name, popupgroup, "Image", "").empty() )
			alphaMap = Getx2RCBool(name, popupgroup, "AlphaMap");

		string config = "";
		//---------------------------------------------------------------
		//---------------------------------------------------------------
		config = Getx2RCLine(name, popupgroup, "CustomAlphaFade", "");

		numTokens = SplitString(config, one, two, three, four, five, six, seven);
	
		if ( config.empty() )
		{
			alphaFade = false;
		}
		else if (numTokens == 1)
		{
			alphaFadeStep = atoi( one.c_str() );
			if (alphaFadeStep < 1 || alphaFadeStep > 255)
				alphaFadeStep = 2;
			alphaFade = true;
		}
		else if (numTokens == 2)
		{
			alphaFadeStep = atoi( one.c_str() );
			if (alphaFadeStep < 1 || alphaFadeStep > 255)
				alphaFadeStep = 2;
			alphaFadeDelay = atoi( two.c_str() );
			if (alphaFadeDelay < 1 || alphaFadeDelay > 500)
				alphaFadeDelay = 10;
			alphaFade = true;
		}

		int value = Getx2RCNamedValue(name, popupgroup, "AlphaFade", alphaFadeValues, -1);

		if (value == -1)
		{
			alphaFade = Getx2RCBool(name, popupgroup, "AlphaFade", alphaFade);
		}
		else if ( value == 2 )
		{
			if ( BangToCall.empty() )
				alphaFade = false;
			else
				alphaFade = true;
		}
		else if ( value == 1 )
		{
			alphaFade = true;
		}

		if ( !alphaMap && (alphaFade || alphaTransparency != 255) )
			fakealphaMap = true;
		//---------------------------------------------------------------
	}
	else if ( !IsOS(OS_2KXP) )
	{
		alphaTransparency = 255;
		alphaFade = false;
		alphaMap = false;
		fakealphaMap = false;
	}

	//---------------------------------------------------------------
	//---------------------------------------------------------------
	if ( IsOS(OS_2KXP) && (alphaTransparency != 255 || alphaFade || alphaMap || fakealphaMap) )
	{
		ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

		if (alphaFade)
			UpdateWindowTransparent(hWnd, 0);
		else
			UpdateWindowTransparent(hWnd, alphaTransparency);
	}

	if ( quicklaunchMode != 0 || tooltipMode != 0 )
	{
		// struct specifying control classes to register
		INITCOMMONCONTROLSEX iccex; 

		/* INITIALIZE COMMON CONTROLS */
		iccex.dwICC = ICC_WIN95_CLASSES;
		iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);

		InitCommonControlsEx(&iccex);

		bool balloontooltip = false;
		string balloonTitle = "";
		int balloonIcon = 0;
		if ( IsOS( OS_2KXP ) || IsOS( OS_ME ) )
		{
			numTokens = SplitString(Getx2RCLine(name, popupgroup, "BalloonTooltip", "false"), one, two, three, four, five, six, seven);

			if ( _stricmp( one.c_str(), "false") != NULL )
			{
				balloontooltip = true;

				if ( numTokens >= 1 && !one.empty() && _stricmp( one.c_str(), "true") != NULL )
				{
					balloonTitle = one;
					if ( numTokens >= 2 )
						balloonIcon = atoi( two.c_str() );
					if ( balloonIcon > 3 || balloonIcon < 0 )
						balloonIcon = 0;
				}
			}
			else
			{
				balloontooltip = false;
			}
		}
		else
		{
			balloontooltip = false;
		}

		ToolTips = CreateWindowEx(WS_EX_TOOLWINDOW, TOOLTIPS_CLASS, TEXT(""), WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP | (balloontooltip ? TTS_BALLOON : 0),
									CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
									NULL, NULL, hInstance, NULL);

		if (ToolTips != 0)
		{
			SetWindowPos( ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE ); 

			SendMessage( ToolTips, TTM_SETMAXTIPWIDTH, 0, 500 );

			SendMessage( ToolTips, (UINT) TTM_SETTIPBKCOLOR, (WPARAM) (COLORREF) (Getx2RCColor(name, popupgroup, "ToolTipColor", GetSysColor(COLOR_INFOBK))), 0 );   
			SendMessage( ToolTips, (UINT) TTM_SETTIPTEXTCOLOR, (WPARAM) (COLORREF) (Getx2RCColor(name, popupgroup, "ToolTipTextColor", GetSysColor(COLOR_INFOTEXT))), 0);  
		
			numTokens = SplitString(Getx2RCLine(name, popupgroup, "TooltipDurations", "-1 -1"), one, two, three, four, five, six, seven);
		
			int initial = -1;
			int autopop = -1;

			if ( numTokens >= 1 )
			{
				initial = atoi(one.c_str());
				if ( numTokens >= 2 )
					autopop = atoi(two.c_str());
			}

			if ( initial <= 0 )
				initial = -1;
			if ( autopop <= 1000 )
				autopop = -1;

			if ( initial != -1 )
				SendMessage( ToolTips, (UINT) TTM_SETDELAYTIME,	(WPARAM) (DWORD) TTDT_INITIAL, (LPARAM) MAKELONG(initial, 0) );
			if ( autopop != -1 )
				SendMessage( ToolTips, (UINT) TTM_SETDELAYTIME,	(WPARAM) (DWORD) TTDT_AUTOPOP, (LPARAM) MAKELONG(autopop, 0) );

			if ( balloontooltip && !balloonTitle.empty() )
				SendMessage( ToolTips, TTM_SETTITLE, (WPARAM) (int) balloonIcon, (LPARAM) balloonTitle.c_str() );
		
			TOOLINFO ti;
			RECT clientRect;

			GetClientRect(hWnd, &clientRect);
			ti.cbSize = sizeof(TOOLINFO);
			ti.uFlags = TTF_SUBCLASS;
			ti.hwnd = hWnd;
			ti.uId = 0;
			ti.rect = clientRect;
			ti.hinst = hInstance;
			ti.lpszText = 0;
			ti.lParam = 0;

			SendMessage(ToolTips, TTM_ADDTOOL, 0, LPARAM(&ti));
		}

		updateHint(hWnd, "", true);
	}

	updateRegion();
}

//////////////////////////////////////////////////////////////////////
// Styles
//////////////////////////////////////////////////////////////////////

void Popup::setAlwaysOnTop(bool newalwaysOnTop)
{
	if ( newalwaysOnTop != alwaysOnTop )
	{
		alwaysOnTop = newalwaysOnTop;
		
		SetWindowPos(hWnd, newalwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
			0, 0, 0, 0,
			SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
	}
}

void Popup::OverlayButtonPainter(HDC hDC, HBITMAP hBitmap, int x, int y, int width, int height, int buttonType)
{
	if ( !ghostbuttonlist->empty() )
	{
		if ( (alphaMap || fakealphaMap) && hBitmap != NULL )
		{
			//Prepare Combined Image
			BITMAP bm;
			GetObject(hBitmap, sizeof(BITMAP), &bm);

			if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
			{
				BYTE *pPixel = (LPBYTE)bm.bmBits;

				int iSize = bm.bmHeight * bm.bmWidth;

				for (int j = 0; j < iSize; ++j)
				{
					//Convert Text to Full Opacity
					if (pPixel[3] == 0)
						pPixel[3] = 255;
					else if ( fakealphaMap )
						pPixel[3] = 255;

					pPixel+= 4;
				}
			}
		}

		for(GhostButtonListIterator gblit = ghostbuttonlist->begin(); gblit != ghostbuttonlist->end(); gblit++)
		{
			if ( *gblit != NULL )
			{
				int gbx = x + xParseCoord((*gblit)->orgx, 0, width, 0);
				int gby = y + xParseCoord((*gblit)->orgy, 0, height, 0);
				int gbwidth = xParseDimen((*gblit)->orgwidth, width, width);
				int gbheight = xParseDimen((*gblit)->orgheight, height, height);

				(*gblit)->buttontexture->applyOverlay(hDC, gbx, gby, gbwidth, gbheight, buttonType);
			}
			else
			{
				break;
			}

			if ( (alphaMap || fakealphaMap) && hBitmap != NULL )
			{
				//Prepare Combined Image
				BITMAP bm;
				GetObject(hBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				{
					BYTE *pPixel = (LPBYTE)bm.bmBits;

					int iSize = bm.bmHeight * bm.bmWidth;

					for (int j = 0; j < iSize; ++j)
					{
						//Convert Text to Full Opacity
						if (pPixel[3] == 0)
							pPixel[3] = 255;
						else if ( fakealphaMap )
							pPixel[3] = 255;

						pPixel+= 4;
					}
				}
			}
		}
	}
}

void Popup::OverlayPopupPainter(HDC hDC, HBITMAP hBitmap)
{
	if ( !ghostpopuplist->empty() )
	{
		if ( (alphaMap || fakealphaMap) && hBitmap != NULL )
		{
			//Prepare Combined Image
			BITMAP bm;
			GetObject(hBitmap, sizeof(BITMAP), &bm);

			if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
			{
				BYTE *pPixel = (LPBYTE)bm.bmBits;

				int iSize = bm.bmHeight * bm.bmWidth;

				for (int j = 0; j < iSize; ++j)
				{
					//Convert Text to Full Opacity
					if (pPixel[3] == 0)
						pPixel[3] = 255;
					else if ( fakealphaMap )
						pPixel[3] = 255;

					pPixel+= 4;
				}
			}
		}

		for(GhostPopupListIterator gplit = ghostpopuplist->begin(); gplit != ghostpopuplist->end(); gplit++)
		{
			if ( *gplit != NULL )
			{
				int gtx = xParseCoord((*gplit)->orgx, 0, actualwidth, 0);
				int gty = xParseCoord((*gplit)->orgy, 0, actualheight, 0);
				int gtwidth = xParseDimen((*gplit)->orgwidth, actualwidth, actualwidth);
				int gtheight = xParseDimen((*gplit)->orgheight, actualheight, actualheight);

				(*gplit)->popuptexture->applyOverlay(hDC, gtx, gty, gtwidth, gtheight, (*gplit)->alphatransparency );
			}
			else
			{
				break;
			}

			if ( (alphaMap || fakealphaMap) && hBitmap != NULL )
			{
				//Prepare Combined Image
				BITMAP bm;
				GetObject(hBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				{
					BYTE *pPixel = (LPBYTE)bm.bmBits;

					int iSize = bm.bmHeight * bm.bmWidth;

					for (int j = 0; j < iSize; ++j)
					{
						//Convert Text to Full Opacity
						if (pPixel[3] == 0)
							pPixel[3] = 255;
						else if ( fakealphaMap )
							pPixel[3] = 255;

						pPixel+= 4;
					}
				}
			}
		}
	}
}

void Popup::UpdateWindowTransparent(HWND hWnd, int factor)
{
	/* First, see if we can get the API call we need. If we've tried
    * once, we don't need to try again. */
	if (!alphaInitialized)
	{
		HMODULE hDLL = LoadLibrary ("user32");

		pSetLayeredWindowAttributes = 
					(PSLWA) GetProcAddress(hDLL, "SetLayeredWindowAttributes");

		pUpdateLayeredWindow = 
					(PULW) GetProcAddress(hDLL, "UpdateLayeredWindow");

		alphaInitialized = TRUE;

		ResetBackground();
   
		if (pSetLayeredWindowAttributes == NULL || pUpdateLayeredWindow == NULL)
		{
		    ModifyStyleEx(hWnd, WS_EX_LAYERED, 0 );

			alphaTransparency = 255;
			alphaFade = false;
			alphaMap = false;
			fakealphaMap = false;

			return;
		}
	}

	if ( resetBackground )
	{
		resetBackground = false;
		ResetBackground();
	}

	/* Now, we need to set the 'layered window attributes'. This
	 * is where the alpha values get set. */
  	if (alphaMap || fakealphaMap)
	{
		//------------------------------------------------------------------
		// double buffer for flicker-free paint
		HDC bufferDC = CreateCompatibleDC(NULL);

		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
		alpha.bmiHeader.biHeight=actualheight;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
		VOID *pvBits;
				
		HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

		HBITMAP oldbufferBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		// Use Complete Backup for WindowsInternal Repaint (Covering Windows, ...)
		if ( alphaFade && isFading && fadeBackupBitmap != NULL && fadeBackupDC != NULL )
		{
			BitBlt(bufferDC, 0, 0, actualwidth, actualheight, fadeBackupDC, 0, 0, SRCCOPY);
		}
		else
		//1----------------------------------------------------------------
		{
		//1----------------------------------------------------------------
				
			// make a rendition of the background
			HDC backgroundDC = CreateCompatibleDC(NULL);
				
			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
			alpha.bmiHeader.biHeight=actualheight;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			HBITMAP backgroundBitmap = CreateDIBSection( backgroundDC,
											&alpha,
											DIB_RGB_COLORS,
											&pvBits,
											NULL,
											0 );

			HBITMAP oldbackgroundBMP = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);

			//Normal Background Painting
			popuptexture->apply(backgroundDC, 0, 0, actualwidth, actualheight);

			OverlayPopupPainter(backgroundDC, backgroundBitmap);

			//Prepare Combined Image
			BITMAP bm0;
			GetObject(backgroundBitmap, sizeof(BITMAP), &bm0);

			if ((bm0.bmBits != 0) && (bm0.bmBitsPixel == 32))
			{
				BYTE *pPixel = (LPBYTE)bm0.bmBits;

				int iSize = bm0.bmHeight * bm0.bmWidth;

				for (int j = 0; j < iSize; ++j)
				{
					//Convert Text to Full Opacity
					if (pPixel[3] == 0)
						pPixel[3] = 255;
					else if ( fakealphaMap )
						pPixel[3] = 255;

					pPixel+= 4;
				}
			}
				
			// create & keep a cached rendition of the default popupmenu
			if (defaultBitmap == NULL)
			{
				if ( defaultDC == NULL )
					defaultDC = CreateCompatibleDC(NULL);

				BITMAPINFO alpha;
				ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
				alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
				alpha.bmiHeader.biHeight=actualheight;    // Set size you need
				alpha.bmiHeader.biPlanes=1;
				alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
				alpha.bmiHeader.biSizeImage=0;
				alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
				alpha.bmiHeader.biClrUsed= 0;
				alpha.bmiHeader.biClrImportant= 0;
				VOID *pvBits;

				defaultBitmap = CreateDIBSection( defaultDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

				olddefaultBMP = (HBITMAP) SelectObject(defaultDC, defaultBitmap);
					
				// blt background into double buffer
				BitBlt(defaultDC, 0, 0, actualwidth, actualheight, backgroundDC, 0, 0, SRCCOPY);

				//Only Painted if NOT Shaded
				if ( shaded == -1 )
				{
					int state = 0;
					int arrowstate = 0;
					for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
					{
						if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType != 2 )
						{
							//initialize
							state = 0;
							arrowstate = 0;

							if ( (*eit)->entryType == 1 )
							{
								state = 10;
							}
							else if ( (*eit)->entryType >= 0 )
							{
								state = 1;
							}
							else if ( (*eit)->entryType < 0 )
							{
								state = 3;
								arrowstate = 31;
							}
							else
							{
								continue;
							}
							
							//Paint Entry Texture
							buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
							OverlayButtonPainter( defaultDC, defaultBitmap, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
							//Paint Arrow
							if (arrowstate != 0)
							{
								buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate );
								OverlayButtonPainter( defaultDC, defaultBitmap, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate );
								buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
							}

							//If No Separator, Icon and Text Painting is valid
							if (state != 10)
							{
								buttonicon->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->m_pidl, DefaultIcon, alphaMap );

								if (quicklaunchMode == 0)
									buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
							}
						}
					}
				}

				//Prepare Combined Image
				BITMAP bm1;
				GetObject(defaultBitmap, sizeof(BITMAP), &bm1);

				if ((bm1.bmBits != 0) && (bm1.bmBitsPixel == 32))
				{
					BYTE *pPixel = (LPBYTE)bm1.bmBits;

					int iSize = bm1.bmHeight * bm1.bmWidth;

					for (int j = 0; j < iSize; ++j)
					{
						//Convert Text to Full Opacity
						if (pPixel[3] == 0)
							pPixel[3] = 255;
						else if ( fakealphaMap )
							pPixel[3] = 255;

						pPixel+= 4;
					}
				}
			}

			// blt default background into double buffer
			BitBlt(bufferDC, 0, 0, actualwidth, actualheight, defaultDC, 0, 0, SRCCOPY);

			if ( quicklaunchMode == 0 )
			{
				if ( topBorder > 0 )
				{
					buttontexture->apply( bufferDC, 0, 0, actualwidth, topBorder, -1 );
					OverlayButtonPainter( bufferDC, bufferBitmap, 0, 0, actualwidth, topBorder, -1 );
				}
				if ( bottomBorder > 0 )
				{
					buttontexture->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, -2);
					OverlayButtonPainter( bufferDC, bufferBitmap, 0, actualheight-bottomBorder, actualwidth, bottomBorder, -2 );
				}
				if (showCaption == 1)
				{
					if ( pParentPopup != NULL && pParentEntry != NULL )
						buttonicon->apply( bufferDC, 0, 0, actualwidth, topBorder, 0, pParentEntry->iconPath, pParentEntry->hWnd, pParentEntry->m_pidl, "", alphaMap );
					if ( PinnedMode )
						buttonicon->apply( bufferDC, 0, 0, actualwidth, topBorder, -1, "", NULL, NULL, "", alphaMap );

					buttonfont->apply( bufferDC, 0, 0, actualwidth, topBorder, captionname, 0);
				}
				else if (showCaption == 2)
				{
					if ( pParentPopup != NULL && pParentEntry != NULL )
						buttonicon->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, 0, pParentEntry->iconPath, pParentEntry->hWnd, pParentEntry->m_pidl, "", alphaMap );
					if ( PinnedMode )
						buttonicon->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, -1, "", NULL, NULL, "", alphaMap );

					buttonfont->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, captionname, 0);
				}
				else if (showCaption == 3)
				{
					if ( pParentPopup != NULL && pParentEntry != NULL )
						buttonicon->apply( bufferDC, 0, 0, actualwidth, actualheight, 0, pParentEntry->iconPath, pParentEntry->hWnd, pParentEntry->m_pidl, "", alphaMap );
					if ( PinnedMode )
						buttonicon->apply( bufferDC, 0, 0, actualwidth, actualheight, -1, "", NULL, NULL, "", alphaMap );

					buttonfont->apply( bufferDC, 0, 0, actualwidth, actualheight, captionname, 0);
				}
			}
			//Only Painted if NOT Shaded
			if ( shaded == -1 )
			{
				int state = 0;
				int arrowstate = 0;
				if ( quicklaunchMode == 0 && hasInfoEntry )
				{
					for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
					{
						if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType == 2 )
						{
							state = 5;

							//Normal Background Painting behind ActiveEntry
							BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

							//Paint Entry Texture
							buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
							OverlayButtonPainter( bufferDC, bufferBitmap, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
							
							buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
						}
					}
				}
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					//initialize
					state = 0;
					arrowstate = 0;

					if ( currentPart == (*eit)->hiddenPart && (*eit) == hoverentry && (*eit) != NULL )
					{
						if ( (*eit)->entryType == 0 )
						{
							state = 2;
						}
						else if ( (*eit)->entryType < 0 )
						{
							state = 4;
							arrowstate = 41;
						}
						else
						{
							continue;
						}
							
						//Normal Background Painting behind ActiveEntry
						BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

						//Paint Entry Texture
						buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
						OverlayButtonPainter( bufferDC, bufferBitmap, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
						
						//Paint Arrow
						if (arrowstate != 0)
						{
							buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate );
							OverlayButtonPainter( bufferDC, bufferBitmap, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate );
							buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
						}

						//If No Info, Icon is valid
						buttonicon->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->m_pidl, DefaultIcon, alphaMap );
						if (quicklaunchMode == 0)
							buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
					}
				}
			}

			//Prepare Combined Image
			BITMAP bm2;
			GetObject(bufferBitmap, sizeof(BITMAP), &bm2);

			if ((bm2.bmBits != 0) && (bm2.bmBitsPixel == 32))
			{
				BYTE *pPixel = (LPBYTE)bm2.bmBits;

				int iSize = bm2.bmHeight * bm2.bmWidth;

				for (int j = 0; j < iSize; ++j)
				{
					//Convert Text to Full Opacity
					if (pPixel[3] == 0)
						pPixel[3] = 255;

					//Magic Pink
					if (hasHookedWindows && pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
						pPixel[3] = 0;

					//Remove Prepared Alpha (1 back to 0)
					if (pPixel[3] == 1)
						pPixel[3] = 0;

					if ( fakealphaMap )
						pPixel[3] = 255;

					pPixel+= 4;
				}
			}
			//------------------------------------------------------------------

			//Create fadeBackupBitmap, if it doesn't exist!
			//----------------------------------------------------------------------
			if ( alphaFade && isFading && fadeBackupBitmap == NULL && fadeBackupDC == NULL )
			{
				fadeBackupDC = CreateCompatibleDC(NULL);
				
				BITMAPINFO alpha;
				ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
				alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
				alpha.bmiHeader.biHeight=actualheight;    // Set size you need
				alpha.bmiHeader.biPlanes=1;
				alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
				alpha.bmiHeader.biSizeImage=0;
				alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
				alpha.bmiHeader.biClrUsed= 0;
				alpha.bmiHeader.biClrImportant= 0;
				VOID *pvBits;

				fadeBackupBitmap = CreateDIBSection( fadeBackupDC,
														&alpha,
														DIB_RGB_COLORS,
														&pvBits,
														NULL,
														0 );

				oldfadeBackupBMP = (HBITMAP) SelectObject(fadeBackupDC, fadeBackupBitmap);

				BitBlt(fadeBackupDC, 0, 0, actualwidth, actualheight, bufferDC, 0, 0, SRCCOPY);
			}
			//----------------------------------------------------------------------

			backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, oldbackgroundBMP);
			DeleteObject(backgroundBitmap);
			DeleteDC(backgroundDC);	
			DeleteObject(oldbackgroundBMP);
		//1
		}
		//1}
	
		POINT ptWindowScreenPosition = {actualx, actualy};
		SIZE szWindow = {actualwidth, actualheight};

		POINT ptSrc = {0,0};

		BLENDFUNCTION blendPixelFunction = { AC_SRC_OVER, 0, factor, AC_SRC_ALPHA };

		pUpdateLayeredWindow(hWnd, NULL, &ptWindowScreenPosition, &szWindow, bufferDC, &ptSrc, 0, &blendPixelFunction, ULW_ALPHA );
		
		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldbufferBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);	
		DeleteObject(oldbufferBMP);
	}
	else
	{
		pSetLayeredWindowAttributes(hWnd, 0, factor, LWA_ALPHA);
	}
}

//////////////////////////////////////////////////////////////////////
// Popup Creation -> StepParsing, Folder Scanning, Shell Scanning, ...
//////////////////////////////////////////////////////////////////////

void Popup::ScanKeyCodes(ENTRY* processEntry, string processString)
{
	//Ampersand Keyboard navigation
	//------------------------------------------------------------
	char* m_pszTitle = strdup(processString.c_str());

	// search the string for '&', that is keyboard-shortcuts
	int len = lstrlen(m_pszTitle);
	for (int i = 0; i < len; i++)
	{
		if (m_pszTitle[i] == '&' && m_pszTitle[i + 1] != '\0')
		{
			processEntry->keyCode = islower(m_pszTitle[i + 1]) ? toupper(m_pszTitle[i + 1]) : m_pszTitle[i + 1];
			break;
		}
	}
	
	if (processEntry->keyCode == -1)
	{
		processEntry->keyCode = islower(m_pszTitle[0]) ? toupper(m_pszTitle[0]) : m_pszTitle[0];
		processEntry->keyCode1 = islower(m_pszTitle[1]) ? toupper(m_pszTitle[1]) : m_pszTitle[1];
	}
	//------------------------------------------------------------
}

bool Popup::ParseDynamicEvars(string input, string &output)
{
	char expandedEvar[MAX_LINE_LENGTH] = { 0 };
	CString worker0 = input.c_str();
	CString worker1 = "";

	int	pos = -1;

	worker0.Replace("%#", "$");

	pos = worker0.Find("$");
	pos = worker0.Find("$", pos+1);

	if ( pos != -1 )
	{
		while ( pos != -1 )
		{
			worker1 = worker0;
			worker1 = worker1.Right(worker1.GetLength()-worker1.Find("$"));
			worker1 = worker1.Left(worker1.Find("$", 1)+1);

			VarExpansion( expandedEvar, worker1 );

			worker0.Replace(worker1, expandedEvar);

			pos = worker0.Find("$");
			pos = worker0.Find("$", pos+1);
		}

		output = (string) worker0;

		return true;
	}
	else
	{ 
		output = input;

		return false;
	}
}

void Popup::ParseStepRC(bool externCall)
{
	if ( hasDynamicContent )
	{	
		string str_singleCheckString = "";
		string str_completeCheckString = "";

		for (StringListIterator sit = popupcontentList.begin(); sit != popupcontentList.end(); ++sit)
		{
			ParseDynamicEvars((*sit), str_singleCheckString);
		
			str_completeCheckString = (str_completeCheckString + str_singleCheckString);
		}
		
		//Check if the entryCheckString has changed (only reason for real updateing)
		if ( strcmp(str_completeCheckStringBackup.c_str(), str_completeCheckString.c_str()) != NULL )
		{		
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				delete *eit;
				*eit = NULL;
			}

			entrylist.clear();

			for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
			{
				(*plit)->hide();
				delete *plit;
				*plit = NULL;
			}

			subpopupList.clear();

			//Reset old Pointers
			//-------------------------
			currentPart = 0;

			startentry = NULL;
			lastactiveentry = NULL;
			hoverentry = NULL;
			//-------------------------
			
			str_completeCheckStringBackup = str_completeCheckString;
		}
		else
		{
			if ((isTaskPopup || hasDynamicContent) && !hasInfoEntry)
				SetTimer(hWnd, TIMER_UPDATE, tasksUpdate, NULL);
			else if (hasInfoEntry)
				SetTimer(hWnd, TIMER_UPDATE, 1000, NULL);

			return;
		}
	}
	
	StringList hookcontentList;
	StringList settingsList = popupcontentList;

	if ( !settingsList.empty() && settingsList.size() == 1 )
	{
		char szFirst[MAX_LINE_LENGTH] = { 0 };
		char szSecond[MAX_LINE_LENGTH] = { 0 };
		char szThird[MAX_LINE_LENGTH] = { 0 };

		char *buffer[] = { szFirst, szSecond, szThird };

		int nTokens = LCTokenize((*settingsList.begin()).c_str(), buffer, 2, szThird);

		CString str_third = TrimAndStripQuote((string)szThird).c_str();

		if ( _stricmp(str_third.Left(19), "!popupdynamicfolder") == NULL || _stricmp(str_third.Left(14), "!dynamicfolder") == NULL || _stricmp(str_third.Left(12), "!popupfolder") == NULL )
		{
			CString easyHelper = TrimAndStripQuote( (string)szThird ).c_str();

			// Get FolderPath
			easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find(":")-1);

			// Strip surrounding '"'
			easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();

			rootScanPath = (string)easyHelper;

			if ( _stricmp(str_third.Left(19), "!popupdynamicfolder") == NULL || _stricmp(str_third.Left(14), "!dynamicfolder") == NULL )
			{
				ScanSubMenu( rootScanPath, -2 );

				bool newIteration = true;
				CString localworker1 = easyHelper;
				CString localworker2 = "";

				while (newIteration == true)
				{
					if (localworker1.Find("|") != -1)
					{
						CNotifyDirCheck *checkDirectory = new CNotifyDirCheck();
				
						checkdirectorylist.insert(checkdirectorylist.end(), checkDirectory);
					
						localworker2 = localworker1.Left(localworker1.Find("|"));

						if ( localworker2.Find("*") == -1 )
							checkDirectory->SetDirectory( localworker2 );
						else
							checkDirectory->SetDirectory( localworker2.Left(localworker2.Find("*")) );

						//set your callback to work with each new event
						checkDirectory->SetPopup( this );

						checkDirectory->Run();

						localworker1 = localworker1.Right(localworker1.GetLength()-localworker1.Find("|")-1);
					}
					else
					{
						CNotifyDirCheck *checkDirectory = new CNotifyDirCheck();
				
						checkdirectorylist.insert(checkdirectorylist.end(), checkDirectory);
					
						if ( localworker1.Find("*") == -1 )
							checkDirectory->SetDirectory( localworker1 );
						else
							checkDirectory->SetDirectory( localworker1.Left(localworker1.Find("*")) );

						//set your callback to work with each new event
						checkDirectory->SetPopup( this );

						checkDirectory->Run();

						newIteration = false;
					}
				}
			}
			else
			{
				ScanSubMenu( rootScanPath, -1 );
			}

			return;
		}
		else if ( _stricmp(str_third.Left(25), "!popupdynamicactionfolder") == NULL || _stricmp(str_third.Left(18), "!popupactionfolder") == NULL )
		{
			CString easyHelper = szThird;

			// Strip !action between ":" ... ":"
			int pos = easyHelper.Find(":");
			if ( easyHelper.Find(":", pos+3) == -1 )
			{
				return;
			}

			//Find 2nd ":" from backwards
			int sep2 = easyHelper.ReverseFind(':');
			sep2 = easyHelper.Find(":", sep2-3); // ...:c:\path

			// Get universalAction
			CString universalAction = easyHelper.Mid(pos+1, sep2-pos-1);

			easyHelper = easyHelper.Right(easyHelper.GetLength()-sep2-1);

			// Strip surrounding '"'
			easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();

			rootScanPath = (string)easyHelper;
			rootUniversalAction = (string)universalAction;

			if ( _stricmp(str_third.Left(25), "!popupdynamicactionfolder") == NULL )
			{
				ScanSubMenu( rootScanPath, -2, rootUniversalAction );

				bool newIteration = true;
				CString localworker1 = easyHelper;
				CString localworker2 = "";

				while (newIteration == true)
				{
					if (localworker1.Find("|") != -1)
					{
						CNotifyDirCheck *checkDirectory = new CNotifyDirCheck();
				
						checkdirectorylist.insert(checkdirectorylist.end(), checkDirectory);
					
						localworker2 = localworker1.Left(localworker1.Find("|"));

						if ( localworker2.Find("*") == -1 )
							checkDirectory->SetDirectory( localworker2 );
						else
							checkDirectory->SetDirectory( localworker2.Left(localworker2.Find("*")) );

						//set your callback to work with each new event
						checkDirectory->SetPopup( this );

						checkDirectory->Run();

						localworker1 = localworker1.Right(localworker1.GetLength()-localworker1.Find("|")-1);
					}
					else
					{
						CNotifyDirCheck *checkDirectory = new CNotifyDirCheck();
				
						checkdirectorylist.insert(checkdirectorylist.end(), checkDirectory);
					
						if ( localworker1.Find("*") == -1 )
							checkDirectory->SetDirectory( localworker1 );
						else
							checkDirectory->SetDirectory( localworker1.Left(localworker1.Find("*")) );

						//set your callback to work with each new event
						checkDirectory->SetPopup( this );

						checkDirectory->Run();

						newIteration = false;
					}
				}
			}
			else
			{
				ScanSubMenu( rootScanPath, -1, rootUniversalAction );
			}

			return;
		}
		else if ( _stricmp(str_third, "!popuptasks") == NULL )
		{
			quicklaunchMode = 0;

			isTaskPopup = true;

			ScanTasks();

			return;
		}
		else if ( _stricmp(str_third, "!popupcontrolpanel") == NULL || _stricmp(str_third, "!popupprinters") == NULL 
						|| _stricmp(str_third, "!popuprecyclebin") == NULL || _stricmp(str_third, "!popuprecentdocuments") == NULL
						|| _stricmp(str_third, "!popupnetwork") == NULL || _stricmp(str_third, "!popupmycomputer") == NULL )
		{
			quicklaunchMode = 0;

			if ( _stricmp(str_third, "!popupcontrolpanel") == NULL )
				ScanShellFolder( CSIDL_CONTROLS );
			else if (_stricmp(str_third, "!popupmycomputer") == NULL )
				ScanShellFolder( CSIDL_DRIVES );
			else if (_stricmp(str_third, "!popupnetwork") == NULL )
				ScanShellFolder( CSIDL_NETWORK );
			else if (_stricmp(str_third, "!popupprinters") == NULL )
				ScanShellFolder( CSIDL_PRINTERS );
			else if (_stricmp(str_third, "!popuprecentdocuments") == NULL )
				ScanShellFolder( CSIDL_RECENT );
			else if (_stricmp(str_third, "!popuprecyclebin") == NULL )
				ScanShellFolder( CSIDL_BITBUCKET );
		
			return;
		}
	}
	
	if (!settingsList.empty())
	{
		char szFirst[MAX_LINE_LENGTH] = { 0 };
		char szSecond[MAX_LINE_LENGTH] = { 0 };
		char szThird[MAX_LINE_LENGTH] = { 0 };

		char *buffer[] = { szFirst, szSecond, szThird };

		//Initializing of Values
		int itemCounter = 0;
		int nextXPosition = leftBorder;
		int nextYPosition = topBorder;
		int hiddenPart = 0;
		int menuBreaks = 0;
		int maxNeededWidth = 0;
		int adaptedmaxHeight = 0;
		bool customIcon = false;
		int nTokens = 0;

		//For Folder ~Folder
		//--------------------------------------
		Popup* addsettingPopup = NULL;
		string addsettingCustomIcon = "";
		bool input = false;
		int subfolderopen = 0;
		StringList sendToPopup;
		//--------------------------------------

		//Helper CStrings
		CString str_first = "";
		CString str_second = "";
		CString str_third = "";

		//SubPopupLayout
		CString str_fourth = "";
		//SubPopupLayout Group
		CString str_fifth = "";

		CString easyHelper = "";
		CString folderCaption = "";

		for (StringListIterator sit = settingsList.begin(); sit != settingsList.end(); ++sit)
		{
			//Check for Dynamic Settings %#
			string str_check = "";

			if ( !hasDynamicContent )
				hasDynamicContent = ParseDynamicEvars((*sit), str_check);
			else
				ParseDynamicEvars((*sit), str_check);

			//Read Settings line ( *(x)Popup )
			//Parse the Line for %# Dynamic Evars
			nTokens = LCTokenize(str_check.c_str() , buffer, 2, szThird);
			//-----------------------------------

			//Icon
			str_first = szFirst;
			str_first.MakeLower();
			str_first.Replace(".icon=", "");
			str_first = TrimAndStripQuote( (string)str_first ).c_str();

			if ( _stricmp(str_first, ".noexist") != NULL )
			{
				str_first.Replace("/", "\\");
				str_first.Replace("\\\\", "\\");
				customIcon = true;
			}
			else
			{
				customIcon = false;
			}
		
			//Caption
			str_second = szSecond;
			//Action
			str_third = TrimAndStripQuote( (string) szThird ).c_str();

			int foundExtraConfig = str_third.Find("%%");
			if ( foundExtraConfig != -1 && _stricmp(str_second.Left(5), "!hook") != NULL )
			{
				str_fourth = str_third.Right(str_third.GetLength()-foundExtraConfig-2);
				str_fourth.Replace("%%", "");

				int foundExtraGroupConfig = str_fourth.Find("%");
				if ( foundExtraGroupConfig != -1 )
				{
					str_fifth = str_fourth.Right(str_fourth.GetLength()-foundExtraGroupConfig-1);
					str_fourth = str_fourth.Left(foundExtraGroupConfig);
				}
				
				str_third = str_third.Left(foundExtraConfig);
				str_third = TrimAndStripQuote( (string) str_third ).c_str();
			}
			else
			{
				str_fourth = "";
				str_fifth = "";
			}
		
			//-----------------------------------------------------
			// MenuBreak
			//-----------------------------------------------------
			if ( nTokens >= 2 && (_stricmp(str_second, "Menubreak") == NULL || _stricmp(str_third, "Menubreak") == NULL) )
			{
				if ( quicklaunchMode != 0 )
					continue;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;

				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextYPosition = topBorder;

				menuBreaks++;
			}
			//-----------------------------------------------------
			// Manually Created SubFolder START
			//-----------------------------------------------------
			else if ( nTokens >= 2 && (_stricmp(str_second, "Folder") == NULL || _stricmp(str_third, "Folder") == NULL) )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);
					subfolderopen++;
				}
				else
				{
					//Search for Popup Start with !New and Create New Popup
					Popup *popup = NULL;
					if (_stricmp(str_second, "Folder") == NULL)
					{
						if ( customSubFolderList.empty() && str_fourth.IsEmpty() )
							popup = new Popup(name, "sub_Unknown", popuptexture, buttontexture, buttonicon, buttonfont, ghostpopuplist, ghostbuttonlist);
						else if ( !str_fourth.IsEmpty() )
							popup = new Popup((string)str_fourth, "sub_Unknown", NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, (str_fifth.IsEmpty() ? name : (string)str_fifth));
						else if ( subFolderLevel < customSubFolderList.size() )
							popup = new Popup(customSubFolderList[subFolderLevel], "sub_Unknown", NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
						else
							popup = new Popup(*customSubFolderList.end(), "sub_Unknown", NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
					}
					if (_stricmp(str_third, "Folder") == NULL)
					{
						folderCaption = str_second;
						if ( tooltipMode != 0 )
						{
							if ( strstr(str_second, "||") != NULL )
							{
								easyHelper = str_second.Left(str_second.Find("||"));
								easyHelper = ("sub_" + easyHelper);

								if ( customSubFolderList.empty() && str_fourth.IsEmpty() )
									popup = new Popup(name, (string)easyHelper, popuptexture, buttontexture, buttonicon, buttonfont, ghostpopuplist, ghostbuttonlist);
								else if ( !str_fourth.IsEmpty() )
									popup = new Popup((string)str_fourth, (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, (str_fifth.IsEmpty() ? name : (string)str_fifth));
								else if ( subFolderLevel < customSubFolderList.size() )
									popup = new Popup(customSubFolderList[subFolderLevel], (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
								else
									popup = new Popup(*customSubFolderList.end(), (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
							}
							else
							{
								easyHelper = ("sub_" + str_second);

								if ( customSubFolderList.empty() && str_fourth.IsEmpty() )
									popup = new Popup(name, (string)easyHelper, popuptexture, buttontexture, buttonicon, buttonfont, ghostpopuplist, ghostbuttonlist);
								else if ( !str_fourth.IsEmpty() )
									popup = new Popup((string)str_fourth, (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, (str_fifth.IsEmpty() ? name : (string)str_fifth));
								else if ( subFolderLevel < customSubFolderList.size() )
									popup = new Popup(customSubFolderList[subFolderLevel], (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
								else
									popup = new Popup(*customSubFolderList.end(), (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
							}
						}
						else
						{
							if ( strstr(str_second, "||") != NULL )
							{
								easyHelper = str_second.Left(str_second.Find("||"));
								easyHelper = ("sub_" + easyHelper);
							}
							else
							{
								easyHelper = ("sub_" + str_second);
							}

							if ( customSubFolderList.empty() && str_fourth.IsEmpty() )
								popup = new Popup(name, (string)easyHelper, popuptexture, buttontexture, buttonicon, buttonfont, ghostpopuplist, ghostbuttonlist);
							else if ( !str_fourth.IsEmpty() )
								popup = new Popup((string)str_fourth, (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, (str_fifth.IsEmpty() ? name : (string)str_fifth));
							else if ( subFolderLevel < customSubFolderList.size() )
								popup = new Popup(customSubFolderList[subFolderLevel], (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
							else
								popup = new Popup(*customSubFolderList.end(), (string)easyHelper, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
						}
					}
			
					if (popup)
					{
						subpopupList.insert(subpopupList.end(), popup);
								
						addsettingPopup = popup;
						addsettingCustomIcon = str_first;

						//Allow following settings to be tracked
						input = true;

						continue;
					}
				}
			}
			//-----------------------------------------------------
			// Manually Created SubFolder END and CREATION
			//-----------------------------------------------------
			else if (nTokens >= 1 && (_stricmp(str_first, "~Folder") == NULL || _stricmp(str_second, "~Folder") == NULL))
			{
				bool closeFolder = false;
				if (input)
				{
					//Add to specific Popup Settings list
					if (subfolderopen > 0)
					{
						sendToPopup.insert(sendToPopup.end(), *sit);
						subfolderopen--;
					}
					else
					{
						closeFolder = true;
					}
				}
				
				if (!input || closeFolder)
				{
					//Close Popup Definition, give Settings and Load
					if (addsettingPopup)
					{
						ENTRY* newitem = new ENTRY;
						
						//This is the parent for the Created PopupSubMenu
						addsettingPopup->pParentPopup = this;
						addsettingPopup->pParentEntry = newitem;

						addsettingPopup->load(hInstance);
						addsettingPopup->popupcontentList = sendToPopup;
						addsettingPopup->ParseStepRC();

						newitem->entryType = -1;

						if (quicklaunchMode != 0)
						{
							itemCounter++;

							newitem->tooltip = addsettingPopup->captionname;

							newitem->width = quicklaunchItemSize;
							newitem->height = quicklaunchItemSize;
							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							if (quicklaunchMode == 1)
							{
								if (itemCounter%quicklaunchWrapCount != 0)
								{
									nextXPosition = nextXPosition+newitem->width+xSpacing;
									nextYPosition = nextYPosition;
								}
								else
								{
									nextXPosition = leftBorder;
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
							}
							else if (quicklaunchMode == 2)
							{
								if (itemCounter%quicklaunchWrapCount != 0)
								{
									nextXPosition = nextXPosition;
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
								else
								{
									nextXPosition = nextXPosition+newitem->width+xSpacing;
									nextYPosition = topBorder;
								}
							}
						}
						else
						{
							newitem->x = nextXPosition;
							newitem->y = nextYPosition;
							newitem->height = folderHeight;

							if (nextYPosition+newitem->height+bottomBorder > maxHeight)
							{
								if (autoMenuBreak)
								{
									if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
										adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

									nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
									nextYPosition = topBorder;

									newitem->x = nextXPosition;
									newitem->y = nextYPosition;

									menuBreaks++;

									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
								else
								{
									if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
										adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
									
									nextYPosition = topBorder;

									newitem->y = nextYPosition;

									hiddenPart++;
									
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
							}
							else
							{
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						
						newitem->hiddenPart = hiddenPart;
						newitem->multiPart = menuBreaks;
						
						easyHelper = addsettingPopup->subname.c_str();
						easyHelper.Replace("sub_", "");

						if ( tooltipMode != 0 )
						{
							if ( strstr(folderCaption, "||") != NULL )
							{
								newitem->caption = folderCaption.Left(folderCaption.Find("||"));
								if ( strstr(folderCaption, "\\n") != NULL )
									folderCaption.Replace("\\n", "\n");
								newitem->tooltip = folderCaption.Right(folderCaption.GetLength()-folderCaption.Find("||")-2);
							}
							else
							{
								newitem->caption = easyHelper;
							}	
						}
						else
						{
							if ( strstr(folderCaption, "||") != NULL )
							{
								easyHelper = folderCaption.Left(folderCaption.Find("||"));
							}
						
							newitem->caption = easyHelper;
						}

						ScanKeyCodes(newitem, (string) easyHelper);

						if ( DefaultFolderIcon.empty() )
						{
							if ( _stricmp(addsettingCustomIcon.c_str(), ".noexist") == NULL )
								newitem->iconPath = "*";
							else
								newitem->iconPath = addsettingCustomIcon;
						}
						else
						{
							if ( _stricmp(addsettingCustomIcon.c_str(), ".noexist") == NULL )
								newitem->iconPath = DefaultFolderIcon;
							else
								newitem->iconPath = addsettingCustomIcon;
						}

						POINT tmp = buttonfont->measure(newitem->caption);
						
						if (maxNeededWidth < tmp.x)
							maxNeededWidth = tmp.x;

						entrylist.insert(entrylist.end(), newitem);
					}
					addsettingPopup = NULL;
					addsettingCustomIcon = "";
					input = false;
					sendToPopup.clear();
				}
			}
			//-----------------------------------------------------
			// SEPARATOR
			//-----------------------------------------------------
			else if ( _stricmp(str_second, "!separator") == NULL || _stricmp(str_second, "!seperator") == NULL )
			{
				if ( quicklaunchMode != 0 )
					continue;

				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;	
					
					newitem->entryType = 1;

					newitem->x = nextXPosition;
					if ( nextYPosition != topBorder )
					{
						newitem->y = nextYPosition+separatorySpacing;
						nextYPosition = newitem->y;
					}
					else
					{
						newitem->y = nextYPosition;
					}
					
					newitem->height = separatorHeight;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
						}
					}
					else
					{
						nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
					}
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// MODULE HOOK
			//-----------------------------------------------------
			else if ( _stricmp(str_second.Left(5), "!hook") == NULL )
			{
				if ( quicklaunchMode != 0 )
					continue;

				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					//Disable FakeAlphaMap
					fakealphaMap = false;

					str_second = str_second.Right(str_second.GetLength()-6);
					int hookleft = atoi( str_second.Left(str_second.Find(":")) );
					str_second = str_second.Right(str_second.GetLength()-str_second.Find(":")-1);
					int hookright = atoi(str_second.Left(str_second.Find(":")));
					str_second = str_second.Right(str_second.GetLength()-str_second.Find(":")-1);
					int hookwidth = atoi(str_second.Left(str_second.Find(":")));
					str_second = str_second.Right(str_second.GetLength()-str_second.Find(":")-1);
					int hookheight = atoi(str_second);

					string hWndbuffer = IntToString( (int) hWnd );
					string config = str_third;

					config.append( " \"$themedir$theme.rc\" " );
					config.append( hWndbuffer );

					LSExecute(NULL, config.c_str(), SW_SHOWNORMAL);

					if (hookleft == 0 && hookright == 0 && hookwidth == 0 && hookheight == 0)
						continue;

					if (nextYPosition-ySpacing+hookheight+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							easyHelper = str_third;
							if (easyHelper.Find("%%") != -1)
							{
								easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find("%%")-2);
								easyHelper = easyHelper.Left(easyHelper.Find("%%"));
								easyHelper += " ";
								easyHelper += IntToString(hookleft).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextXPosition).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextYPosition).c_str();
							
								hookcontentList.insert(hookcontentList.end(), (string)easyHelper);
							}

							menuBreaks++;

							nextYPosition = nextYPosition+hookheight+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							easyHelper = str_third;
							if (easyHelper.Find("%%") != -1)
							{
								easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find("%%")-2);
								easyHelper = easyHelper.Left(easyHelper.Find("%%"));
								easyHelper += " ";
								easyHelper += IntToString(hookleft).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextXPosition).c_str();
								easyHelper += " ";
								easyHelper += IntToString(nextYPosition).c_str();
							
								hookcontentList.insert(hookcontentList.end(), (string)easyHelper);
							}

							hiddenPart++;
							
							nextYPosition = nextYPosition+hookheight+ySpacing;
						}
					}
					else
					{
						easyHelper = str_third;
						if (easyHelper.Find("%%") != -1)
						{
							easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find("%%")-2);
							easyHelper = easyHelper.Left(easyHelper.Find("%%"));
							easyHelper += " ";
							easyHelper += IntToString(hookleft).c_str();
							easyHelper += " ";
							easyHelper += IntToString(nextXPosition).c_str();
							easyHelper += " ";
							easyHelper += IntToString(nextYPosition).c_str();
							
							hookcontentList.insert(hookcontentList.end(), (string)easyHelper);
						}

						nextYPosition = nextYPosition+hookheight+ySpacing;
					}

					if (maxNeededWidth < hookleft+hookwidth+hookright)
						maxNeededWidth = hookleft+hookwidth+hookright;

					hasHookedWindows = true;
				}
			}
			//-----------------------------------------------------
			// INFO ENTRY (xLabel(Light)) REQUIRED
			//-----------------------------------------------------
			else if ( _stricmp(str_second.Left(5), "!info") == NULL )
			{
				if ( quicklaunchMode != 0 )
					continue;

				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					//Needs the original "szThird", cause "str_third" hasn't Quotations anymore
					newitem->info = szThird;

					newitem->entryType = 2;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = infoHeight;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
					{
						nextYPosition = nextYPosition+newitem->height+ySpacing;
					}
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					newitem->caption = InfoUpdate( newitem->info );

					//Measure with Type 2 only check InfoFont Settings
					POINT tmp = buttonfont->measure(newitem->caption, 2);
					
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);

					hasInfoEntry = true;
				}
			}
			//-----------------------------------------------------
			// INSERT POPUPFOLDERS
			//-----------------------------------------------------
			else if ( _stricmp(str_second.Left(13), "!insertfolder") == NULL || _stricmp(str_second.Left(14), "!xinsertfolder") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					easyHelper = str_second;
					
					// Get FolderPath
					easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find(":")-1);
					easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();
					//---------------------------------------------------------------------------------------
					// START Scanning
					//---------------------------------------------------------------------------------------
					string searchPath = (string)easyHelper;
					string wildcardPath = "";
					string itemName = "";
					string tmpHelper = "";

					HANDLE hFindHandle;
					WIN32_FIND_DATA FindData;

					vector<string> folders;
					vector<string> files;

					bool newIteration = true;
					CString worker = searchPath.c_str();
					while (newIteration == true)
					{
						if (worker.Find("|") != -1)
						{
							searchPath = worker.Left(worker.Find("|"));
							worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
						}
						else
						{
							searchPath = worker;
							newIteration = false;
						}
						
						//Check for ending BackSlash
						tmpHelper = searchPath.substr(searchPath.length()-1, searchPath.length());
						bool addBackslash = false;
						if (tmpHelper != "\\" && strstr(searchPath.c_str(), "*.") == NULL)
						{
							wildcardPath = (searchPath + "\\*.*");
							addBackslash = true;
						}
						else if (strstr(searchPath.c_str(), "*.") == NULL)
						{
							wildcardPath = (searchPath + "*.*");
						}
						else
						{
							wildcardPath = searchPath;
							searchPath = searchPath.substr(0, searchPath.find('*'));
						}

						hFindHandle = FindFirstFile(wildcardPath.c_str(), &FindData);
						
						for(;;)
						{
							if(INVALID_HANDLE_VALUE == hFindHandle)
								break;

							itemName = FindData.cFileName;

							//It is a folder
							if (_stricmp(itemName.c_str(), ".") != NULL && _stricmp(itemName.c_str(), "..") != NULL && !(FindData.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN))
							{
								if (addBackslash)
									tmpHelper = (searchPath + "\\" + itemName);
								else
									tmpHelper = (searchPath + itemName);
								
								if (FindData.dwFileAttributes &  FILE_ATTRIBUTE_DIRECTORY)
								{
									//Check for double folders and unite
									CString helper = "";
									bool found = false;
									for (StringListIterator sit = folders.begin(); sit != folders.end(); ++sit)
									{
										easyHelper = (*sit).c_str();
										
										if (_stricmp( itemName.c_str(), easyHelper.Right( itemName.length() ) ) == NULL)
										{
											easyHelper += "|";
											easyHelper += tmpHelper.c_str();
											(*sit) = (string)easyHelper;

											found = true;
											break;
										}
									}
									
									if ( !found )
										folders.insert(folders.end(), tmpHelper);
								}
								else
								{
									files.insert(files.end(), tmpHelper);
								}
							}

							if(!FindNextFile(hFindHandle, &FindData))
								break;
						}
					}

					FindClose(hFindHandle);
					//---------------------------------------------------------------------------------------
					// END Scanning
					//---------------------------------------------------------------------------------------

					//Sorting
					Sort(folders);
					Sort(files);

					StringListIterator sit;

					for (sit = folders.begin(); sit != folders.end(); ++sit)
					{
						ENTRY* newitem = new ENTRY;
						
						easyHelper = (*sit).c_str();
						easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);

						//Remove Custom Sort HelperStrings
						if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
							easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
						//--------------------------------
						newitem->caption = easyHelper;

						ScanKeyCodes(newitem, (string) easyHelper);
									
						newitem->entryType = -2;

						if (quicklaunchMode != 0)
						{
							itemCounter++;

							newitem->width = quicklaunchItemSize;
							newitem->height = quicklaunchItemSize;
							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							if (quicklaunchMode == 1)
							{
								if (itemCounter%quicklaunchWrapCount != 0)
								{
									nextXPosition = nextXPosition+newitem->width+xSpacing;
									nextYPosition = nextYPosition;
								}
								else
								{
									nextXPosition = leftBorder;
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
							}
							else if (quicklaunchMode == 2)
							{
								if (itemCounter%quicklaunchWrapCount != 0)
								{
									nextXPosition = nextXPosition;
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
								else
								{
									nextXPosition = nextXPosition+newitem->width+xSpacing;
									nextYPosition = topBorder;
								}
							}
						}
						else
						{
							newitem->x = nextXPosition;
							newitem->y = nextYPosition;
							newitem->height = folderHeight;
										
							if (nextYPosition+newitem->height+bottomBorder > maxHeight)
							{
								if (autoMenuBreak)
								{
									if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
										adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

									nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
									nextYPosition = topBorder;

									newitem->x = nextXPosition;
									newitem->y = nextYPosition;

									menuBreaks++;

									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
								else
								{
									if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
										adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
										
									nextYPosition = topBorder;

									newitem->y = nextYPosition;

									hiddenPart++;
										
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
							}
							else
							{
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
								
						newitem->hiddenPart = hiddenPart;
						newitem->multiPart = menuBreaks;

						//Folders MUSTN'T have Quotes
						//Scan and process Piped Folders
						bool newIteration = true;
						CString worker = (*sit).c_str();
						while (newIteration == true)
						{
							if (worker.Find("|") != -1)
							{
								newitem->pipedList.insert(newitem->pipedList.end(), (string)worker.Left(worker.Find("|")));
								worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
							}
							else
							{
								newitem->pipedList.insert(newitem->pipedList.end(), (string)worker);
								newIteration = false;
							}
						}

						newitem->action = (*sit);
					
						if ( DefaultFolderIcon.empty() )
							newitem->iconPath = "*";
						else
							newitem->iconPath = DefaultFolderIcon;

						POINT tmp = buttonfont->measure(newitem->caption);
						
						if (maxNeededWidth < tmp.x)
							maxNeededWidth = tmp.x;

						entrylist.insert(entrylist.end(), newitem);
					}

					for (sit = files.begin(); sit != files.end(); ++sit)
					{
						ENTRY* newitem = new ENTRY;
						
						easyHelper = (*sit).c_str();
						easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);
						if (easyHelper.ReverseFind('.') != -1 && !showExtensions)
						{
							easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
						}
						else if (easyHelper.ReverseFind('.') != -1 && noshowExtensions.size() != 0)
						{
							CString help1;
							CString help2;

							for (StringListIterator isit = noshowExtensions.begin(); isit != noshowExtensions.end(); ++isit)
							{
								help1 = easyHelper;
								help2 = (*isit).c_str();
								help1.MakeLower();
								help2.MakeLower();

								if ( help1.Find(help2) >= help1.GetLength()-5 )
								{
									easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
									break;
								}
							}
						}

						//Remove Custom Sort HelperStrings
						if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
							easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
						//--------------------------------
						newitem->caption = easyHelper;

						ScanKeyCodes(newitem, (string) easyHelper);
					
						if (quicklaunchMode != 0)
						{
							itemCounter++;

							newitem->width = quicklaunchItemSize;
							newitem->height = quicklaunchItemSize;
							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							if (quicklaunchMode == 1)
							{
								if (itemCounter%quicklaunchWrapCount != 0)
								{
									nextXPosition = nextXPosition+newitem->width+xSpacing;
									nextYPosition = nextYPosition;
								}
								else
								{
									nextXPosition = leftBorder;
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
							}
							else if (quicklaunchMode == 2)
							{
								if (itemCounter%quicklaunchWrapCount != 0)
								{
									nextXPosition = nextXPosition;
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
								else
								{
									nextXPosition = nextXPosition+newitem->width+xSpacing;
									nextYPosition = topBorder;
								}
							}
						}
						else
						{
							newitem->x = nextXPosition;
							newitem->y = nextYPosition;
							newitem->height = entryHeight;
										
							if (nextYPosition+newitem->height+bottomBorder > maxHeight)
							{
								if (autoMenuBreak)
								{
									if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
										adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

									nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
									nextYPosition = topBorder;

									newitem->x = nextXPosition;
									newitem->y = nextYPosition;

									menuBreaks++;

									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
								else
								{
									if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
										adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
											
									nextYPosition = topBorder;

									newitem->y = nextYPosition;

									hiddenPart++;
										
									nextYPosition = nextYPosition+newitem->height+ySpacing;
								}
							}
							else
							{
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
										
						newitem->hiddenPart = hiddenPart;
						newitem->multiPart = menuBreaks;

						//Files MUST have Quotes
						newitem->action = ("\"" + (*sit) + "\"");

						newitem->iconPath = (*sit);

						POINT tmp = buttonfont->measure(newitem->caption);
						
						if (maxNeededWidth < tmp.x)
							maxNeededWidth = tmp.x;

						entrylist.insert(entrylist.end(), newitem);
					}
				}
			}
			//-----------------------------------------------------
			// DYNAMIC POPUPFOLDERS
			//-----------------------------------------------------
			else if ( _stricmp(str_third.Left(19), "!popupdynamicfolder") == NULL || _stricmp(str_third.Left(14), "!dynamicfolder") == NULL || _stricmp(str_third.Left(12), "!popupfolder") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					newitem->entryType = -1;
					if ( _stricmp(str_third.Left(19), "!popupdynamicfolder") == NULL || _stricmp(str_third.Left(14), "!dynamicfolder") == NULL )
						newitem->entryType = -2;

					if (quicklaunchMode != 0)
					{
						itemCounter++;

						newitem->width = quicklaunchItemSize;
						newitem->height = quicklaunchItemSize;
						newitem->x = nextXPosition;
						newitem->y = nextYPosition;

						if (quicklaunchMode == 1)
						{
							if (itemCounter%quicklaunchWrapCount != 0)
							{
								nextXPosition = nextXPosition+newitem->width+xSpacing;
								nextYPosition = nextYPosition;
							}
							else
							{
								nextXPosition = leftBorder;
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else if (quicklaunchMode == 2)
						{
							if (itemCounter%quicklaunchWrapCount != 0)
							{
								nextXPosition = nextXPosition;
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								nextXPosition = nextXPosition+newitem->width+xSpacing;
								nextYPosition = topBorder;
							}
						}
					}
					else
					{
						newitem->x = nextXPosition;
						newitem->y = nextYPosition;
						newitem->height = folderHeight;
					
						if (nextYPosition+newitem->height+bottomBorder > maxHeight)
						{
							if (autoMenuBreak)
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

								nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
								nextYPosition = topBorder;

								newitem->x = nextXPosition;
								newitem->y = nextYPosition;

								menuBreaks++;

								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
								
								nextYPosition = topBorder;

								newitem->y = nextYPosition;

								hiddenPart++;
								
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else
						{
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}

					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					if ( tooltipMode != 0 )
					{
						if ( strstr(str_second, "||") != NULL )
						{
							newitem->caption = str_second.Left(str_second.Find("||"));
							if ( strstr(str_second, "\\n") != NULL )
								str_second.Replace("\\n", "\n");
							newitem->tooltip = str_second.Right(str_second.GetLength()-str_second.Find("||")-2);
						}
						else
						{
							newitem->caption = str_second;
						}
						
					}
					else
					{
						if ( strstr(str_second, "||") != NULL )
						{
							easyHelper = str_second.Left(str_second.Find("||"));
							newitem->caption = easyHelper;
						}
						else
						{
							newitem->caption = str_second;
						}
					}

					ScanKeyCodes(newitem, (string) str_second);
									
					easyHelper = str_third;
					
					// Get FolderPath
					easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.Find(":")-1);
					easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();

					//Scan and process Piped Folders
					bool newIteration = true;
					CString worker = easyHelper;
					while (newIteration == true)
					{
						if (worker.Find("|") != -1)
						{
							newitem->pipedList.insert(newitem->pipedList.end(), (string)worker.Left(worker.Find("|")));
							worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
						}
						else
						{
							newitem->pipedList.insert(newitem->pipedList.end(), (string)worker);
							newIteration = false;
						}
					}

					newitem->action = easyHelper;

					if ( DefaultFolderIcon.empty() )
					{
						if (customIcon)
							newitem->iconPath = str_first;
						else
							newitem->iconPath = "*";
					}
					else
					{
						if (customIcon)
							newitem->iconPath = str_first;
						else
							newitem->iconPath = DefaultFolderIcon;
					}

					newitem->layout = (string) str_fourth;
					newitem->defaultlayout = (string) str_fifth;

					POINT tmp = buttonfont->measure(newitem->caption);
					
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// DYNAMIC ACTION POPUPFOLDERS
			//-----------------------------------------------------
			else if ( _stricmp(str_third.Left(25), "!popupdynamicactionfolder") == NULL || _stricmp(str_third.Left(18), "!popupactionfolder") == NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					newitem->entryType = -1;
					if ( _stricmp(str_third.Left(25), "!popupdynamicactionfolder") == NULL )
						newitem->entryType = -2;

					if (quicklaunchMode != 0)
					{
						itemCounter++;

						newitem->width = quicklaunchItemSize;
						newitem->height = quicklaunchItemSize;
						newitem->x = nextXPosition;
						newitem->y = nextYPosition;

						if (quicklaunchMode == 1)
						{
							if (itemCounter%quicklaunchWrapCount != 0)
							{
								nextXPosition = nextXPosition+newitem->width+xSpacing;
								nextYPosition = nextYPosition;
							}
							else
							{
								nextXPosition = leftBorder;
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else if (quicklaunchMode == 2)
						{
							if (itemCounter%quicklaunchWrapCount != 0)
							{
								nextXPosition = nextXPosition;
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								nextXPosition = nextXPosition+newitem->width+xSpacing;
								nextYPosition = topBorder;
							}
						}
					}
					else
					{
						newitem->x = nextXPosition;
						newitem->y = nextYPosition;
						newitem->height = folderHeight;
					
						if (nextYPosition+newitem->height+bottomBorder > maxHeight)
						{
							if (autoMenuBreak)
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

								nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
								nextYPosition = topBorder;

								newitem->x = nextXPosition;
								newitem->y = nextYPosition;

								menuBreaks++;

								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
								
								nextYPosition = topBorder;

								newitem->y = nextYPosition;

								hiddenPart++;
								
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else
						{
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}

					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					if ( tooltipMode != 0 )
					{
						if ( strstr(str_second, "||") != NULL )
						{
							newitem->caption = str_second.Left(str_second.Find("||"));
							if ( strstr(str_second, "\\n") != NULL )
								str_second.Replace("\\n", "\n");
							newitem->tooltip = str_second.Right(str_second.GetLength()-str_second.Find("||")-2);
						}
						else
						{
							newitem->caption = str_second;
						}
					}
					else
					{
						if ( strstr(str_second, "||") != NULL )
						{
							easyHelper = str_second.Left(str_second.Find("||"));
							newitem->caption = easyHelper;
						}
						else
						{
							newitem->caption = str_second;
						}
					}

					ScanKeyCodes(newitem, (string) str_second);
									
					easyHelper = str_third;
					
					// Get FolderPath (Strip !action between ":" ... ":"
					int pos = easyHelper.Find(":");
					if ( easyHelper.Find(":", pos+3) == -1 )
					{
						delete newitem;
						newitem = NULL;
						continue;
					}

					//Find 2nd ":" from backwards
					int sep2 = easyHelper.ReverseFind(':');
					sep2 = easyHelper.Find(":", sep2-3); // ...:c:\path

					// Get universalAction
					newitem->universalaction = easyHelper.Mid(pos+1, sep2-pos-1);

					easyHelper = easyHelper.Right(easyHelper.GetLength()-sep2-1);
					
					easyHelper = TrimAndStripQuote( (string)easyHelper ).c_str();

					//Scan and process Piped Folders
					bool newIteration = true;
					CString worker = easyHelper;
					while (newIteration == true)
					{
						if (worker.Find("|") != -1)
						{
							newitem->pipedList.insert(newitem->pipedList.end(), (string)worker.Left(worker.Find("|")));
							worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
						}
						else
						{
							newitem->pipedList.insert(newitem->pipedList.end(), (string)worker);
							newIteration = false;
						}
					}

					newitem->action = easyHelper;

					if ( DefaultFolderIcon.empty() )
					{
						if (customIcon)
							newitem->iconPath = str_first;
						else
							newitem->iconPath = "*";
					}
					else
					{
						if (customIcon)
							newitem->iconPath = str_first;
						else
							newitem->iconPath = DefaultFolderIcon;
					}

					newitem->layout = (string) str_fourth;
					newitem->defaultlayout = (string) str_fifth;

					POINT tmp = buttonfont->measure(newitem->caption);
					
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// POPUPTASKS
			//-----------------------------------------------------
			else if ( _stricmp(str_third, "!popuptasks") == NULL )
			{
				if ( quicklaunchMode != 0 )
					continue;

				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					newitem->hWnd = hWnd;

					newitem->entryType = -1;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = folderHeight;
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
					{
						nextYPosition = nextYPosition+newitem->height+ySpacing;
					}
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					if ( tooltipMode != 0 )
					{
						if ( strstr(str_second, "||") != NULL )
						{
							newitem->caption = str_second.Left(str_second.Find("||"));
							if ( strstr(str_second, "\\n") != NULL )
								str_second.Replace("\\n", "\n");
							newitem->tooltip = str_second.Right(str_second.GetLength()-str_second.Find("||")-2);
						}
						else
						{
							newitem->caption = str_second;
						}
						
					}
					else
					{
						if ( strstr(str_second, "||") != NULL )
						{
							easyHelper = str_second.Left(str_second.Find("||"));
							newitem->caption = easyHelper;
						}
						else
						{
							newitem->caption = str_second;
						}
					}

					ScanKeyCodes(newitem, (string) str_second);
			
					if (customIcon)
						easyHelper = str_first;
					else
						easyHelper = DefaultFolderIcon.c_str();

					newitem->iconPath = easyHelper;

					newitem->layout = (string) str_fourth;
					newitem->defaultlayout = (string) str_fifth;

					POINT tmp = buttonfont->measure(newitem->caption);
					
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// POPUP SHELLFOLDERS
			//-----------------------------------------------------
			else if ( _stricmp(str_third, "!popupcontrolpanel") == NULL || _stricmp(str_third, "!popupprinters") == NULL 
						|| _stricmp(str_third, "!popuprecyclebin") == NULL || _stricmp(str_third, "!popuprecentdocuments") == NULL
						|| _stricmp(str_third, "!popupnetwork") == NULL || _stricmp(str_third, "!popupmycomputer") == NULL )
			{
				if ( quicklaunchMode != 0 )
					continue;

				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{	
					ENTRY* newitem = new ENTRY;

					if ( _stricmp(str_third, "!popupcontrolpanel") == NULL )
						newitem->clsid = CSIDL_CONTROLS;
					else if (_stricmp(str_third, "!popupmycomputer") == NULL )
						newitem->clsid = CSIDL_DRIVES;
					else if (_stricmp(str_third, "!popupnetwork") == NULL )
						newitem->clsid = CSIDL_NETWORK;
					else if (_stricmp(str_third, "!popupprinters") == NULL )
						newitem->clsid = CSIDL_PRINTERS;
					else if (_stricmp(str_third, "!popuprecentdocuments") == NULL )
						newitem->clsid = CSIDL_RECENT;
					else if (_stricmp(str_third, "!popuprecyclebin") == NULL )
						newitem->clsid = CSIDL_BITBUCKET;
					else
					{
						delete newitem;
						continue;
					}

					newitem->entryType = -1;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;
					newitem->height = entryHeight;

					//Shell Folder PIDL for Icon Extraction
					SHGetSpecialFolderLocation(NULL, newitem->clsid, &newitem->m_pidl);
					
					if (nextYPosition+newitem->height+bottomBorder > maxHeight)
					{
						if (autoMenuBreak)
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

							nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
							nextYPosition = topBorder;

							newitem->x = nextXPosition;
							newitem->y = nextYPosition;

							menuBreaks++;

							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
						else
						{
							if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
								adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
							nextYPosition = topBorder;

							newitem->y = nextYPosition;

							hiddenPart++;
							
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
					else
					{
						nextYPosition = nextYPosition+newitem->height+ySpacing;
					}
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					if ( tooltipMode != 0 )
					{
						if ( strstr(str_second, "||") != NULL )
						{
							newitem->caption = str_second.Left(str_second.Find("||"));
							if ( strstr(str_second, "\\n") != NULL )
								str_second.Replace("\\n", "\n");
							newitem->tooltip = str_second.Right(str_second.GetLength()-str_second.Find("||")-2);
						}
						else
						{
							newitem->caption = str_second;
						}
						
					}
					else
					{
						if ( strstr(str_second, "||") != NULL )
						{
							easyHelper = str_second.Left(str_second.Find("||"));
							newitem->caption = easyHelper;
						}
						else
						{
							newitem->caption = str_second;
						}
					}

					ScanKeyCodes(newitem, (string) str_second);
			
					if (customIcon)
						newitem->iconPath = str_first;

					newitem->layout = (string) str_fourth;
					newitem->defaultlayout = (string) str_fifth;

					POINT tmp = buttonfont->measure(newitem->caption);
					
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
			//-----------------------------------------------------
			// NORMAL ENTRYS
			//-----------------------------------------------------
			else if ( _stricmp(str_second, "!popupimage") != NULL && _stricmp(str_third, "!popuprun") != NULL
					 && _stricmp(str_third, "!datetime") != NULL && _stricmp(str_second, "!remoteamp") != NULL 
					 && _stricmp(str_third, "!popupdesktops") != NULL )
			{
				if (input)
				{
					//Add to specific Popup Settings list
					sendToPopup.insert(sendToPopup.end(), *sit);	
				}
				else
				{
					ENTRY* newitem = new ENTRY;

					if (quicklaunchMode != 0)
					{
						itemCounter++;

						newitem->tooltip = str_second;

						newitem->width = quicklaunchItemSize;
						newitem->height = quicklaunchItemSize;
						newitem->x = nextXPosition;
						newitem->y = nextYPosition;

						if (quicklaunchMode == 1)
						{
							if (itemCounter%quicklaunchWrapCount != 0)
							{
								nextXPosition = nextXPosition+newitem->width+xSpacing;
								nextYPosition = nextYPosition;
							}
							else
							{
								nextXPosition = leftBorder;
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else if (quicklaunchMode == 2)
						{
							if (itemCounter%quicklaunchWrapCount != 0)
							{
								nextXPosition = nextXPosition;
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								nextXPosition = nextXPosition+newitem->width+xSpacing;
								nextYPosition = topBorder;
							}
						}
					}
					else
					{
						newitem->x = nextXPosition;
						newitem->y = nextYPosition;
						newitem->height = entryHeight;
						
						if (nextYPosition+newitem->height+bottomBorder > maxHeight)
						{
							if (autoMenuBreak)
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

								nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
								nextYPosition = topBorder;

								newitem->x = nextXPosition;
								newitem->y = nextYPosition;

								menuBreaks++;

								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
							else
							{
								if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
									adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
								
								nextYPosition = topBorder;

								newitem->y = nextYPosition;

								hiddenPart++;
								
								nextYPosition = nextYPosition+newitem->height+ySpacing;
							}
						}
						else
						{
							nextYPosition = nextYPosition+newitem->height+ySpacing;
						}
					}
						
					newitem->hiddenPart = hiddenPart;
					newitem->multiPart = menuBreaks;

					if ( tooltipMode != 0 )
					{
						if ( strstr(str_second, "||") != NULL )
						{
							newitem->caption = str_second.Left(str_second.Find("||"));
							if ( strstr(str_second, "\\n") != NULL )
								str_second.Replace("\\n", "\n");
							newitem->tooltip = str_second.Right(str_second.GetLength()-str_second.Find("||")-2);
						}
						else
						{
							newitem->caption = str_second;
						}
						
					}
					else
					{
						if ( strstr(str_second, "||") != NULL )
						{
							easyHelper = str_second.Left(str_second.Find("||"));
							newitem->caption = easyHelper;
						}
						else
						{
							newitem->caption = str_second;
						}
					}

					ScanKeyCodes(newitem, (string) str_second);
		
					if ( PathFileExists(str_third) )
						newitem->action = ("\"" + str_third + "\"");
					else
						newitem->action = str_third;

					if (customIcon)
						easyHelper = str_first;
					else
						easyHelper = TrimAndStripQuote( newitem->action ).c_str();

					if ( customIcon )
						newitem->iconPath = easyHelper;
					else if ( easyHelper.Left(1) != "!" )
						newitem->iconPath = easyHelper;
					else 
						newitem->iconPath = DefaultIcon;

					POINT tmp = buttonfont->measure(newitem->caption);
					
					if (maxNeededWidth < tmp.x)
						maxNeededWidth = tmp.x;

					entrylist.insert(entrylist.end(), newitem);
				}
			}
		}

		if (quicklaunchMode != 0)
		{
			int tmpCount = 0;
			if (itemCounter < quicklaunchWrapCount)
				tmpCount = itemCounter;
			else
				tmpCount = quicklaunchWrapCount;

			if (quicklaunchMode == 1)
			{
				actualwidth = leftBorder+tmpCount*quicklaunchItemSize+(tmpCount-1)*xSpacing+rightBorder;
				actualheight = nextYPosition+((itemCounter%quicklaunchWrapCount != 0) ? quicklaunchItemSize : 0)+bottomBorder;
			}
			else if (quicklaunchMode == 2)
			{
				actualwidth = nextXPosition+((itemCounter%quicklaunchWrapCount != 0) ? quicklaunchItemSize : 0)+rightBorder;
				actualheight = topBorder+tmpCount*quicklaunchItemSize+(tmpCount-1)*ySpacing+bottomBorder;
			}
		}
		else
		{
			//Check Max/Min EntryWidth
			maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);

			// Check Last MenuColumn for Height
			if (adaptedmaxHeight != 0 && adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
				adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

			//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
			POINT tmp = buttonfont->measure(captionname, 1);
			int neededTitleWidth = leftBorder+tmp.x+rightBorder;

			//Adapt EntryWidth based on Max/Min Width
			if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
			{
				actualwidth = maxWidth;
				maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
			}
			else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
			{
				actualwidth = minWidth;
				maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
			}
			else
			{
				actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
				actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
			}

			if (hiddenPart != 0 || menuBreaks != 0)
				actualheight = adaptedmaxHeight;
			else
				actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);
			actualheight = max(actualheight, minHeight);

			//Hook Positioning
			//----------------------------------------------
			string x = "";
			string y = "";
			string space = "";
		
			for (sit = hookcontentList.begin(); sit != hookcontentList.end(); ++sit)
			{
				x = "0";
				y = "0";
				space = "0";
				easyHelper = (*sit).c_str();
				y = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind(' '));
				easyHelper = easyHelper.Left(easyHelper.ReverseFind(' '));
				x = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind(' '));
				easyHelper = easyHelper.Left(easyHelper.ReverseFind(' '));
				space = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind(' '));
				easyHelper = easyHelper.Left(easyHelper.ReverseFind(' '));
				
				int iterate = leftBorder;
				while ( atoi(x.c_str()) > iterate )
					iterate = iterate+maxNeededWidth+xSpacing;

				x = IntToString(iterate+atoi(space.c_str()));

				easyHelper += " ";
				easyHelper += x.c_str();
				easyHelper += " ";
				easyHelper += y.c_str();

				LSExecute(NULL, easyHelper, SW_SHOWNORMAL);
			}
		}

		hookcontentList.clear();
		//----------------------------------------------

		startentry = NULL;

		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		{
			if (startentry == NULL && (*eit)->entryType <= 0)
				startentry = *eit;

			if (quicklaunchMode == 0)
			{
				if (menuBreaks == 0)
				{
					(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
				}
				else
				{
					(*eit)->width = maxNeededWidth;
					
					(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);
				}
			}

			if ( subFolderLevel <= cacheLevel )
			{
				if ( (*eit)->entryType < 0 && (*eit)->clsid == 0 && (*eit)->hWnd == NULL )
					CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit), false, true );
				else if ( (*eit)->clsid != 0 && (*eit)->hWnd == NULL )
					CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit)->clsid, (*eit), false, true );
			}
		}

		sendToPopup.clear();

		maxPart = hiddenPart;
		maxMenuBreak = menuBreaks;
	}

	reposition(2, 0, 0, actualwidth, actualheight);
	repaint(true);
	
	settingsList.clear();

	if ( !hasDynamicContent )
	{
		popupcontentList.clear();
		str_completeCheckStringBackup = "";
	}

	KillTimer(hWnd, TIMER_UPDATE);

	if ((isTaskPopup || hasDynamicContent) && !hasInfoEntry)
		SetTimer(hWnd, TIMER_UPDATE, tasksUpdate, NULL);
	else if (hasInfoEntry)
		SetTimer(hWnd, TIMER_UPDATE, 1000, NULL);
}

void Popup::ScanSubMenu(string scanPath, int dynamic, string universalAction)
{
	bool forcedDriveRescan = true;
	bool NotifyUpdate = false;

	if ( scanPath.empty() && dynamic == -100 && universalAction.empty() )
	{
		scanPath = rootScanPath;
		dynamic = -2;
		universalAction = rootUniversalAction;
		
		forcedDriveRescan = true;
		NotifyUpdate = true;
	}
	
	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{	
		delete *eit;
		*eit = NULL;
	}

	entrylist.clear();

	for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
	{
		(*plit)->hide();
		delete *plit;
		*plit = NULL;
	}

	subpopupList.clear();

	//Reset old Pointers
	//-------------------------
	currentPart = 0;

	startentry = NULL;
	lastactiveentry = NULL;
	hoverentry = NULL;
	//-------------------------

	CString easyHelper = "";

	//---------------------------------------------------------------------------------------
	// START Scanning
	//---------------------------------------------------------------------------------------
	string searchPath = "";
	string wildcardPath = "";
	string itemName = "";
	string tmpHelper = "";
	
	//BOOL for only FolderMatching ActionFolder
	bool actionFolder_FolderList = false;
	
	StringList folders;
	StringList files;

	/*CString cacheHelper = "";
	int cacheCounter = 0;

	char themepath[MAX_LINE_LENGTH] = {0};
	VarExpansion( themepath, "$themedir$" );
	strcat(themepath, "xpopupcache.ini");

	if ( !forcedDriveRescan )
	{
		if ( pParentPopup == NULL )
		{
			cacheHelper = "";
		}
		else
		{
			cacheHelper = pParentPopup->captionname.c_str();
			cacheHelper += "-";
		}
		cacheHelper += scanPath.c_str();

		cacheCounter = (int) GetPrivateProfileInt(cacheHelper, "EntryCount", 0, themepath);
		
		if ( cacheCounter != 0 )
		{
			char buffer[MAX_LINE_LENGTH] = {0};
			int pos = -1;
			for (int i = 1; i<=cacheCounter; i++)
			{
				GetPrivateProfileString(cacheHelper, IntToString( i ).c_str(), "", buffer, MAX_LINE_LENGTH, themepath);
				easyHelper = buffer;
				pos = easyHelper.Find("Folder?");
				if ( pos != -1 )
				{
					easyHelper = easyHelper.Right(easyHelper.GetLength()-7);
					folders.insert(folders.end(), (string)easyHelper);
				}
				else
				{
					files.insert(files.end(), (string)easyHelper);
				}
			}
		}
		else
		{
			forcedDriveRescan = true;
		}
	}*/
	
	//---------------------------------------------------------------------
	if ( forcedDriveRescan )
	{
	//---------------------------------------------------------------------
		
		HANDLE hFindHandle;
		WIN32_FIND_DATA FindData;

		bool newIteration = true;
		CString worker = scanPath.c_str();

		while (newIteration == true)
		{
			if (worker.Find("|") != -1)
			{
				searchPath = worker.Left(worker.Find("|"));
				worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
			}
			else
			{
				searchPath = worker;
				newIteration = false;
			}

			//Check for ending BackSlash
			tmpHelper = searchPath.substr(searchPath.length()-1, searchPath.length());
			bool addBackslash = false;
			
			if ( !universalAction.empty() && (strstr(searchPath.c_str(), "*\\") != NULL || strstr(searchPath.c_str(), "*/") != NULL) )
			{
				searchPath = searchPath.substr(0, searchPath.length()-2);
				wildcardPath = (searchPath + "*.*");
				actionFolder_FolderList = true;
			}
			else if ( tmpHelper != "\\" && tmpHelper != "/" && strstr(searchPath.c_str(), "*.") == NULL)
			{
				wildcardPath = (searchPath + "\\*.*");
				addBackslash = true;
			}
			else if (strstr(searchPath.c_str(), "*.") == NULL)
			{
				wildcardPath = (searchPath + "*.*");
			}
			else
			{
				wildcardPath = searchPath;
				searchPath = searchPath.substr(0, searchPath.find('*'));
			}

			hFindHandle = FindFirstFile(wildcardPath.c_str(), &FindData);
			
			for(;;)
			{
				if(INVALID_HANDLE_VALUE == hFindHandle)
					break;

				itemName = FindData.cFileName;

				//It is a folder
				if (_stricmp(itemName.c_str(), ".") != NULL && _stricmp(itemName.c_str(), "..") != NULL && !(FindData.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN))
				{
					if (addBackslash)
						tmpHelper = (searchPath + "\\" + itemName);
					else
						tmpHelper = (searchPath + itemName);
					
					if (FindData.dwFileAttributes &  FILE_ATTRIBUTE_DIRECTORY)
					{
						//Check for double folders and unite
						CString helper = "";
						bool found = false;
						for (StringListIterator sit = folders.begin(); sit != folders.end(); ++sit)
						{
							easyHelper = (*sit).c_str();
							
							if (_stricmp( itemName.c_str(), easyHelper.Right( itemName.length() ) ) == NULL)
							{
								easyHelper += "|";
								easyHelper += tmpHelper.c_str();
								(*sit) = (string)easyHelper;

								found = true;
								break;
							}
						}
						
						if ( !found )
						{
							folders.insert(folders.end(), tmpHelper);
						}
					}
					else if ( !actionFolder_FolderList )
					{
						files.insert(files.end(), tmpHelper);
					}
				}

				if(!FindNextFile(hFindHandle, &FindData))
					break;
			}
		}

		FindClose(hFindHandle);

	//---------------------------------------------------------------------
	}
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------------------------
	// END Scanning
	//---------------------------------------------------------------------------------------

	//No SubFolders in !popupactionfolder
	if ( !universalAction.empty() )
	{
		if ( !actionFolder_FolderList )
		{
			folders.clear();
		}
		else 
		{
			files = folders;
			folders.clear();
		}
	}
	
	//Sorting
	Sort(folders);
	Sort(files);

	//Initializing of Values
	int itemCounter = 0;
	int nextXPosition = leftBorder;
	int nextYPosition = topBorder;
	int hiddenPart = 0;
	int menuBreaks = 0;
	int maxNeededWidth = 0;
	int adaptedmaxHeight = 0;

	//Insert AutoSeparator if wished between Title and Entrys
	if (autoSeparator >= 2 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		if ( nextYPosition != topBorder )
		{
			newitem->y = nextYPosition+separatorySpacing;
			nextYPosition = newitem->y;
		}
		else
		{
			newitem->y = nextYPosition;
		}
			
		newitem->height = separatorHeight;
			
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
							
				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	StringListIterator sit;

	for (sit = folders.begin(); sit != folders.end(); ++sit)
	{
		itemCounter++;

		ENTRY* newitem = new ENTRY;
		
		easyHelper = (*sit).c_str();
		easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);

		//Remove Custom Sort HelperStrings
		if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
			easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
		//--------------------------------
		newitem->caption = easyHelper;

		ScanKeyCodes(newitem, (string) easyHelper);
			
		newitem->entryType = -1;
		if (dynamic == -2)
			newitem->entryType = -2;

		if (quicklaunchMode != 0)
		{
			newitem->width = quicklaunchItemSize;
			newitem->height = quicklaunchItemSize;
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			if (quicklaunchMode == 1)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = nextYPosition;
				}
				else
				{
					nextXPosition = leftBorder;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else if (quicklaunchMode == 2)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = topBorder;
				}
			}
		}
		else
		{
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;
			newitem->height = folderHeight;
					
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
					
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
					
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
			{
				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
		}
				
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		//Folders MUSTN'T have Quotes
		//Scan and process Piped Folders
		bool newIteration = true;
		CString worker = (*sit).c_str();
		while (newIteration == true)
		{
			if (worker.Find("|") != -1)
			{
				newitem->pipedList.insert(newitem->pipedList.end(), (string)worker.Left(worker.Find("|")));
				worker = worker.Right(worker.GetLength()-worker.Find("|")-1);
			}
			else
			{
				newitem->pipedList.insert(newitem->pipedList.end(), (string)worker);
				newIteration = false;
			}
		}

		newitem->action = (*sit);
	
		if ( DefaultFolderIcon.empty() )
			newitem->iconPath = "*";
		else
			newitem->iconPath = DefaultFolderIcon;

		POINT tmp = buttonfont->measure(newitem->caption);

		if (maxNeededWidth < tmp.x)
			maxNeededWidth = tmp.x;

		entrylist.insert(entrylist.end(), newitem);
	}

	//Insert AutoSeparator if wished between Folder and Files
	if (autoSeparator >= 1 && folders.size() >= 1 && files.size() >= 1 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		if ( nextYPosition != topBorder )
		{
			newitem->y = nextYPosition+separatorySpacing;
			nextYPosition = newitem->y;
		}
		else
		{
			newitem->y = nextYPosition;
		}
					
		newitem->height = separatorHeight;
					
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
							
				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	for (sit = files.begin(); sit != files.end(); ++sit)
	{
		itemCounter++;

		ENTRY* newitem = new ENTRY;
		
		easyHelper = (*sit).c_str();
		easyHelper = easyHelper.Right(easyHelper.GetLength()-easyHelper.ReverseFind('\\')-1);
		if (easyHelper.ReverseFind('.') != -1 && !showExtensions)
		{
			easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
		}
		else if (easyHelper.ReverseFind('.') != -1 && noshowExtensions.size() != 0)
		{
			CString help1;
			CString help2;

			for (StringListIterator isit = noshowExtensions.begin(); isit != noshowExtensions.end(); ++isit)
			{
				help1 = easyHelper;
				help2 = (*isit).c_str();
				help1.MakeLower();
				help2.MakeLower();

				if ( help1.Find(help2) >= help1.GetLength()-5 )
				{
					easyHelper = easyHelper.Left(easyHelper.ReverseFind('.'));
					break;
				}
			}
		}

		//Remove Custom Sort HelperStrings
		if ( easyHelper.Find("[") == 0 && easyHelper.Find("]") == 3 )
			easyHelper = easyHelper.Right(easyHelper.GetLength()-4);
		//--------------------------------
		newitem->caption = easyHelper;

		ScanKeyCodes(newitem, (string) easyHelper);

		if (quicklaunchMode != 0)
		{
			newitem->width = quicklaunchItemSize;
			newitem->height = quicklaunchItemSize;
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			if (quicklaunchMode == 1)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = nextYPosition;
				}
				else
				{
					nextXPosition = leftBorder;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else if (quicklaunchMode == 2)
			{
				if (itemCounter%quicklaunchWrapCount != 0)
				{
					nextXPosition = nextXPosition;
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					nextXPosition = nextXPosition+newitem->width+xSpacing;
					nextYPosition = topBorder;
				}
			}
		}
		else
		{
			newitem->x = nextXPosition;
			newitem->y = nextYPosition;
			newitem->height = entryHeight;
				
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
						
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
					
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
			{
				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		//Files MUST have Quotes
		if ( !universalAction.empty() )
			newitem->universalaction = (universalAction + " \"" + (*sit) + "\"");
		
		newitem->action = ("\"" + (*sit) + "\"");

		newitem->iconPath = (*sit);

		POINT tmp = buttonfont->measure(newitem->caption);

		if (maxNeededWidth < tmp.x)
			maxNeededWidth = tmp.x;

		entrylist.insert(entrylist.end(), newitem);
	}

	//Insert AutoSeparator if wished at the Bottom
	if (autoSeparator == 3 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		if ( nextYPosition != topBorder )
		{
			newitem->y = nextYPosition+separatorySpacing;
			nextYPosition = newitem->y;
		}
		else
		{
			newitem->y = nextYPosition;
		}
					
		newitem->height = separatorHeight;
					
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
							
				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	if (quicklaunchMode != 0)
	{
		int tmpCount = 0;
		if (itemCounter < quicklaunchWrapCount)
			tmpCount = itemCounter;
		else
			tmpCount = quicklaunchWrapCount;

		if (quicklaunchMode == 1)
		{
			actualwidth = leftBorder+tmpCount*quicklaunchItemSize+(tmpCount-1)*xSpacing+rightBorder;
			actualheight = nextYPosition+((itemCounter%quicklaunchWrapCount != 0) ? quicklaunchItemSize : 0)+bottomBorder;
		}
		else if (quicklaunchMode == 2)
		{
			actualwidth = nextXPosition+((itemCounter%quicklaunchWrapCount != 0) ? quicklaunchItemSize : 0)+rightBorder;
			actualheight = topBorder+tmpCount*quicklaunchItemSize+(tmpCount-1)*ySpacing+bottomBorder;
		}
	}
	else
	{
		//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
		POINT tmp = buttonfont->measure(captionname, 1);
		int neededTitleWidth = leftBorder+tmp.x+rightBorder;

		//Check Max/Min EntryWidth
		maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);

		//Adapt EntryWidth based on Max/Min Width
		if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
		{
			actualwidth = maxWidth;
			maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
		}
		else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
		{
			actualwidth = minWidth;
			maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
		}
		else
		{
			actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
			actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
		}

		if (hiddenPart != 0 || menuBreaks != 0)
			actualheight = adaptedmaxHeight;
		else
			actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);

		actualheight = max(actualheight, minHeight);
	}

	//Reset Helper Variables
	//cacheHelper = "";
	//cacheCounter = 1;

	startentry = NULL;
	
	for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (startentry == NULL && (*eit)->entryType <= 0)
			startentry = *eit;

		/*if ( forcedDriveRescan && (*eit)->entryType <= 0 )
		{
			if ( pParentPopup == NULL )
			{
				cacheHelper = "";
			}
			else
			{
				cacheHelper = pParentPopup->captionname.c_str();
				cacheHelper += "-";
			}
			cacheHelper += pParentEntry->action.c_str();
			if ( (*eit)->entryType < 0 ) //Folder
				WritePrivateProfileString(cacheHelper, IntToString( cacheCounter ).c_str(), ("Folder?" + TrimAndStripQuote((*eit)->action)).c_str(), themepath);
			else
				WritePrivateProfileString(cacheHelper, IntToString( cacheCounter ).c_str(), TrimAndStripQuote((*eit)->action).c_str(), themepath);

			cacheCounter++;
		}*/

		if (quicklaunchMode == 0)
		{
			if (menuBreaks == 0)
			{
				(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
			}
			else
			{
				(*eit)->width = maxNeededWidth;
				
				(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);	
			}
		}

		if ( subFolderLevel <= cacheLevel )
		{
			if ( (*eit)->entryType < 0 && (*eit)->clsid == 0 && (*eit)->hWnd == NULL )
				CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit), false, true );
		}
	}

	/*if ( forcedDriveRescan )
	{
		WritePrivateProfileString(cacheHelper, "EntryCount", IntToString( cacheCounter-1 ).c_str(), themepath);
	}*/

	maxPart = hiddenPart;
	maxMenuBreak = menuBreaks;

	reposition(2, 0, 0, actualwidth, actualheight);
	//Important UpdateWindowTransparent crashes, don't know why !??? :|
	if (!NotifyUpdate)
		repaint(true);
	else
		resetBackground = true;
}

void Popup::ScanShellFolder(int clsid)
{
	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{	
		delete *eit;
		*eit = NULL;
	}

	entrylist.clear();

	//Reset old Pointers
	//-------------------------
	currentPart = 0;

	startentry = NULL;
	lastactiveentry = NULL;
	hoverentry = NULL;
	//-------------------------

	//SHELL PART 1
	LPITEMIDLIST m_pidlFull = NULL;
	IShellFolder *m_pshf = NULL;
	SHGetSpecialFolderLocation(NULL, clsid, &m_pidlFull);

	if (m_pidlFull)
	{
		SHFILEINFO shfi;

		SHGetFileInfo((LPSTR) m_pidlFull, 0, &shfi, sizeof(SHFILEINFO), SHGFI_ATTRIBUTES | SHGFI_DISPLAYNAME |
		                                        SHGFI_PIDL | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);

		if (shfi.dwAttributes & SFGAO_FOLDER && !(shfi.dwAttributes & SFGAO_REMOVABLE))
		{
			IShellFolder *pshfDesktop;
			SHGetDesktopFolder(&pshfDesktop);

			pshfDesktop->BindToObject(m_pidlFull, NULL, IID_IShellFolder,
			                          (void **) &m_pshf);

			pshfDesktop->Release();
		}
	}

	//---------------------------------------------------------------------------------------
	// START Scanning
	//---------------------------------------------------------------------------------------
	IEnumIDList *peidl;
	vector<LPITEMIDLIST> PIDLList;
	LPITEMIDLIST pidl;

	HRESULT hr = m_pshf->EnumObjects(NULL, SHCONTF_FOLDERS |
	                         SHCONTF_NONFOLDERS, &peidl);

	if ( SUCCEEDED(hr) )
	{
		while (peidl->Next(1, &pidl, NULL) == S_OK)
		{
			LPITEMIDLIST pidlFull = JoinIDLists(m_pidlFull, pidl);

			PIDLList.insert(PIDLList.end(), pidlFull);
		}

		peidl->Release();
	}
	//---------------------------------------------------------------------------------------
	// END Scanning
	//---------------------------------------------------------------------------------------

	//Sorting
	if (clsid != CSIDL_DRIVES && clsid != CSIDL_RECENT)
		SortShellFolder(PIDLList);

	//Initializing of Values
	int nextXPosition = leftBorder;
	int nextYPosition = topBorder;
	int hiddenPart = 0;
	int menuBreaks = 0;
	int maxNeededWidth = 0;
	int adaptedmaxHeight = 0;

	//Insert AutoSeparator if wished between Title and Entrys
	if (autoSeparator >= 2 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		if ( nextYPosition != topBorder )
		{
			newitem->y = nextYPosition+separatorySpacing;
			nextYPosition = newitem->y;
		}
		else
		{
			newitem->y = nextYPosition;
		}
					
		newitem->height = separatorHeight;
					
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
							
				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	for (vector<LPITEMIDLIST>::iterator pit = PIDLList.begin(); pit != PIDLList.end(); ++pit)
	{
		SHFILEINFO shfi;

		SHGetFileInfo((LPCSTR) (*pit), 0, &shfi, sizeof(SHFILEINFO),
										  SHGFI_ATTRIBUTES | SHGFI_DISPLAYNAME | SHGFI_PIDL );

		if (shfi.dwAttributes & SFGAO_FOLDER)
		{
			//Entry
			ENTRY* newitem = new ENTRY;

			newitem->m_pidl = (*pit);

			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			newitem->caption = shfi.szDisplayName;

			ScanKeyCodes(newitem, (string) shfi.szDisplayName);

			newitem->height = entryHeight;
						
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
						
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
			{
				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
								
			newitem->hiddenPart = hiddenPart;
			newitem->multiPart = menuBreaks;

			POINT tmp = buttonfont->measure(newitem->caption);

			if (maxNeededWidth < tmp.x)
				maxNeededWidth = tmp.x;

			entrylist.insert(entrylist.end(), newitem);
		}
		else
		{
			//Entry
			ENTRY* newitem = new ENTRY;

			newitem->m_pidl = (*pit);

			newitem->x = nextXPosition;
			newitem->y = nextYPosition;

			newitem->caption = shfi.szDisplayName;

			ScanKeyCodes(newitem, (string) shfi.szDisplayName);
			
			newitem->height = entryHeight;
						
			if (nextYPosition+newitem->height+bottomBorder > maxHeight)
			{
				if (autoMenuBreak)
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

					nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
					nextYPosition = topBorder;

					newitem->x = nextXPosition;
					newitem->y = nextYPosition;

					menuBreaks++;

					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
				else
				{
					if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
						adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
					nextYPosition = topBorder;

					newitem->y = nextYPosition;

					hiddenPart++;
						
					nextYPosition = nextYPosition+newitem->height+ySpacing;
				}
			}
			else
			{
				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
								
			newitem->hiddenPart = hiddenPart;
			newitem->multiPart = menuBreaks;

			POINT tmp = buttonfont->measure(newitem->caption);

			if (maxNeededWidth < tmp.x)
				maxNeededWidth = tmp.x;

			entrylist.insert(entrylist.end(), newitem);
		}
	}

	//Insert AutoSeparator if wished at the Bottom
	if (autoSeparator == 3 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		if ( nextYPosition != topBorder )
		{
			newitem->y = nextYPosition+separatorySpacing;
			nextYPosition = newitem->y;
		}
		else
		{
			newitem->y = nextYPosition;
		}
					
		newitem->height = separatorHeight;
					
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
							
				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	//Check Max/Min EntryWidth
	maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);
		
	//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
	POINT tmp = buttonfont->measure(captionname, 1);
	int neededTitleWidth = leftBorder+tmp.x+rightBorder;

	//Adapt EntryWidth based on Max/Min Width
	if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
	{
		actualwidth = maxWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
	{
		actualwidth = minWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else
	{
		actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
		actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
	}

	if (hiddenPart != 0 || menuBreaks != 0)
		actualheight = adaptedmaxHeight;
	else
		actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);
	actualheight = max(actualheight, minHeight);

	startentry = NULL;

	for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (startentry == NULL && (*eit)->entryType <= 0)
			startentry = *eit;

		if (menuBreaks == 0)
		{
			(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
		}
		else
		{
			(*eit)->width = maxNeededWidth;
				
			(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);
		}
	}

	maxPart = hiddenPart;
	maxMenuBreak = menuBreaks;

	reposition(2, 0, 0, actualwidth, actualheight);
	repaint(true);
}

void Popup::CreateLoadingPopup()
{
	if ( pParentPopup != NULL )
		pParentPopup->repaint();
	
	if (noLoadingPopup || alphaFade)
		return;
	
	//Create/Show "Loading" Popup
	ENTRY* newitem = new ENTRY;

	newitem->entryType = 2;

	newitem->x = leftBorder;
	newitem->y = topBorder;
	newitem->height = entryHeight;
	newitem->caption = "Loading ...";
	newitem->info = "Loading ...";

	newitem->hiddenPart = 0;
	newitem->multiPart = 0;

	POINT tmp = buttonfont->measure(newitem->caption);

	newitem->width = tmp.x;

	tmp = buttonfont->measure(captionname, 1);

	tmp.x = max(newitem->width, tmp.x);

	newitem->width = tmp.x;

	entrylist.insert(entrylist.end(), newitem);

	actualwidth = leftBorder + tmp.x + rightBorder;
	actualheight = topBorder + newitem->height + bottomBorder;

	//Enable InfoEntry temporary
	hasInfoEntry = true;

	reposition(2, 0, 0, actualwidth, actualheight);
	repaint(true);
	
	show(false, true);
	
	//Disable InfoEntry 
	hasInfoEntry = false;
	//---------------------------
}

void Popup::ScanTasks()
{
	isTaskPopup = true;

	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		delete *eit;
		*eit = NULL;
	}

	entrylist.clear();
	
	//Reset old Pointers
	//-------------------------
	currentPart = 0;

	startentry = NULL;
	lastactiveentry = NULL;
	hoverentry = NULL;
	//-------------------------

	//---------------------------------------------------------------------------------------
	// START Scanning
	//---------------------------------------------------------------------------------------
	tasklist.clear();

	EnumWindows(Popup::EnumWindowsProc, (LPARAM) this);
	//---------------------------------------------------------------------------------------
	// END Scanning
	//---------------------------------------------------------------------------------------

	//Sorting
	SortTasks(tasklist);

	//Initializing of Values
	int nextXPosition = leftBorder;
	int nextYPosition = topBorder;
	int hiddenPart = 0;
	int menuBreaks = 0;
	int maxNeededWidth = 0;
	int adaptedmaxHeight = 0;

	//Insert AutoSeparator if wished between Title and Entrys
	if (autoSeparator >= 2 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		if ( nextYPosition != topBorder )
		{
			newitem->y = nextYPosition+separatorySpacing;
			nextYPosition = newitem->y;
		}
		else
		{
			newitem->y = nextYPosition;
		}
					
		newitem->height = separatorHeight;
					
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
							
				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	for (hWndTaskListIterator tit = tasklist.begin(); tit != tasklist.end(); ++tit)
	{
		ENTRY* newitem = new ENTRY;

		newitem->x = nextXPosition;
		
		char windowCaption[MAX_LINE_LENGTH] = {0};
		GetWindowText( (*tit), windowCaption, MAX_LINE_LENGTH );
		
		newitem->caption = windowCaption;

		ScanKeyCodes(newitem, (string) windowCaption);

		newitem->y = nextYPosition;
		newitem->height = entryHeight;
				
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
					
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
				
				nextYPosition = nextYPosition+newitem->height+ySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		newitem->hWnd = (*tit);

		POINT tmp = buttonfont->measure(newitem->caption);

		if (maxNeededWidth < tmp.x)
			maxNeededWidth = tmp.x;

		entrylist.insert(entrylist.end(), newitem);
	}

	//Insert AutoSeparator if wished at the Bottom
	if (autoSeparator == 3 && quicklaunchMode == 0)
	{
		ENTRY* newitem = new ENTRY;	
					
		newitem->entryType = 1;

		newitem->x = nextXPosition;
		if ( nextYPosition != topBorder )
		{
			newitem->y = nextYPosition+separatorySpacing;
			nextYPosition = newitem->y;
		}
		else
		{
			newitem->y = nextYPosition;
		}
					
		newitem->height = separatorHeight;
					
		if (nextYPosition+newitem->height+bottomBorder > maxHeight)
		{
			if (autoMenuBreak)
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;

				nextXPosition = nextXPosition+maxNeededWidth+xSpacing;
				nextYPosition = topBorder;

				newitem->x = nextXPosition;
				newitem->y = nextYPosition;

				menuBreaks++;

				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
			else
			{
				if (adaptedmaxHeight < nextYPosition-ySpacing+bottomBorder)
					adaptedmaxHeight = nextYPosition-ySpacing+bottomBorder;
							
				nextYPosition = topBorder;

				newitem->y = nextYPosition;

				hiddenPart++;
							
				nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
			}
		}
		else
		{
			nextYPosition = nextYPosition+newitem->height+ySpacing+separatorySpacing;
		}
						
		newitem->hiddenPart = hiddenPart;
		newitem->multiPart = menuBreaks;

		entrylist.insert(entrylist.end(), newitem);
	}

	//Check Max/Min EntryWidth
	maxNeededWidth = max(min(maxNeededWidth, maxEntryWidth), minEntryWidth);

	//Measure with Type 1 only check TitleFont Settings for maxNeededWidth
	POINT tmp = buttonfont->measure(captionname, 1);
	int neededTitleWidth = leftBorder+tmp.x+rightBorder;

	//Adapt EntryWidth based on Max/Min Width
	if ( leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder >= maxWidth )
	{
		actualwidth = maxWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else if (leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder <= minWidth )
	{
		actualwidth = minWidth;
		maxNeededWidth = (actualwidth-leftBorder-rightBorder-menuBreaks*xSpacing)/(menuBreaks+1);
	}
	else
	{
		actualwidth = leftBorder+(menuBreaks+1)*maxNeededWidth+menuBreaks*xSpacing+rightBorder;
		actualwidth = min(max(neededTitleWidth, actualwidth), maxWidth);
	}

	if (hiddenPart != 0 || menuBreaks != 0)
		actualheight = adaptedmaxHeight;
	else
		actualheight = min(nextYPosition-ySpacing+bottomBorder, maxHeight);
	actualheight = max(actualheight, minHeight);

	startentry = NULL;

	for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (startentry == NULL && (*eit)->entryType <= 0)
			startentry = *eit;

		if (menuBreaks == 0)
		{
			(*eit)->width = max(maxNeededWidth, actualwidth-leftBorder-rightBorder);
		}
		else
		{
			(*eit)->width = maxNeededWidth;
			
			(*eit)->x = leftBorder + (*eit)->multiPart*(maxNeededWidth + xSpacing);
		}
	}

	maxPart = hiddenPart;
	maxMenuBreak = menuBreaks;

	reposition(2, 0, 0, actualwidth, actualheight);
	repaint(true);

	if ((isTaskPopup || hasDynamicContent) && !hasInfoEntry)
		SetTimer(hWnd, TIMER_UPDATE, tasksUpdate, NULL);
	else if (hasInfoEntry)
		SetTimer(hWnd, TIMER_UPDATE, 1000, NULL);
}

BOOL WINAPI Popup::EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	Popup *popup = (Popup *) lParam;
	if (popup && IsAppWindow(hWnd))
	{
		popup->tasklist.insert(popup->tasklist.end(), hWnd);
	}
	return TRUE;
}

void Popup::CreateSubMenu(string newsubname, string newrootpath, ENTRY* pNewParentEntry, bool jumpTo, bool noShow)
{
	CString help = newsubname.c_str();
	help.Replace("sub_", "");

	for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
	{
		if (_stricmp( help, (*plit)->captionname.c_str()) == NULL)
		{
			return;
		}
	}

	Popup *popup = NULL;

	if ( customSubFolderList.empty() && pNewParentEntry->layout.empty() )
		popup = new Popup(name, newsubname, popuptexture, buttontexture, buttonicon, buttonfont, ghostpopuplist, ghostbuttonlist);
	else if ( !pNewParentEntry->layout.empty() )
		popup = new Popup(pNewParentEntry->layout, newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, (pNewParentEntry->defaultlayout.empty() ? name : pNewParentEntry->defaultlayout));
	else if ( subFolderLevel < customSubFolderList.size() )
		popup = new Popup(customSubFolderList[subFolderLevel], newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
	else
		popup = new Popup(*customSubFolderList.end(), newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
			
	subpopupList.insert(subpopupList.end(), popup);
	
	popup->pParentPopup = this;
	popup->pParentEntry = pNewParentEntry;
			
	popup->load(hInstance);

	if ( !noShow )
		popup->CreateLoadingPopup();
	
	if (!newrootpath.empty())
	{
		popup->rootScanPath = newrootpath;
		popup->rootUniversalAction = pNewParentEntry->universalaction;
		
		popup->ScanSubMenu(newrootpath, pNewParentEntry->entryType, pNewParentEntry->universalaction);

		if ( pNewParentEntry->entryType == -2 )
		{
			popup->dynamicPopup = true;

			bool newIteration = true;
			CString localworker1 = popup->rootScanPath.c_str();
			CString localworker2 = "";

			while (newIteration == true)
			{
				if (localworker1.Find("|") != -1)
				{
					CNotifyDirCheck *checkDirectory = new CNotifyDirCheck();
			
					popup->checkdirectorylist.insert(popup->checkdirectorylist.end(), checkDirectory);
					
					localworker2 = localworker1.Left(localworker1.Find("|"));

					if ( localworker2.Find("*") == -1 )
						checkDirectory->SetDirectory( localworker2 );
					else
						checkDirectory->SetDirectory( localworker2.Left(localworker2.Find("*")) );

					//set your callback to work with each new event
					checkDirectory->SetPopup( popup );

					checkDirectory->Run();

					localworker1 = localworker1.Right(localworker1.GetLength()-localworker1.Find("|")-1);
				}
				else
				{
					CNotifyDirCheck *checkDirectory = new CNotifyDirCheck();
				
					popup->checkdirectorylist.insert(popup->checkdirectorylist.end(), checkDirectory);
					
					if ( localworker1.Find("*") == -1 )
						checkDirectory->SetDirectory( localworker1 );
					else
						checkDirectory->SetDirectory( localworker1.Left(localworker1.Find("*")) );

					//set your callback to work with each new event
					checkDirectory->SetPopup( popup );

					checkDirectory->Run();

					newIteration = false;
				}
			}
		}
	}

	if ( !noShow )
	{	
		popup->show();

		if (jumpTo && popup->startentry != NULL)
			SetCursorPos(popup->actualx+popup->startentry->x+2, popup->actualy+popup->startentry->y+2);
	}
}

void Popup::CreateShellSubMenu( string newsubname, int newclsid, ENTRY* pNewParentEntry, bool jumpTo, bool noShow)
{
	CString help = newsubname.c_str();
	help.Replace("sub_", "");

	for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
	{
		if (_stricmp( help, (*plit)->captionname.c_str()) == NULL)
		{
			return;
		}
	}

	Popup *popup = NULL;
	
	if ( customSubFolderList.empty() && pNewParentEntry->layout.empty() )
		popup = new Popup(name, newsubname, popuptexture, buttontexture, buttonicon, buttonfont, ghostpopuplist, ghostbuttonlist);
	else if ( !pNewParentEntry->layout.empty() )
		popup = new Popup(pNewParentEntry->layout, newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, (pNewParentEntry->defaultlayout.empty() ? name : pNewParentEntry->defaultlayout));
	else if ( subFolderLevel < customSubFolderList.size() )
		popup = new Popup(customSubFolderList[subFolderLevel], newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
	else
		popup = new Popup(*customSubFolderList.end(), newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
			
	subpopupList.insert(subpopupList.end(), popup);

	popup->pParentPopup = this;
	popup->pParentEntry = pNewParentEntry;
										
	popup->load(hInstance);

	if (!noShow)
		popup->CreateLoadingPopup();

	popup->ScanShellFolder(newclsid);

	if (!noShow)
	{
		popup->show();
	
		if (jumpTo && popup->startentry != NULL)
			SetCursorPos(popup->actualx+popup->startentry->x+2, popup->actualy+popup->startentry->y+2);
	}
}

void Popup::CreateTaskSubMenu( string newsubname, ENTRY* pNewParentEntry, bool jumpTo, bool noShow)
{
	Popup *popup = NULL;
	
	if ( customSubFolderList.empty() && pNewParentEntry->layout.empty() )
		popup = new Popup(name, newsubname, popuptexture, buttontexture, buttonicon, buttonfont, ghostpopuplist, ghostbuttonlist);
	else if ( !pNewParentEntry->layout.empty() )
		popup = new Popup(pNewParentEntry->layout, newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, (pNewParentEntry->defaultlayout.empty() ? name : pNewParentEntry->defaultlayout));
	else if ( subFolderLevel < customSubFolderList.size() )
		popup = new Popup(customSubFolderList[subFolderLevel], newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);
	else
		popup = new Popup(*customSubFolderList.end(), newsubname, NULL, NULL, NULL, NULL, NULL, NULL, subFolderLevel, name);

	subpopupList.insert(subpopupList.end(), popup);

	popup->pParentPopup = this;
	popup->pParentEntry = pNewParentEntry;
										
	popup->load(hInstance);

	if (!noShow)
		popup->CreateLoadingPopup();

	popup->ScanTasks();

	if (!noShow)
	{
		popup->show();	
		
		if (jumpTo && popup->startentry != NULL)
			SetCursorPos(popup->actualx+popup->startentry->x+2, popup->actualy+popup->startentry->y+2);
	}
}

//////////////////////////////////////////////////////////////////////
// Window Related: Show/Hide/Reposition
//////////////////////////////////////////////////////////////////////

void Popup::reposition(int type, int newx, int newy, int newwidth, int newheight, int externCall)
{
	if ( externCall == 1 )
	{
		minWidth = newwidth;
		minHeight = newheight;
		maxWidth = newwidth;
		maxHeight = newheight;

		for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
		{
			(*plit)->hide();
			delete *plit;
			*plit = NULL;
		}

		subpopupList.clear();

		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		{
			delete *eit;
			*eit = NULL;
		}

		entrylist.clear();

		//Reset old Pointers
		//-------------------------
		currentPart = 0;

		startentry = NULL;
		lastactiveentry = NULL;
		hoverentry = NULL;
		//-------------------------

		ParseStepRC(true);
	}
	else
	{
		if ( type == 1 )
		{
			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
		}
		else if ( type == 2 )
		{
			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);
		}
		else if ( type == 3 )
		{
			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOZORDER);
		}
		
		//Set new Rect for ToolTip Tracker
		if ( type != 1  && ToolTips && (quicklaunchMode != 0 || tooltipMode != 0) )
			updateHint(hWnd, "", true);
	}
}

void Popup::show(bool setcursor, bool loading)
{
	if ( visible )
	{
		if ( hasDynamicContent )
			ParseStepRC(false);

		if ( pParentPopup != NULL && pParentEntry != NULL )
		{
			if (pParentEntry->hWnd != NULL && pParentPopup->hWnd == pParentEntry->hWnd)
				ScanTasks();
		}

		if ( setcursor && startentry != NULL )
		{
			SetCursorPos(actualx+startentry->x+2, actualy+startentry->y+2);
			hoverentry = startentry;
			repaint();
		}

		RepositionSubPopupMenus();

		if ( PinnedMode )
		{
			SetForegroundWindow(hWnd);
			SetActiveWindow(hWnd);
			SetFocus(hWnd);
		}

		return;
	}

	//Dynamic Popups
	if ( !loading )
	{
		if ( hasDynamicContent )
			ParseStepRC(false);

		if ( pParentPopup != NULL && pParentEntry != NULL )
		{
			if (pParentEntry->hWnd != NULL && pParentPopup->hWnd == pParentEntry->hWnd)
				ScanTasks();
		}
	}

	bool showit = false;

	//UnPin "SubPopupMenus", if they are called by Parent
	if ( PinnedMode && pParentPopup != NULL && pParentEntry != NULL )
	{
		PinnedMode = false;
		setAlwaysOnTop(true);

		showit = true;
	}

	//Check if Main Popup is visible
	Popup* tmpPointer = pParentPopup;
	if (tmpPointer != NULL && !showit)
	{
		while (tmpPointer->pParentPopup != NULL)
		{
			if ( tmpPointer->isVisible() && tmpPointer->PinnedMode )
			{
				showit = true;
				break;
			}
			tmpPointer = tmpPointer->pParentPopup;
		}
			
		if ( !tmpPointer->isVisible() )
			return;
		else
			showit = true;
	}
	else
	{
		showit = true;
	}

	if (showit)
	{
		RepositionSubPopupMenus();

		if ( hasInfoEntry )
		{
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( !(*eit)->info.empty() )
					(*eit)->caption = InfoUpdate( (*eit)->info );
			}

			repaint(false, true);
		}

		if (alphaFade)
		{
			UpdateAlpha = 0;
			UpdateWindowTransparent(hWnd, UpdateAlpha);
			ShowWindow(hWnd, SW_SHOW);
			RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);	
				
			KillTimer(hWnd, TIMER_ALPHAFADEOUT);
			SetTimer(hWnd, TIMER_ALPHAFADEIN, alphaFadeDelay, NULL);
		}
		else if (alphaMap || fakealphaMap)
		{
			UpdateWindowTransparent(hWnd, alphaTransparency);
			ShowWindow(hWnd, SW_SHOW);
			RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW );
		} 
		else
		{
			ShowWindow(hWnd, SW_SHOW);
			RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW );
		}

		//Only Main Popups
		if (pParentPopup == NULL)
			SetTimer(hWnd, TIMER_ACTIVE, hideDelay, NULL);

		visible = true;

		visiblepopupList.insert(visiblepopupList.end(), this);

		if ( setcursor && startentry != NULL)
			SetCursorPos(actualx+startentry->x+2, actualy+startentry->y+2);

		if ( !BangToCall.empty() )
		{
			if ( !openSound.empty() )
				ParseBangCommand(hWnd, "!playsound", openSound.c_str());

			if ( !openCommand.empty() )
			{
				CString result = openCommand.c_str();

				CString worker = BangToCall.c_str();
				worker.Replace("!", "");
				
				result.Replace("%[popupname]%", worker);
				result.Replace("%{popupname}%", worker);

				LSExecute(hWnd, result, SW_SHOWNORMAL);
			}

			SetForegroundWindow(hWnd);
			SetActiveWindow(hWnd);
			SetFocus(hWnd);
		}
	}

	//Make sure Parent Popup Tooltips remain visible
	if ( ToolTips && !loading )
	{
		Popup* tmpPointer = pParentPopup;
		if ( tmpPointer != NULL )
		{
			if ( tmpPointer->ToolTips )			
				SetWindowPos( tmpPointer->ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE ); 

			while (tmpPointer->pParentPopup != NULL)
			{
				tmpPointer = tmpPointer->pParentPopup;

				if ( tmpPointer->ToolTips )			
					SetWindowPos( tmpPointer->ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE ); 
			}
		}
	}
}

void Popup::RepositionSubPopupMenus()
{
	//Reposition "SubPopupMenus"
	if (pParentPopup != NULL && pParentEntry != NULL)
	{
		int x = 0;
		int y = 0;

		//QuickLaunch Mode
		if (quicklaunchMode != 0)
		{
			if (quicklaunchMode == 1)
			{
				x = pParentPopup->actualx + pParentEntry->x + overlapX;
					
				if (quicklaunchOpenTo == 1)
					y = pParentPopup->actualy - actualheight + overlapY;
				else if (quicklaunchOpenTo == 2)
					y = pParentPopup->actualy + pParentPopup->actualheight + overlapY;
			}
			else if (quicklaunchMode == 2)
			{
				if (quicklaunchOpenTo == 1)
					x = pParentPopup->actualx + pParentPopup->actualwidth + overlapX;
				else if (quicklaunchOpenTo == 2)
					x = pParentPopup->actualx - actualwidth + overlapX;

				y = pParentPopup->actualy + pParentEntry->y + overlapY;
			}

			//InScreen Check
			if (x+actualwidth > screenWidth)
				x = pParentPopup->actualx-actualwidth-overlapX;

			if (y+actualheight > screenHeight)
				y = screenHeight-actualheight;

			//Minimum x, y is screenX/Y
			if ( x < screenX )
				x = screenX;
			if ( y < screenY )
				y = screenY;

			reposition(1, x, y, 0, 0);
		}
		//Normal PopupMenus
		else
		{
			if ( openToLeft )
			{
				if ( pParentEntry->multiPart == 0 )
					x = pParentPopup->actualx - actualwidth - overlapX;
				else
					x = pParentPopup->actualx + pParentEntry->x - actualwidth - pParentPopup->xSpacing/2 - overlapX;
							
				y = pParentPopup->actualy + pParentEntry->y + overlapY;

				//InScreen Check
				if ( x < screenX && pParentEntry->multiPart == 0 && pParentPopup->maxMenuBreak != 0 )
					x = pParentPopup->actualx + pParentEntry->x + pParentEntry->width + pParentPopup->xSpacing/2 + overlapX;
				else if ( x < screenX )
					x = pParentPopup->actualx + pParentPopup->actualwidth + overlapX;

				if (y+actualheight > screenHeight)
					y = screenHeight-actualheight;

				//Maximum x is screenWidth, Minimum y is screenY
				if ( x > screenWidth )
					x = screenWidth-actualwidth;
				if ( y < screenY )
					y = screenY;
			}
			else
			{
				if ( pParentEntry->multiPart != pParentPopup->maxMenuBreak && pParentPopup->maxMenuBreak != 0 )
					x = pParentPopup->actualx + pParentEntry->x + pParentEntry->width + pParentPopup->xSpacing/2 + overlapX;
				else
					x = pParentPopup->actualx + pParentPopup->actualwidth + overlapX;
			
				y = pParentPopup->actualy + pParentEntry->y + overlapY;

				//InScreen Check
				if ( x+actualwidth > screenWidth && pParentEntry->multiPart == 0 )
					x = pParentPopup->actualx - actualwidth - overlapX;
				else if ( x+actualwidth > screenWidth )
					x = pParentPopup->actualx + pParentEntry->x - actualwidth - pParentPopup->xSpacing/2 - overlapX;

				if (y+actualheight > screenHeight)
					y = screenHeight-actualheight;

				//Minimum x, y is screenX/Y
				if ( x < screenX )
					x = screenX;
				if ( y < screenY )
					y = screenY;
			}

			reposition(1, x, y, 0, 0);
		}
	}
}

void Popup::hide(bool noFade)
{
	bool wasVisible = visible;

	if ( !PinnedMode && visible )
	{
		if (alphaFade && !noFade)
		{
			UpdateAlpha = alphaTransparency;

			KillTimer(hWnd, TIMER_ALPHAFADEIN);
			SetTimer(hWnd, TIMER_ALPHAFADEOUT, alphaFadeDelay, NULL);
		}
		else if ( !alphaFade || noFade )
		{
			KillTimer(hWnd, TIMER_ALPHAFADEIN);
			KillTimer(hWnd, TIMER_ALPHAFADEOUT);
			ShowWindow(hWnd, SW_HIDE);
		
			lastactiveentry = NULL;
			hoverentry = NULL;
			lastkeyentry = -1;
			
			visible = false;
		}

		for(PopupListIterator plit = visiblepopupList.begin(); plit != visiblepopupList.end(); plit++)
		{
			if ( (*plit) == this )
			{
				visiblepopupList.erase( plit );
				break;
			}
		}
		
		KillTimer(hWnd, TIMER_ACTIVE);
	
		for(plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
		{
			if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
				(*plit)->hide(noFade);
		}
		
		if ( !BangToCall.empty() )
		{	
			if ( !closeSound.empty() && !execSoundPlayed )
				ParseBangCommand(hWnd, "!playsound", closeSound.c_str());

			execSoundPlayed = false;

			if ( !closeCommand.empty() && wasVisible )
			{
				CString result = closeCommand.c_str();

				CString worker = BangToCall.c_str();
				worker.Replace("!", "");
				
				result.Replace("%[popupname]%", worker);
				result.Replace("%{popupname}%", worker);

				LSExecute(hWnd, result, SW_SHOWNORMAL);
			}
		}
	}
	else if ( PinnedMode )
	{
		for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
		{
			if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
				(*plit)->hide(noFade);
		}
	}

	//Make sure Parent Popup Tooltips remain visible
	if ( ToolTips )
	{
		Popup* tmpPointer = pParentPopup;
		if ( tmpPointer != NULL )
		{
			if ( tmpPointer->ToolTips )			
				SetWindowPos( tmpPointer->ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE ); 

			while (tmpPointer->pParentPopup != NULL)
			{
				tmpPointer = tmpPointer->pParentPopup;

				if ( tmpPointer->ToolTips )			
					SetWindowPos( tmpPointer->ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE ); 
			}
		}
	}
}

void Popup::ToggleShade()
{
	if (shaded != -1)
	{
		if ( !BangToCall.empty() && !unshadeCommand.empty() )
		{
			CString result = unshadeCommand.c_str();

			CString worker = BangToCall.c_str();
			worker.Replace("!", "");
				
			result.Replace("%[popupname]%", worker);
			result.Replace("%{popupname}%", worker);

			LSExecute(hWnd, result, SW_SHOWNORMAL);
		}

		int tmpHeight = shaded;
		shaded = -1;
		reposition(2, 0, 0, actualwidth, tmpHeight);
		repaint(true);
	}
	else
	{
		if ( !BangToCall.empty() && !shadeCommand.empty() )
		{
			CString result = shadeCommand.c_str();

			CString worker = BangToCall.c_str();
			worker.Replace("!", "");
				
			result.Replace("%[popupname]%", worker);
			result.Replace("%{popupname}%", worker);

			LSExecute(hWnd, result, SW_SHOWNORMAL);
		}

		shaded = actualheight;
		reposition(2, 0, 0, actualwidth, max(max(topBorder+bottomBorder, popuptexture->getMinNeededHeight()), orgminHeight));
		repaint(true);
	}
}

//------------------------------------------------------------
//Mouse Related Actions (Clicks, Enter, Leave, Move, ...)
//------------------------------------------------------------

void Popup::ontemplateButtonDown(const int &x, const int &y, int titleActionType, int bottomActionType)
{
	if ( (titleActionType == 6 && MoveAllowed && y >= 0 && y <= topBorder)
			|| (bottomActionType == 6 && MoveAllowed && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		mousePressed = true;
		GetCursorPos(&savedpt);
		ScreenToClient(hWnd, &savedpt);
		moveButtonDown = true;
		SetCapture(hWnd);
	}
	else
	{
		if ( dragndropEnabled )
			ActionAtPoint( x, y, false, false, 1 );

		SetCapture(hWnd);
	}
}

void Popup::ontemplateButtonUp(const int &x, const int &y, int titleActionType, int bottomActionType, int mousebutton)
{
	bool normalactions = true;
	bool somethingdone = false;
	if ( ((titleActionType == 1 || titleActionType == 2 || titleActionType == 7) && y >= 0 && y <= topBorder )
			|| ((bottomActionType == 1 || bottomActionType == 2 || bottomActionType == 7) && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		normalactions = false;
		somethingdone = true;
		if ( PinnedMode && (titleActionType == 2 || titleActionType == 7) && y >= 0 && y <= topBorder )
		{
			PinnedMode = false;
			setAlwaysOnTop(true);
		}
		if ( PinnedMode && (bottomActionType == 2 || bottomActionType == 7) && y >= actualheight-bottomBorder && y <= actualheight )
		{
			PinnedMode = false;
			setAlwaysOnTop(true);
		}
		if (!PinnedMode && titleActionType == 1 && y >= 0 && y <= topBorder )
		{
			PinnedMode = true;
			if ( pinnedPopupNotOnTop )
				setAlwaysOnTop(false);
			else
				setAlwaysOnTop(true);
		}
		if (!PinnedMode && bottomActionType == 1 && y >= actualheight-bottomBorder && y <= actualheight )
		{
			PinnedMode = true;
			if ( pinnedPopupNotOnTop )
				setAlwaysOnTop(false);
			else
				setAlwaysOnTop(true);
		}
		if ( ((titleActionType == 2 || titleActionType == 7) && y >= 0 && y <= topBorder)
			|| ((bottomActionType == 2 || bottomActionType == 7) && y >= actualheight-bottomBorder && y <= actualheight) )
			hide();
	}
	else if ( (titleActionType == 3 && y >= 0 && y <= topBorder) 
				|| (bottomActionType == 3 && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		normalactions = false;
		somethingdone = true;
		if ( PinnedMode )
		{
			PinnedMode = false;
			setAlwaysOnTop(true);
			hide();
		}
		else if ( !PinnedMode )
		{
			PinnedMode = true;
			if ( pinnedPopupNotOnTop )
				setAlwaysOnTop(false);
			else
				setAlwaysOnTop(true);
		}
	}
	else if ( (titleActionType == 4 && y >= 0 && y <= topBorder) 
				|| (bottomActionType == 4 && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		normalactions = false;
		if ( ShadeAllowed && !(0x8000 & GetKeyState(VK_CONTROL)) && !(0x8000 & GetKeyState(VK_SHIFT)) )
		{
			ToggleShade();
			somethingdone = true;
		}
	}
	else if ( (titleActionType == 5 && y >= 0 && y <= topBorder) 
				|| (bottomActionType == 5 && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		normalactions = false;
		if ( PinnedMode )
		{
			somethingdone = true;
			setAlwaysOnTop(!alwaysOnTop);
		}
	}
	else if ( titleActionType == 6 || bottomActionType == 6 )
	{
		if ( moveButtonDown )
		{
			somethingdone = true;
			normalactions = false;
			if ( MoveAllowed && mousePressed && mouseMoved && pParentPopup != NULL )
			{
				if ( AutoPin && !PinnedMode )
				{
					PinnedMode = true;
					if (pinnedPopupNotOnTop)
						setAlwaysOnTop(false);
					else
						setAlwaysOnTop(true);
					SetTimer(hWnd, TIMER_ACTIVE, hideDelay, NULL);
				}	
			}
			mouseMoved = false;

			//To be on the save side
			mousePressed = false;

			moveButtonDown = false;
		}
	}
	else if ( (titleActionType == 8 && y >= 0 && y <= topBorder) 
			|| (bottomActionType == 8 && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		normalactions = false;
		if ( maxPart != 0)
		{
			currentPart--;
			if (currentPart < 0)
				currentPart = maxPart;

			oldmovetracker.x = 0;

			repaint(true);
		}
		somethingdone = true;
	}
	else if ( (titleActionType == 9 && y >= 0 && y <= topBorder) 
			|| (bottomActionType == 9 && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		normalactions = false;
		if ( maxPart != 0)
		{
			currentPart++;
			if (currentPart > maxPart)
				currentPart = 0;

			oldmovetracker.x = 0;

			repaint(true);
		}
		somethingdone = true;
	}

	if ( normalactions && shaded == -1 )
	{
		if ( mousebutton == 1 )
		{	
			if ( dragndropEnabled )
			{
				ActionAtPoint( x, y, false, false, 2 );
		
				if ( _stricmp(dragndropSource.c_str(), "-1") != NULL && !dragndropDestination.empty() )
				{
					string destination = "";
					destination.append(dragndropDestination);

					//Filter FolderClicks
					if ( _stricmp(dragndropSource.c_str(), destination.c_str()) != NULL )
					{	
						CString worker = dragndropSource.c_str();
						worker = worker.Right(worker.GetLength()-worker.ReverseFind('\\'));
						
						destination.append( worker );

						if ( _stricmp(dragndropSource.c_str(), destination.c_str()) != NULL && StrRStrI( destination.c_str(), NULL, dragndropSource.c_str() ) == NULL )
						{
							ReleaseCapture();

							if ( MessageBox(NULL, ("Are you sure you want to move the following file/folder?\n\n" + dragndropSource + "\nto:\n" + destination ).c_str(), "User Confirmation", MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND) == IDYES )
								MoveFile( dragndropSource.c_str(), destination.c_str() );
						}
						else
						{
							//ActionAtPoint: First Bool is Normal Action, Second is ContextMenu
							ActionAtPoint(x, y, true);
						}
					}
					else
					{
						//ActionAtPoint: First Bool is Normal Action, Second is ContextMenu
						ActionAtPoint(x, y, true);
					}
				}
				else
				{
					//ActionAtPoint: First Bool is Normal Action, Second is ContextMenu
					ActionAtPoint(x, y, true);
				}
			}
			else
			{
				//ActionAtPoint: First Bool is Normal Action, Second is ContextMenu
				ActionAtPoint(x, y, true);
			}
		}
		else if ( mousebutton == 2 )
		{
			//ActionAtPoint: First Bool is Normal Action, Second is ContextMenu
			ActionAtPoint(x, y, false, true);
		}
	}
	if ( !somethingdone && shaded == -1 && mousebutton == 3 && (x >= 4 || y >= 4) )
	{
		//Bool Value: True = Upwards, False = downwards
		repaint(SelectNextEntry(false));
	}

	ReleaseCapture();
}

void Popup::onLButtonDblClk(int x, int y)
{
	if ( ((showCaption == 1 || showCaption == 0) && HideAllowed && y >= 0 && y <= topBorder)
		|| ((showCaption == 2 || showCaption == 0) && HideAllowed && y >= actualheight-bottomBorder && y <= actualheight) )
	{
		if (PinnedMode)
		{
			PinnedMode = false;
			setAlwaysOnTop(true);
			hide();
		}
		else
		{
			hide();
		}
	}
	else 
	{
		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		{
			if (currentPart != (*eit)->hiddenPart || (*eit)->entryType >= 0)
				continue;

			if ( x < ((*eit)->x + (*eit)->width) && x >= (*eit)->x &&
				 y < ((*eit)->y + (*eit)->height) && y >= (*eit)->y )
			{
				for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
				{
					if (_stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
					{
						if ( !(*eit)->action.empty() )
						{
							if ((*eit)->pipedList.size() == 1)
							{
								Popup *tmpPointer = pParentPopup;
								if (tmpPointer != NULL)
								{
									while (tmpPointer->pParentPopup != NULL)
										tmpPointer = tmpPointer->pParentPopup;
									tmpPointer->hide(true);
								}
								else
								{
									hide(true);
								}
								
								LSExecute(NULL, (folderDoubleClickAction + " " + (*eit)->action).c_str(), NULL);
							}
							else
							{
								HMENU GroupMenu = CreatePopupMenu();

								int counter = 1;
								for(StringListIterator sit = (*eit)->pipedList.begin(); sit != (*eit)->pipedList.end(); ++sit)
								{
									AppendMenu( GroupMenu, MF_STRING, counter, (*sit).c_str() );
									counter++;
								}

								// display the menu
								POINT pt;
								GetCursorPos( &pt );

								int command = TrackPopupMenu( GroupMenu,
															  TPM_RETURNCMD | TPM_RIGHTBUTTON,
															  pt.x, pt.y, 0,
															  this->gethWnd(),
															  NULL );

								if ( command )
								{
									StringListIterator sit = (*eit)->pipedList.begin();
									counter = 1; 
									while (counter < command)
									{
										sit++;
										counter++;
									}

									if ( counter <= (*eit)->pipedList.size() )
									{
										Popup *tmpPointer = pParentPopup;
										if (tmpPointer != NULL)
										{
											while (tmpPointer->pParentPopup != NULL)
												tmpPointer = tmpPointer->pParentPopup;
											tmpPointer->hide(true);
										}
										else
											hide(true);

										LSExecute(NULL, (folderDoubleClickAction + " " + (*sit)).c_str(), NULL);
									}
								}
											
								if (GroupMenu)
									DestroyMenu(GroupMenu);
							}
						}
						break;
					}
				}
			}
		}
	}
}

void Popup::onWheelDown(int x, int y)
{
	if ( (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		if ( maxPart != 0)
		{
			currentPart++;
			if (currentPart > maxPart)
				currentPart = 0;

			oldmovetracker.x = 0;

			repaint(true);
		}
	}
	else
	{
		//Bool Value: True = Upwards, False = downwards
		repaint(SelectNextEntry(false));
	}
}

void Popup::onWheelUp(int x, int y)
{
	if ( (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		if ( maxPart != 0)
		{
			currentPart--;
			if (currentPart < 0)
				currentPart = maxPart;

			oldmovetracker.x = 0;

			repaint(true);
		}
	}
	else
	{
		//Bool Value: True = Upwards, False = downwards
		repaint(SelectNextEntry(true));
	}
}

void Popup::onKey(int nKey)
{
	switch (nKey)
	{
		case VK_F5:
		{
			if ( !BangToCall.empty() && pParentPopup == NULL )
			{
				ParseBangCommand(hWnd, "!popuprescan", "");
			}
			else if ( pParentPopup != NULL )
			{
				for(PopupListIterator plit = pParentPopup->subpopupList.begin(); plit != pParentPopup->subpopupList.end(); plit++)
				{
					if ( (*plit) == this )
					{
						pParentPopup->subpopupList.erase( plit );
						(*plit)->hide(true);
						delete *plit;
						*plit = NULL;
						break;
					}
				}
			}
		}
		break;

		case VK_PRIOR:
		{
			if ( maxPart != 0)
			{
				currentPart--;
				if (currentPart < 0)
					currentPart = maxPart;

				oldmovetracker.x = 0;

				repaint(true);
			}
		}
		break;

		case VK_NEXT:
		{
			if ( maxPart != 0)
			{
				currentPart++;
				if (currentPart > maxPart)
					currentPart = 0;

				oldmovetracker.x = 0;

				repaint(true);
			}
		}
		break;

		case VK_RIGHT:
		{
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( (*eit) == hoverentry && (*eit)->entryType < 0 )
				{
					lastactiveentry = (*eit);
					bool alreadyCreated = false;
					for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
					{
						if (_stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
						{
							(*plit)->hoverentry = (*plit)->startentry;
							(*plit)->show(true);
							alreadyCreated = true;
						}
						else
						{
							if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
								(*plit)->hide();
						}
					}
					if (!alreadyCreated && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
						CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit), true);
					else if (!alreadyCreated && (*eit)->clsid != 0)
						CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit)->clsid, (*eit), true);
					else if (!alreadyCreated && (*eit)->hWnd == hWnd)
						CreateTaskSubMenu( ("sub_"+(*eit)->caption), (*eit), true);
				
					break;
				}
			}
		}
		break;

		case VK_LEFT:
		{
			if (pParentPopup != NULL)
			{
				if ( pParentPopup->lastactiveentry != NULL )
				{
					pParentPopup->hoverentry = pParentPopup->lastactiveentry;
					SetCursorPos(pParentPopup->actualx+pParentPopup->lastactiveentry->x+2, pParentPopup->actualy+pParentPopup->lastactiveentry->y+2);	
				}
				else if ( pParentPopup->startentry != NULL )
				{
					pParentPopup->hoverentry = pParentPopup->startentry;
					SetCursorPos(pParentPopup->actualx+pParentPopup->startentry->x+2, pParentPopup->actualy+pParentPopup->startentry->y+2);	
				}

				if ( !PinnedMode && visible )
					hide();

				pParentPopup->repaint();

				SetForegroundWindow(pParentPopup->gethWnd());
				SetActiveWindow(pParentPopup->gethWnd());
				SetFocus(pParentPopup->gethWnd());
			}
		}
		break;

		case VK_ESCAPE:
		{
			if ( HideAllowed )
			{
				//Search for Main Popup and Hide All SubPopups
				Popup* tmpPopupPointer = pParentPopup;
				if (tmpPopupPointer != NULL)
				{
					while (tmpPopupPointer->pParentPopup != NULL)
						tmpPopupPointer = tmpPopupPointer->pParentPopup;

					if ( !tmpPopupPointer->PinnedMode && tmpPopupPointer->isVisible() )
						tmpPopupPointer->hide();
				}
				else
				{
					if ( !PinnedMode && visible )
						hide();
				}
			}
		}
		break;

		case VK_UP:
		{
			for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
			{
				if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
					(*plit)->hide();
			}

			//Bool Value: True = Upwards, False = downwards
			repaint(SelectNextEntry(true));
		}
		break;

		case VK_DOWN:
		{
			for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
			{
				if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
					(*plit)->hide();
			}

			//Bool Value: True = Upwards, False = downwards
			repaint(SelectNextEntry(false));
		}
		break;

		case VK_RETURN:
		{
			POINT pt;
			GetCursorPos(&pt);
			ScreenToClient(hWnd, &pt);

			ActionAtPoint(pt.x, pt.y, true);
		}
		break;

		default:
		{
			int howmuchfolders = 0;
			int howmuchfiles = 0;
			int iWhatFound = 0;
			int entrycounter = 0;
			
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				entrycounter++;
				if ( (*eit)->keyCode == nKey && (*eit)->entryType < 0 )
					howmuchfolders++;
				else if ( (*eit)->keyCode == nKey && (*eit)->entryType >= 0 )
					howmuchfiles++;

				if ( hoverentry == (*eit) && (*eit)->keyCode == nKey )
					lastkeyentry = entrycounter;
			}

			if ( howmuchfolders == 0 && howmuchfiles == 0 )
				break;
	
			entrycounter = 0;
			for (eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				entrycounter++;
				if ( ((*eit)->keyCode == nKey || (inChordingTimeOut && (*eit)->keyCode1 == nKey)) && entrycounter > lastkeyentry )
				{
					if ( (*eit)->entryType < 0 )
						lastactiveentry = (*eit);
					hoverentry = (*eit);
					lastkeyentry = entrycounter;
				
					if ( (*eit)->hiddenPart != currentPart )
						currentPart = (*eit)->hiddenPart;

					SetCursorPos(actualx+(*eit)->x+2, actualy+(*eit)->y+2);

					iWhatFound = (((*eit)->entryType < 0 && !(*eit)->action.empty()) ? 2 : 1);

					break;
				}
			}
			if (lastkeyentry != 0 && iWhatFound == 0)
			{
				entrycounter = 0;
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					entrycounter++;
					if ( (*eit)->keyCode == nKey || (inChordingTimeOut && (*eit)->keyCode1 == nKey) )
					{
						if ( (*eit)->entryType < 0 )
							lastactiveentry = (*eit);
						hoverentry = (*eit);
						lastkeyentry = entrycounter;
					
						if ( (*eit)->hiddenPart != currentPart )
							currentPart = (*eit)->hiddenPart;

						SetCursorPos(actualx+(*eit)->x+2, actualy+(*eit)->y+2);

						iWhatFound = (((*eit)->entryType < 0 && !(*eit)->action.empty()) ? 2 : 1);

						break;
					}
				}
			}

			repaint(true);

			//One Folder ->Open It
			if (iWhatFound == 2 && howmuchfolders == 1 && howmuchfiles == 0 && lastactiveentry != NULL)
			{
				bool alreadyCreated = false;
				for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
				{
					if (_stricmp( lastactiveentry->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
					{
						(*plit)->hoverentry = (*plit)->startentry;
						(*plit)->show(true);
						alreadyCreated = true;
					}
					else
					{
						if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
							(*plit)->hide();
					}
				}
				if (!alreadyCreated && lastactiveentry->clsid == 0 && lastactiveentry->hWnd == NULL)
					CreateSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry->action, lastactiveentry, true);
				else if (!alreadyCreated && lastactiveentry->clsid != 0)
					CreateShellSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry->clsid, lastactiveentry,  true);
				else if (!alreadyCreated && lastactiveentry->hWnd == hWnd)
					CreateTaskSubMenu( ("sub_"+lastactiveentry->caption), lastactiveentry, true);
			}
			//One File -> Open It
			else if (iWhatFound == 1 && howmuchfolders == 0 && howmuchfiles == 1 && hoverentry != NULL)
			{
				//Normal Action
				if ( hoverentry->entryType >= 0 && (!hoverentry->action.empty() || !hoverentry->universalaction.empty()) && hoverentry->m_pidl == NULL )
				{
					ENTRY *tmpEntry = hoverentry;

					Popup *tmpPointer = pParentPopup;
					if (tmpPointer != NULL)
					{
						while (tmpPointer->pParentPopup != NULL)
							tmpPointer = tmpPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}

					if ( !tmpEntry->universalaction.empty() )
						LSExecute(NULL, tmpEntry->universalaction.c_str(), NULL);
					else
						LSExecute(NULL, tmpEntry->action.c_str(), NULL);
					
					break;
				}
				//System Action
				else if ( hoverentry->entryType >= 0 && hoverentry->m_pidl != NULL )
				{
					ENTRY *tmpEntry = hoverentry;

					Popup *tmpPointer = pParentPopup;
					if (tmpPointer != NULL)
					{
						while (tmpPointer->pParentPopup != NULL)
							tmpPointer = tmpPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}

					Invoke(tmpEntry->m_pidl);
					
					break;
				}
				//Tasks
				else if ( hoverentry->hWnd != NULL )
				{
					ENTRY *tmpEntry = hoverentry;

					Popup *tmpPointer = pParentPopup;
					if (tmpPointer != NULL)
					{
						while (tmpPointer->pParentPopup != NULL)
							tmpPointer = tmpPointer->pParentPopup;

						if ( !execSound.empty() )
						{
							tmpPointer->execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						tmpPointer->hide(true);
					}
					else
					{
						if ( !execSound.empty() )
						{
							execSoundPlayed = true;
							ParseBangCommand(hWnd, "!playsound", execSound.c_str());
						}
						hide(true);
					}

					SetForegroundWindow( tmpEntry->hWnd );

					if ( IsIconic(tmpEntry->hWnd) )
					{
						//if ( pSwitchToThisWindow )
						//	pSwitchToThisWindow( tmpEntry->hWnd, true );
						//else
							ShowWindowAsync( tmpEntry->hWnd, SW_RESTORE );
							BringWindowToTop( tmpEntry->hWnd );
					}
					else
					{
						//if ( pSwitchToThisWindow )
						//	pSwitchToThisWindow( tmpEntry->hWnd, false );
						//else
							ShowWindowAsync( tmpEntry->hWnd, SW_SHOW );
							BringWindowToTop( tmpEntry->hWnd );
					}

					SendMessage(GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM)tmpEntry->hWnd);
					break;
				}
			}
			else
			{
				inChordingTimeOut = true;
				SetTimer(hWnd, TIMER_CHORDING, 750, 0);
			}
		}
		break;
	}
}

void Popup::onMouseMove(int x, int y)
{
	if ( mousePressed )
	{
		KillTimer(hWnd, TIMER_MOUSETRACK);

		isMultiScrollTimerRunning = 0;
		KillTimer(hWnd, TIMER_MULTISCROLLUP);
		KillTimer(hWnd, TIMER_MULTISCROLLDOWN);

		POINT newpt, pt;
		GetCursorPos(&newpt);
		pt = newpt;
		
		newpt.x = newpt.x-savedpt.x;
		newpt.y = newpt.y-savedpt.y;

		if ( !mouseMoved && newpt.x != actualx && newpt.y != actualy )
			mouseMoved = true;

		manualMove = false;

		if ( mouseMoved )
			reposition(1, newpt.x, newpt.y, 0, 0);
	}
	else if ( mouseInside )
	{
		SetTimer(hWnd, TIMER_MOUSETRACK, trackDelay, NULL);

		manualMove = true;

		HoverActions(x, y);
	}

	if( !mouseInside )
	{
		mouseInside = true;
		SetTimer(hWnd, TIMER_MOUSETRACK, trackDelay, NULL);

		if ( visiblepopupList.size() == 1 )
		{
			SetForegroundWindow(hWnd);
			SetActiveWindow(hWnd);
			SetFocus(hWnd);
			BringWindowToTop(hWnd);
		}

		if ( startentry != NULL )
			lastactiveentry = startentry;

		HoverActions(x, y);
	}
}

void Popup::repaint(bool invalidateCache, bool regionUpdate)
{
	if (invalidateCache)
	{
		resetBackground = true;
	}
	
	if ( !alphaMap && !fakealphaMap )
	{
		RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW );	
			
		if ( invalidateCache || regionUpdate || transparencyNeededAlways )
			updateRegion();
	}
	else
	{
		UpdateWindowTransparent(hWnd, alphaTransparency);
	}
}

void Popup::ResetBackground()
{
	if ( defaultBitmap )
	{
		defaultBitmap = (HBITMAP) SelectObject(defaultDC, olddefaultBMP);
		DeleteObject(defaultBitmap);
	}
	if ( defaultDC )
		DeleteObject(defaultDC);
	if ( olddefaultBMP )
		DeleteObject(olddefaultBMP);

	defaultBitmap = NULL;
	defaultDC = NULL;
	olddefaultBMP = NULL;

	if(fadeBackupBitmap)
	{
		fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
		DeleteObject(fadeBackupBitmap);
	}
	if (fadeBackupDC)
		DeleteDC(fadeBackupDC);
	if ( oldfadeBackupBMP )
		DeleteObject(oldfadeBackupBMP);

	fadeBackupBitmap = NULL;
	fadeBackupDC = NULL;
	oldfadeBackupBMP = NULL;
}

void Popup::updateRegion()
{
	HDC tmpDC = CreateCompatibleDC(NULL);
	HBITMAP tmpBmp = CreateBitmap(actualwidth, actualheight, 1, bitsperpixel, NULL);
	HBITMAP oldBMP = (HBITMAP) SelectObject(tmpDC, tmpBmp);

	onPaint(tmpDC);
					
	tmpBmp = (HBITMAP) SelectObject(tmpDC, oldBMP);

	HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
	HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
	CombineRgn(windowRgn, region, NULL, RGN_COPY);

	SetWindowRgn(hWnd, windowRgn, true);	

	DeleteObject(windowRgn);
	DeleteObject(region);

	DeleteObject(tmpBmp);
	DeleteDC(tmpDC);
}

void Popup::onPaint(HDC hDC)
{
	if ( resetBackground )
	{
		resetBackground = false;
		ResetBackground();
	}

	int width = actualwidth;
	int height = actualheight;

	// make a rendition of the background
	HDC	backgroundDC = CreateCompatibleDC(hDC);
	HBITMAP backgroundBitmap = CreateCompatibleBitmap(hDC, width, height);
	HBITMAP oldbackgroundBMP = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
		
	//Normal Background Painting
	popuptexture->apply(backgroundDC, 0, 0, width, height);
	OverlayPopupPainter(backgroundDC, backgroundBitmap);

	// create & keep a cached rendition of the default popupmenu
	if (defaultBitmap == NULL)
	{
		if (defaultDC == NULL)
			defaultDC = CreateCompatibleDC(hDC);

		defaultBitmap = CreateCompatibleBitmap(hDC, width, height);
		olddefaultBMP = (HBITMAP) SelectObject(defaultDC, defaultBitmap);
		
		// blt background into double buffer
		BitBlt(defaultDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);

		//Only Painted if NOT Shaded
		if ( shaded == -1 )
		{
			int state = 0;
			int arrowstate = 0;
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType != 2 )
				{
					//initialize
					state = 0;
					arrowstate = 0;

					if ( (*eit)->entryType == 1 )
					{
						state = 10;
					}
					else if ( (*eit)->entryType == 0 )
					{
						state = 1;
					}
					else if ( (*eit)->entryType < 0 )
					{
						state = 3;
						arrowstate = 31;
					}
					else
					{
						continue;
					}
				
					//Paint Entry Texture
					buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
					OverlayButtonPainter( defaultDC, NULL, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
			
					//Paint Arrow
					if (arrowstate != 0)
					{
						buttontexture->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate );
						OverlayButtonPainter( defaultDC, NULL, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate );
						buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
					}
					
					//If No Separator, Icon and Text Painting is valid
					if (state != 10)
					{
						buttonicon->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->m_pidl, DefaultIcon, alphaMap );

						if (quicklaunchMode == 0)
							buttonfont->apply( defaultDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
					}
				}
			}
		}
	}

	// double buffer for flicker-free paint
	HDC bufferDC = CreateCompatibleDC(hDC);
	HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
	HBITMAP oldbufferBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	// blt default background into double buffer
	BitBlt(bufferDC, 0, 0, width, height, defaultDC, 0, 0, SRCCOPY);
	
	CString helpcaption = captionname.c_str();
	if (helpcaption.Find('&') != helpcaption.Find("& ") && helpcaption.Find('&') != helpcaption.Find("&\r") && helpcaption.Find('&') != helpcaption.Find("&\n") )
		helpcaption.Replace("&", "");
	
	if ( quicklaunchMode == 0 )
	{
		if ( topBorder > 0 )
		{
			buttontexture->apply( bufferDC, 0, 0, actualwidth, topBorder, -1 );
			OverlayButtonPainter( bufferDC, NULL, 0, 0, actualwidth, topBorder, -1 );
		}
		if ( bottomBorder > 0 )
		{
			buttontexture->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, -2 );
			OverlayButtonPainter( bufferDC, NULL, 0, actualheight-bottomBorder, actualwidth, bottomBorder, -2 );
		}
		if (showCaption == 1)
		{
			if ( pParentPopup != NULL && pParentEntry != NULL )
				buttonicon->apply( bufferDC, 0, 0, actualwidth, topBorder, 0, pParentEntry->iconPath, pParentEntry->hWnd, pParentEntry->m_pidl, "", alphaMap );
			if ( PinnedMode )
				buttonicon->apply( bufferDC, 0, 0, actualwidth, topBorder, -1, "", NULL, NULL, "", alphaMap );

			buttonfont->apply( bufferDC, 0, 0, actualwidth, topBorder, (string) helpcaption, 0);
		}
		else if (showCaption == 2)
		{
			if ( pParentPopup != NULL && pParentEntry != NULL )
				buttonicon->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, 0, pParentEntry->iconPath, pParentEntry->hWnd, pParentEntry->m_pidl, "", alphaMap );
			if ( PinnedMode )
				buttonicon->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, -1, "", NULL, NULL, "", alphaMap );

			buttonfont->apply( bufferDC, 0, actualheight-bottomBorder, actualwidth, bottomBorder, (string) helpcaption, 0);
		}
		else if (showCaption == 3)
		{
			if ( pParentPopup != NULL && pParentEntry != NULL )
				buttonicon->apply( bufferDC, 0, 0, actualwidth, actualheight, 0, pParentEntry->iconPath, pParentEntry->hWnd, pParentEntry->m_pidl, "", alphaMap );
			if ( PinnedMode )
				buttonicon->apply( bufferDC, 0, 0, actualwidth, actualheight, -1, "", NULL, NULL, "", alphaMap );

			buttonfont->apply( bufferDC, 0, 0, actualwidth, actualheight, captionname, 0);
		}
	}
	//Only Painted if NOT Shaded
	if ( shaded == -1 )
	{
		int state = 0;
		int arrowstate = 0;
		if ( quicklaunchMode == 0 && hasInfoEntry )
		{
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if ( currentPart == (*eit)->hiddenPart && (*eit)->entryType == 2 )
				{
					state = 5;

					//Normal Background Painting behind ActiveEntry
					BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

					//Paint Entry Texture
					buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
					OverlayButtonPainter( bufferDC, NULL, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );
					
					buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
				}
			}
		}
		for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
		{
			//initialize
			state = 0;
			arrowstate = 0;
		
			if ( currentPart == (*eit)->hiddenPart && (*eit) == hoverentry && (*eit) != NULL)
			{
				if ( (*eit)->entryType == 0 )
				{
					state = 2;
				}
				else if ( (*eit)->entryType < 0 )
				{
					state = 4;
					arrowstate = 41;
				}
				else
				{
					continue;
				}
			
				//Normal Background Painting behind ActiveEntry
				BitBlt(bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, backgroundDC, (*eit)->x, (*eit)->y, SRCCOPY);

				//Paint Entry Texture
				buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state);
				OverlayButtonPainter( bufferDC, NULL, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state );

				//Paint Arrow
				if (arrowstate != 0)
				{
					buttontexture->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate);
					OverlayButtonPainter( bufferDC, NULL, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, arrowstate );
					buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, arrowstate );
				}
						
				//If No Info, Icon is valid
				buttonicon->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, state, (*eit)->iconPath, (*eit)->hWnd, (*eit)->m_pidl, DefaultIcon, alphaMap );

				if (quicklaunchMode == 0)
					buttonfont->apply( bufferDC, (*eit)->x, (*eit)->y, (*eit)->width, (*eit)->height, (*eit)->caption, state );
			}
		}
	}

	// blt the double buffer to the display
	BitBlt(hDC, 0, 0, width, height, bufferDC, 0, 0, SRCCOPY);

	backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, oldbackgroundBMP);
	DeleteObject(backgroundBitmap);
	DeleteDC(backgroundDC);	
	DeleteObject(oldbackgroundBMP);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldbufferBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
	DeleteObject(oldbufferBMP);
}

bool Popup::PopupActive()
{
	char tmpClass[MAX_LINE_LENGTH] = {0};
			
	POINT pt;
	GetCursorPos(&pt);
			
	GetClassName(WindowFromPoint(pt), tmpClass, MAX_LINE_LENGTH);

	if (_stricmp(tmpClass, "xpopup") == NULL	|| _stricmp(tmpClass, "xpopupxp") == NULL)
		return true;
	else if ( autoHide )
		return false;

	GetClassName(GetForegroundWindow(), tmpClass, MAX_LINE_LENGTH);

	if (_stricmp(tmpClass, "xpopup") == NULL	|| _stricmp(tmpClass, "xpopupxp") == NULL)
		return true;

	GetClassName(GetActiveWindow(), tmpClass, MAX_LINE_LENGTH);

	if (_stricmp(tmpClass, "xpopup") == NULL	|| _stricmp(tmpClass, "xpopupxp") == NULL)
		return true;

	GetClassName(GetFocus(), tmpClass, MAX_LINE_LENGTH);

	if (_stricmp(tmpClass, "xpopup") == NULL	|| _stricmp(tmpClass, "xpopupxp") == NULL)
		return true;

	return false;
}


void Popup::onTimer(int timerID)
{
	switch(timerID)
	{	
		case TIMER_MOUSETRACK:
		{
			POINT pt;
			GetCursorPos(&pt);

			if (oldmovetracker.x != pt.x || oldmovetracker.y != pt.y)
			{
				oldmovetracker = pt;
				RECT rc;
				GetWindowRect(hWnd, &rc);

				if(!PtInRect(&rc, pt) || !visible)
				{
					//To be on the save side
					ReleaseCapture();
					mousePressed = false;

					KillTimer(hWnd, TIMER_MOUSETRACK);
					
					isMultiScrollTimerRunning = 0;
					KillTimer(hWnd, TIMER_MULTISCROLLUP);
					KillTimer(hWnd, TIMER_MULTISCROLLDOWN);

					mouseInside = false;
					lastkeyentry = -1;

					if ( hoverentry != NULL ) 
					{
						if ( hoverentry->entryType >= 0 )
						{
							hoverentry = NULL;
							repaint();
						}
						else
						{
							for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
							{
								if (_stricmp( hoverentry->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
								{
									if ( (*plit)->isVisible() )
										return;
								}
							}
							hoverentry = NULL;
							repaint();
						}
					}

					break;
				}
			}
		}
		break;

		case TIMER_ALPHAFADEIN:
		{
			if (UpdateAlpha <= alphaTransparency-alphaFadeStep)
			{
				isFading = true;

				UpdateAlpha += alphaFadeStep;
				UpdateWindowTransparent(hWnd, UpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEIN);
				if (UpdateAlpha != alphaTransparency)
					UpdateWindowTransparent(hWnd, alphaTransparency);

				isFading = false;

				if(fadeBackupBitmap)
				{
					fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
					DeleteObject(fadeBackupBitmap);
				}
				if (fadeBackupDC)
					DeleteDC(fadeBackupDC);
				if ( oldfadeBackupBMP )
					DeleteObject(oldfadeBackupBMP);

				fadeBackupBitmap = 0;
				fadeBackupDC = 0;
				oldfadeBackupBMP = 0;
			}
		}
		break;

		case TIMER_ALPHAFADEOUT:
		{
			if (UpdateAlpha >= alphaFadeStep)
			{
				isFading = true;

				UpdateAlpha -= alphaFadeStep;
				UpdateWindowTransparent(hWnd, UpdateAlpha);		
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEOUT);
				UpdateAlpha = 0;
				UpdateWindowTransparent(hWnd, UpdateAlpha);
				ShowWindow(hWnd, SW_HIDE);

				lastactiveentry = NULL;
				hoverentry = NULL;
				lastkeyentry = -1;
				
				visible = false;

				isFading = false;

				if(fadeBackupBitmap)
				{
					fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
					DeleteObject(fadeBackupBitmap);
				}
				if (fadeBackupDC)
					DeleteDC(fadeBackupDC);
				if ( oldfadeBackupBMP )
					DeleteObject(oldfadeBackupBMP);

				fadeBackupBitmap = 0;
				fadeBackupDC = 0;
				oldfadeBackupBMP = 0;
			}
		}
		break;

		case TIMER_UPDATE:
		{
			if ( !visible )
				break;

			if (hasInfoEntry)
			{
				bool needRepaint = false;
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					if ( !(*eit)->info.empty() )
					{
						string info = InfoUpdate( (*eit)->info );
						if (!needRepaint && _stricmp( (*eit)->caption.c_str() , info.c_str() ) != NULL )
							needRepaint = true;
						(*eit)->caption = info;
					}
				}
				if (needRepaint)
					repaint(false, true);
			}
			
			if (PinnedMode && isTaskPopup)
			{
				KillTimer(hWnd, TIMER_UPDATE);
				ScanTasks();

				//In Screen Check of Tasks Popup
				int x = actualx;
				int y = actualy;

				if ( x+actualwidth > screenWidth || snappedCode == 20 || snappedCode == 21 || snappedCode == 23 )
					x = screenWidth - actualwidth;
				else if ( snappedCode == 40 || snappedCode == 41 || snappedCode == 43 )
					x = 0;

				if ( y+actualheight > screenHeight || snappedCode == 3 || snappedCode == 23 || snappedCode == 43 )
					y = screenHeight - actualheight;
				else if ( snappedCode == 1 || snappedCode == 21 || snappedCode == 41 )
					y = 0;

				//Minimum x, y is screenX/Y
				if ( x < screenX )
					x = screenX;
				if ( y < screenY )
					y = screenY;

				reposition(1, x, y, 0, 0);
			}
			else if (PinnedMode && hasDynamicContent)
			{
				KillTimer(hWnd, TIMER_UPDATE);
				ParseStepRC(false);

				//In Screen Check of Tasks Popup
				int x = actualx;
				int y = actualy;

				if ( x+actualwidth > screenWidth || snappedCode == 20 || snappedCode == 21 || snappedCode == 23 )
					x = screenWidth - actualwidth;
				else if ( snappedCode == 40 || snappedCode == 41 || snappedCode == 43 )
					x = 0;

				if ( y+actualheight > screenHeight || snappedCode == 3 || snappedCode == 23 || snappedCode == 43 )
					y = screenHeight - actualheight;
				else if ( snappedCode == 1 || snappedCode == 21 || snappedCode == 41 )
					y = 0;

				//Minimum x, y is screenX/Y
				if ( x < screenX )
					x = screenX;
				if ( y < screenY )
					y = screenY;

				reposition(1, x, y, 0, 0);
			}
		}
		break;

		case TIMER_ACTIVE:
		{	
			if ( PopupActive() )
				break;

			if ( PinnedMode )
			{
				if (hoverentry != NULL)
					hoverentry = NULL;

				repaint();
			}
			
			hide();
		}
		break;

		case TIMER_HOVER:
		{
			KillTimer(hWnd, TIMER_HOVER);

			ENTRY *entry = NULL;
	
			POINT pt;
			GetCursorPos(&pt);
			ScreenToClient(hWnd, &pt);

			if ( lastactiveentry != NULL )
			{
				entry = lastactiveentry;
				
				if ( pt.x > (entry->x + entry->width) || pt.x < entry->x 
						|| pt.y > (entry->y + entry->height) || pt.y < entry->y 
						|| entry->hiddenPart != currentPart )
				{	
					break;
				}
			}
			else
			{
				for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
				{
					if (currentPart != (*eit)->hiddenPart)
						continue;

					if ( pt.x < ((*eit)->x + (*eit)->width) && pt.x >= (*eit)->x )
					{
						if ( pt.y < ((*eit)->y + (*eit)->height) && pt.y >= (*eit)->y )
						{
							entry = (*eit);
							break;
						}
					}
				}
			}

			if (entry != NULL)
			{
				//Check if mouse still hovers over old entry

				bool alreadyCreated = false;
				for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
				{
					if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
					{
						(*plit)->hide();
						if ( (*plit)->pParentPopup != NULL)
						{
							SetForegroundWindow((*plit)->pParentPopup->gethWnd());
							SetActiveWindow((*plit)->pParentPopup->gethWnd());
							SetFocus((*plit)->pParentPopup->gethWnd());
						}
					}
				}
				for(plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
				{
					if (_stricmp( entry->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
					{
						(*plit)->show();

						if ( ToolTips && (quicklaunchMode != 0 || tooltipMode != 0) )
						{	
							if ( entry->tooltip.empty() )
							{
								if ( quicklaunchMode != 0 || tooltipMode == 1 )
								{
									CString worker = "";
									worker = TrimAndStripQuote(entry->action).c_str();
									worker = worker.Right(worker.GetLength()-worker.ReverseFind('\\')-1);
									if (_stricmp(worker.Right(4), ".lnk") == NULL)
										worker = worker.Left(worker.GetLength()-4);

									entry->tooltip = worker;
								}
							}

							updateHint(hWnd, (char*) entry->tooltip.c_str(), true);
						}

						alreadyCreated = true;		
					}
				}
				if (!alreadyCreated && entry->clsid == 0 && entry->hWnd == NULL)
					CreateSubMenu( ("sub_"+entry->caption), entry->action, entry);
				else if (!alreadyCreated && entry->clsid != 0)
					CreateShellSubMenu( ("sub_"+entry->caption), entry->clsid, entry);
				else if (!alreadyCreated && entry->hWnd == hWnd)
					CreateTaskSubMenu( ("sub_"+entry->caption), entry);
			}
		}
		break;

		case TIMER_CHORDING:
		{
			KillTimer(hWnd, TIMER_CHORDING);
			inChordingTimeOut = false;
		}
		break;

		case TIMER_AUTOUNLOCK:
		{
			KillTimer(hWnd, TIMER_AUTOUNLOCK);
			lockKeyboard = false;
		}
		break;

		case TIMER_MULTISCROLLDOWN:
		{
			//Next
			currentPart++;
			if (currentPart > maxPart)
				currentPart = 0;

			oldmovetracker.x = 0;

			repaint(true);
		}
		break;

		case TIMER_MULTISCROLLUP:
		{
			//Prev
			currentPart--;
			if (currentPart < 0)
				currentPart = maxPart;
			
			oldmovetracker.x = 0;

			repaint(true);
		}
		break;
	}
}

void Popup::onWindowPosChanging(WINDOWPOS *windowPos)
{
	if ( (windowPos->flags & SWP_NOMOVE) != 0 )
		return;

	int nDist = 10;
	int result = 0;

	int snaphoriz = 0;
	int snapvert = 0;

	// left edge
	if ( windowPos->x < customWorkArea.left+nDist && (windowPos->x > customWorkArea.left-nDist || restrictToWorkArea) )
	{
		windowPos->x = customWorkArea.left;
		snaphoriz = 4;
		result = 1;
	}

	// top edge
	if ( windowPos->y < customWorkArea.top+nDist && (windowPos->y > customWorkArea.top-nDist || restrictToWorkArea) )
	{
		windowPos->y = customWorkArea.top;
		snapvert = 1;
		result = 1;
	}

	// bottom edge
	if ( (windowPos->y+actualheight < customWorkArea.bottom+nDist || restrictToWorkArea) && windowPos->y+actualheight > customWorkArea.bottom-nDist)
	{
		windowPos->y = customWorkArea.bottom - actualheight;
		snapvert = 3;
		result = 1;
	}

	// right edge
	if ( (windowPos->x+actualwidth < customWorkArea.right+nDist || restrictToWorkArea) && windowPos->x+actualwidth > customWorkArea.right-nDist)
	{
		windowPos->x = customWorkArea.right - actualwidth;
		snaphoriz = 2;
		result = 1;
	}

	snappedCode = 10*snaphoriz + snapvert;

	if ( result != 0 )
		return;
			
	POINT pos = {windowPos->x, windowPos->y};
	POINT size = {actualwidth, actualheight};

	for( PopupListIterator plit = visiblepopupList.begin(); plit != visiblepopupList.end(); plit++)
	{
		if ( (*plit) == this )
			continue;

		result = isCloseTo( pos, size, (*plit)->actualx, (*plit)->actualy, (*plit)->actualwidth, (*plit)->actualheight, nDist);
		
		if (result != 0)
			break;
	}

	if (result == 1)
	{
		windowPos->x = pos.x;
	}
	else if (result == 2)
	{
		windowPos->y = pos.y;
	}
	else if (result == 3)
	{
		windowPos->x = pos.x;
		windowPos->y = pos.y;
	}
}

void Popup::onMove(int x, int y)
{
	actualx = x;
	actualy = y;

	if ( !BangToCall.empty() )
	{
		CString help = BangToCall.c_str();
		help.Replace("!", "");

		PrefixedLSSetVariable((string)help, "CurrentX", IntToString( x ));
		PrefixedLSSetVariable((string)help, "CurrentY", IntToString( y ));
	}
}

void Popup::onSize(int width, int height)
{
	actualwidth = width;
	actualheight = height;

	if (alphaMap || fakealphaMap)
		SetWindowRgn(hWnd, NULL, true);
	
	if ( !BangToCall.empty() )
	{
		CString help = BangToCall.c_str();
		help.Replace("!", "");

		PrefixedLSSetVariable((string)help, "CurrentWidth", IntToString( width ));
		PrefixedLSSetVariable((string)help, "CurrentHeight", IntToString( height ));
	}
}

BOOL Popup::onWindowMessage( UINT nMessage, WPARAM wParam, LPARAM lParam, LRESULT &lResult )
{
	lResult = 0;

	if ((0x8000 & GetKeyState(VK_CONTROL)) && (0x8000 & GetKeyState(VK_SHIFT)) )
	{
		if (HideAllowed && PinnedMode && (0x8000 & GetKeyState(VK_UP)))
		{
			PinnedMode = false;
			setAlwaysOnTop(true);

			return TRUE;
		}
		else if (!PinnedMode && (0x8000 & GetKeyState(VK_DOWN)))
		{
			PinnedMode = true;
			if (pinnedPopupNotOnTop)
				setAlwaysOnTop(false);
			else 
				setAlwaysOnTop(true);
			SetTimer(hWnd, TIMER_ACTIVE, hideDelay, NULL);

			return TRUE;
		}
	}

	switch ( nMessage )
	{
		case LM_HIDEPOPUP:
		{
			hide();
			return false;
		}

		case WM_CLOSE:
		{
			lResult = 0;
			return true;
		}

		case WM_LBUTTONDBLCLK:
		{
			onLButtonDblClk(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
			return true;
		}

		case WM_LBUTTONDOWN:
		{
			ontemplateButtonDown(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), titleLClickAction, bottomLClickAction);
			return true;
		}

		case WM_LBUTTONUP:
		{
			ontemplateButtonUp(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), titleLClickAction, bottomLClickAction, 1);
			return true;
		}

		case WM_MBUTTONDOWN:
		{
			ontemplateButtonDown(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), titleMClickAction, bottomMClickAction);
			return true;
		}

		case WM_MBUTTONUP:
		{
			ontemplateButtonUp(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), titleMClickAction, bottomMClickAction, 3);
			return true;
		}

		case WM_RBUTTONDOWN:
		{
			ontemplateButtonDown(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), titleRClickAction, bottomRClickAction);
			return true;
		}

		case WM_RBUTTONUP:
		{
			ontemplateButtonUp(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), titleRClickAction, bottomRClickAction, 2);
			return true;
		}

		case WM_MOUSEWHEEL:
		{
			POINT pt;
			RECT childPosition;

			GetCursorPos(&pt);

			char tmpClass[MAX_LINE_LENGTH] = {0};				
			
			for(HWND hWndChild = ::GetTopWindow(hWnd); hWndChild != NULL; hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT))
			{
				GetWindowRect(hWndChild, &childPosition);
				if ( PtInRect(&childPosition, pt) )
				{
					GetClassName(hWndChild, tmpClass, MAX_LINE_LENGTH);
					if (_stricmp(tmpClass, "xlabel") == NULL	|| _stricmp(tmpClass, "xlabelxp") == NULL || _stricmp(tmpClass, "xtray") == NULL || _stricmp(tmpClass, "xtaskbar") == NULL)
					{
						ScreenToClient(hWndChild, &pt);
						::PostMessage(hWndChild, WM_MOUSEWHEEL, wParam, MAKELPARAM((short) pt.x, (short) pt.y));
								
						return true;
					}

					return true;
				}
			}

			ScreenToClient(hWnd, &pt);
			if ((short)(HIWORD(wParam)) < 0)
			{
				onWheelDown(pt.x, pt.y);

				return true;
			}
			else
			{
				onWheelUp(pt.x, pt.y);

				return true;
			}
		}

		case WM_XBUTTONDOWN:
		{
			return true;
		}

		case WM_XBUTTONUP:
		{
			char tmpClass[MAX_LINE_LENGTH] = {0};
			
			POINT pt;
			GetCursorPos(&pt);
			HWND window = WindowFromPoint(pt);

			if ( window != hWnd )
			{
				SendMessage(window, nMessage, wParam, lParam);
				return true;
			}
			else
			{
				if (HIWORD(wParam) == XBUTTON1)
				{
					if (pParentPopup != NULL)
					{
						if (pParentPopup->lastactiveentry != NULL)
						{
							pParentPopup->hoverentry = pParentPopup->lastactiveentry;
							SetCursorPos(pParentPopup->actualx+pParentPopup->lastactiveentry->x+2, pParentPopup->actualy+pParentPopup->lastactiveentry->y+2);	
						}
						else if ( pParentPopup->startentry != NULL )
						{
							pParentPopup->hoverentry = pParentPopup->startentry;
							SetCursorPos(pParentPopup->actualx+pParentPopup->startentry->x+2, pParentPopup->actualy+pParentPopup->startentry->y+2);	
						}

						if ( !PinnedMode && visible )
							hide();

						pParentPopup->repaint();

						SetForegroundWindow(pParentPopup->gethWnd());
						SetActiveWindow(pParentPopup->gethWnd());
						SetFocus(pParentPopup->gethWnd());
					}
				}
				else if (HIWORD(wParam) == XBUTTON2)
				{
					for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
					{
						if ( (*eit) == hoverentry && (*eit)->entryType < 0 )
						{
							lastactiveentry = (*eit);
							bool alreadyCreated = false;
							for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
							{
								if (_stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
								{
									(*plit)->hoverentry = (*plit)->startentry;
									(*plit)->show(true);
									alreadyCreated = true;
								}
								else
								{
									if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
										(*plit)->hide();
								}
							}
							if (!alreadyCreated && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
								CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit), true);
							else if (!alreadyCreated && (*eit)->clsid != 0)
								CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit)->clsid, (*eit), true);
							else if (!alreadyCreated && (*eit)->hWnd == hWnd)
								CreateTaskSubMenu( ("sub_"+(*eit)->caption), (*eit), true);
						
							break;
						}
					}
				}
			}
			
			return true;
		}
	
		case WM_MOUSEMOVE:
		{
			onMouseMove( GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) );
			return true;
		}
	
		case WM_PAINT:
		{
			PAINTSTRUCT ps;

			HDC hDC;

			hDC = BeginPaint( hWnd, &ps );

			onPaint( hDC );

			EndPaint( hWnd, &ps );
			
			return true;
		}

		case WM_WINDOWPOSCHANGING:
		{
			if ( !manualMove || restrictToWorkArea )
				onWindowPosChanging((WINDOWPOS *) lParam);
			
			return true;
		}

		case WM_MOVE:
		{
			onMove( GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) );
			return true;
		}

		case WM_SIZE:
		{
			onSize( GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) );
			return true;
		}

		case WM_TIMER:
		{
			onTimer((int) wParam);
			return true;
		}

		case WM_KEYDOWN:	// catches small letters and swedish keys..
		{
			if ( lockKeyboard )
				return true;

			char tmpClass[MAX_LINE_LENGTH] = {0};
			
			POINT pt;
			GetCursorPos(&pt);
			HWND window = WindowFromPoint(pt);

			GetClassName(window, tmpClass, MAX_LINE_LENGTH);

			if ( window != hWnd && (_stricmp(tmpClass, "xpopup") == NULL || _stricmp(tmpClass, "xpopupxp") == NULL) )
			{
				SendMessage(window, nMessage, wParam, lParam);
				return true;
			}
			else
			{
				lockKeyboard = true;
				SetTimer(hWnd, TIMER_AUTOUNLOCK, 100, NULL);
				onKey(wParam);
				return true;
			}
		}

		case WM_KEYUP:
		{
			KillTimer(hWnd, TIMER_AUTOUNLOCK);
			lockKeyboard = false;
			return true;
		}
		
		case WM_DROPFILES:
		{
			if(dropCommand.empty())	// we shouldn't get here... but somehow we did :p
				break;

			POINT pt;
			GetCursorPos(&pt);
			ScreenToClient(hWnd, &pt);
			for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
			{
				if (currentPart != (*eit)->hiddenPart)
					continue;

				if ( pt.x < ((*eit)->x + (*eit)->width) && pt.x >= (*eit)->x )
				{
					if ( pt.y < ((*eit)->y + (*eit)->height) && pt.y >= (*eit)->y )
					{
						hoverentry = *eit;
						break;
					}
				}
			}

			if ( hoverentry == NULL )
			{
				break;
			}
			else if ( hoverentry != NULL )
			{
				if ( hoverentry->entryType >= 0 || hoverentry->clsid != 0 )
					break;
			}

			int intFileAction = 0;
			char szTarget[MAX_LINE_LENGTH] = {0};

			strcpy(szTarget, TrimAndStripQuote(hoverentry->action).c_str());

			if (_stricmp(dropCommand.c_str(), "move") == 0)
				intFileAction = 1;
			else if (_stricmp(dropCommand.c_str(), "copy") == 0)
				intFileAction = 2;
			else if (_stricmp(dropCommand.c_str(), "shortcut") == 0)
				intFileAction = 3;

			string check = szTarget;
			if (check.substr(1, check.length()-1) == "\\")
				check.append( "*" );
			else
				check.append( "\\*" );
				
			//Test Path for Existence
			WIN32_FIND_DATA FindData;
			HANDLE hFindHandle = FindFirstFile(check.c_str(), &FindData);

			if (intFileAction > 0 && intFileAction < 3)
			{
				char *szSourceFiles = NULL;
				char buffer[MAX_LINE_LENGTH];
				int numFiles;
				UINT len,scursor = 0;

				numFiles = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, NULL);

				szSourceFiles = new char[MAX_LINE_LENGTH*numFiles+1];
				scursor = 0;

				for (int i=0;i<numFiles;i++)
				{
					len = DragQueryFile((HDROP) wParam, i, buffer, MAX_LINE_LENGTH);
					
					strcpy(szSourceFiles+scursor,buffer);
					scursor += len + 1;
				}
					
				DragFinish((HDROP) wParam);

				if( hFindHandle != INVALID_HANDLE_VALUE || intFileAction == 0 )
				{
					szSourceFiles[scursor] = 0;
					strcpy(buffer, szTarget);
					buffer[strlen(buffer)+1] = 0;
					
					SHFILEOPSTRUCT shfop;
					shfop.hwnd = NULL;
					if (intFileAction == 1)
						shfop.wFunc = FO_MOVE;
					else if (intFileAction == 2)
						shfop.wFunc = FO_COPY;
					shfop.pFrom = szSourceFiles;
					shfop.pTo = buffer;
					shfop.fFlags = FOF_NOCONFIRMMKDIR | FOF_ALLOWUNDO;
					SHFileOperation(&shfop);
				}
				
				delete[] szSourceFiles;
			}
			else if (hFindHandle != INVALID_HANDLE_VALUE && intFileAction == 3)
			{
				CString path = szTarget;
				if (path.Right(1) != "\\")
					path += "\\";
				int numDropped;
				char szFile[MAX_LINE_LENGTH];
				
				numDropped = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, 0);
				for (int i = 0; i < numDropped; i++)
				{
					DragQueryFile((HDROP) wParam, i, (char *)&szFile, MAX_LINE_LENGTH);

					HRESULT hres; 
					IShellLink* psl; 

					hres = CoCreateInstance(CLSID_ShellLink, NULL,
					CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID *) &psl); 
					if (SUCCEEDED(hres))
					{
						IPersistFile* ppf; 
						psl->SetPath(szFile); 
						psl->SetDescription("");
 
						hres = psl->QueryInterface(IID_IPersistFile, (LPVOID*)&ppf); 
						if (SUCCEEDED(hres))
						{
							CString work = szFile;
							work = work.Right(work.GetLength()-work.ReverseFind('\\')-1);
							work = work.Left(work.Find("."));
							work += ".lnk";

							WCHAR wsz[MAX_LINE_LENGTH]; 
							MultiByteToWideChar(CP_ACP, 0, (path + work), -1, wsz, MAX_LINE_LENGTH);  
							hres = ppf->Save(wsz, TRUE); 
							ppf->Release(); 
						}
						psl->Release(); 
					}
				}
				DragFinish((HDROP) wParam);	
			}
			FindClose(hFindHandle);
			
			return 0;
		}
	}

	return false;
}

LRESULT WINAPI Popup::windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Popup *popup = (Popup *) GetWindowLong(hWnd, 0);

	if (message == WM_NCCREATE)
	{
		LPCREATESTRUCT pcs = (LPCREATESTRUCT) lParam;
		popup = (Popup *) pcs->lpCreateParams;

		popup->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) popup);
	}

	if (popup)
	{
		LRESULT lResult;

		if (popup->onWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

ENTRY *Popup::ActionAtPoint( int xP, int yP, bool execute, bool context, int filepath)
{
	if ( filepath == 1 )
		dragndropSource = "";
	else if ( filepath == 2 )
		dragndropDestination = "";

	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (currentPart != (*eit)->hiddenPart)
			continue;

		if ( yP < ((*eit)->y + (*eit)->height) && yP >= (*eit)->y )
		{
			if ( xP < ((*eit)->x + (*eit)->width) && xP >= (*eit)->x )
			{
				if ( (*eit)->entryType > 0 )
					return NULL;
				//-------------------------------------------------------------------
				//LEFTCLICK NORMAL EXECUTE AND FOLDEROPENONCLICK (IF ACTIVE)
				//-------------------------------------------------------------------
				if ( execute && !context )
				{
					if ( (*eit)->entryType == 0 )
					{
						if ( !(*eit)->action.empty() || !(*eit)->universalaction.empty() )
						{
							if ( (closeAfterAction && !(0x8000 & GetKeyState(VK_CONTROL))) || (!closeAfterAction && (0x8000 & GetKeyState(VK_CONTROL))))
							{
								Popup *tmpPointer = pParentPopup;
								if (tmpPointer != NULL)
								{
									while (tmpPointer->pParentPopup != NULL)
										tmpPointer = tmpPointer->pParentPopup;

									if ( !execSound.empty() )
									{
										tmpPointer->execSoundPlayed = true;
										ParseBangCommand(hWnd, "!playsound", execSound.c_str());
									}
									tmpPointer->hide(true);
								}
								else
								{	
									if ( !execSound.empty() )
									{
										execSoundPlayed = true;
										ParseBangCommand(hWnd, "!playsound", execSound.c_str());
									}
									hide(true);
								}
							}
							
							if ( !(*eit)->universalaction.empty() )
								LSExecute(NULL, (*eit)->universalaction.c_str(), NULL);
							else
								LSExecute(NULL, (*eit)->action.c_str(), NULL);
						}
						else if ( (*eit)->m_pidl != NULL )
						{
							if ( (closeAfterAction && !(0x8000 & GetKeyState(VK_CONTROL))) || (!closeAfterAction && (0x8000 & GetKeyState(VK_CONTROL))))
							{
								Popup *tmpPointer = pParentPopup;
								if (tmpPointer != NULL)
								{
									while (tmpPointer->pParentPopup != NULL)
										tmpPointer = tmpPointer->pParentPopup;

									if ( !execSound.empty() )
									{
										tmpPointer->execSoundPlayed = true;
										ParseBangCommand(hWnd, "!playsound", execSound.c_str());
									}
									tmpPointer->hide(true);
								}
								else
								{	
									if ( !execSound.empty() )
									{
										execSoundPlayed = true;
										ParseBangCommand(hWnd, "!playsound", execSound.c_str());
									}
									hide(true);
								}
							}

							Invoke((*eit)->m_pidl);
						}
						else if ( (*eit)->hWnd != NULL )
						{
							if ( (closeAfterAction && !(0x8000 & GetKeyState(VK_CONTROL))) || (!closeAfterAction && (0x8000 & GetKeyState(VK_CONTROL))))
							{
								Popup *tmpPointer = pParentPopup;
								if (tmpPointer != NULL)
								{
									while (tmpPointer->pParentPopup != NULL)
										tmpPointer = tmpPointer->pParentPopup;

									if ( !execSound.empty() )
									{
										tmpPointer->execSoundPlayed = true;
										ParseBangCommand(hWnd, "!playsound", execSound.c_str());
									}
									tmpPointer->hide(true);
								}
								else
								{
									if ( !execSound.empty() )
									{
										execSoundPlayed = true;
										ParseBangCommand(hWnd, "!playsound", execSound.c_str());
									}
									hide(true);
								}
							}

							SetForegroundWindow( (*eit)->hWnd );

							if ( IsIconic((*eit)->hWnd) )
							{
								//if ( pSwitchToThisWindow )
								//	pSwitchToThisWindow( (*eit)->hWnd, true );
								//else
									ShowWindowAsync( (*eit)->hWnd, SW_RESTORE );
									BringWindowToTop( (*eit)->hWnd );
							}
							else
							{
								//if ( pSwitchToThisWindow )
								//	pSwitchToThisWindow( (*eit)->hWnd, false );
								//else
									ShowWindowAsync( (*eit)->hWnd, SW_SHOW );
									BringWindowToTop( (*eit)->hWnd );
							}

							SendMessage(GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM)(*eit)->hWnd);
						}

						hoverentry = *eit;
					}
					//Jump Cursor into Folder
					else if ( (*eit)->entryType < 0 )
					{
						bool alreadyCreated = false;
						for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
						{
							if (_stricmp( (*eit)->caption.c_str(), (*plit)->captionname.c_str()) == NULL)
							{
								if ( (0x8000 & GetKeyState(VK_CONTROL)) )
								{
									if ( !(*eit)->action.empty() )
									{
										if ((*eit)->pipedList.size() == 1)
										{
											LSExecute(NULL, (folderDoubleClickAction + " " + (*eit)->action).c_str(), NULL);
										}
										else
										{
											HMENU GroupMenu = CreatePopupMenu();

											int counter = 1;
											for(StringListIterator sit = (*eit)->pipedList.begin(); sit != (*eit)->pipedList.end(); ++sit)
											{
												AppendMenu( GroupMenu, MF_STRING, counter, (*sit).c_str() );
												counter++;
											}

											// display the menu
											POINT pt;
											GetCursorPos( &pt );

											int command = TrackPopupMenu( GroupMenu,
																		  TPM_RETURNCMD | TPM_RIGHTBUTTON,
																		  pt.x, pt.y, 0,
																		  this->gethWnd(),
																		  NULL );

											if ( command )
											{
												StringListIterator sit = (*eit)->pipedList.begin();
												counter = 1; 
												while (counter < command)
												{
													sit++;
													counter++;
												}

												if ( counter <= (*eit)->pipedList.size() )
												{
													LSExecute(NULL, (folderDoubleClickAction + " " + (*sit)).c_str(), NULL);
												}
											}
											
											if (GroupMenu)
												DestroyMenu(GroupMenu);
										}
									}
								}
								else
								{
									(*plit)->show();
								}
								alreadyCreated = true;
							}
							else
							{
								if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
									(*plit)->hide();
							}
						}
						if (!alreadyCreated && (*eit)->clsid == 0 && (*eit)->hWnd == NULL)
							CreateSubMenu( ("sub_"+(*eit)->caption), (*eit)->action, (*eit));
						else if (!alreadyCreated && (*eit)->clsid != 0)
							CreateShellSubMenu( ("sub_"+(*eit)->caption), (*eit)->clsid, (*eit));
						else if (!alreadyCreated && (*eit)->hWnd == hWnd)
							CreateTaskSubMenu( ("sub_"+(*eit)->caption), (*eit));

						hoverentry = *eit;
					}
				}
				//-------------------------------------------------------------------
				//RIGHTCLICK CONTEXT EXECUTE
				//-------------------------------------------------------------------
				//Normal Context Menu
				else if ( !execute && context )
				{
					if ( !(*eit)->action.empty() )
					{
						CString easyWorker = TrimAndStripQuote((*eit)->action).c_str();

						if ( (((*eit)->pipedList.size() == 1 && (*eit)->entryType < 0) || ((*eit)->pipedList.size() <= 1 && (*eit)->entryType >= 0)) && easyWorker.Left(1) != "!" )
						{
							ContextMenu(easyWorker);
						}
						else if ( easyWorker.Left(1) != "!" )
						{
							HMENU GroupMenu = CreatePopupMenu();

							int counter = 1;
							for(StringListIterator sit = (*eit)->pipedList.begin(); sit != (*eit)->pipedList.end(); ++sit)
							{
								AppendMenu( GroupMenu, MF_STRING, counter, (*sit).c_str() );
								counter++;
							}

							// display the menu
							POINT pt;
							GetCursorPos( &pt );

							int command = TrackPopupMenu( GroupMenu,
														  TPM_RETURNCMD | TPM_RIGHTBUTTON,
														  pt.x, pt.y, 0,
														  this->gethWnd(),
														  NULL );

							if ( command )
							{
								StringListIterator sit = (*eit)->pipedList.begin();
								counter = 1; 
								while (counter < command)
								{
									sit++;
									counter++;
								}

								if ( counter <= (*eit)->pipedList.size() )
								{
									easyWorker = TrimAndStripQuote((*sit).c_str()).c_str();
									ContextMenu(easyWorker);
								}
							}
							
							if (GroupMenu)
								DestroyMenu(GroupMenu);
						}
					}
					//ShellFolder Context Menu
					else if ( (*eit)->m_pidl != NULL )
					{
						RInvoke((*eit)->m_pidl);
					}
					//Tasks ContextMenu
					else if ( (*eit)->hWnd != NULL )
					{
						HMENU systemMenu;

						systemMenu = GetSystemMenu( (*eit)->hWnd, FALSE );

						WINDOWPLACEMENT wp;
						wp.length = sizeof(WINDOWPLACEMENT);
						GetWindowPlacement( (*eit)->hWnd, &wp );

						// restore is enabled only if minimized or maximized (not normal)
						EnableMenuItem( systemMenu, SC_RESTORE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

						// move is enabled only if normal
						EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

						// size is enabled only if normal
						EnableMenuItem( systemMenu, SC_SIZE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

						// minimize is enabled only if not minimized
						EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMINIMIZED ? MF_ENABLED : MF_GRAYED) );

						// maximize is enabled only if not maximized
						EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMAXIMIZED ? MF_ENABLED : MF_GRAYED) );

						// let application modify menu
						PostMessage( (*eit)->hWnd, WM_INITMENUPOPUP, (WPARAM) systemMenu, MAKELPARAM( 0, TRUE ) );
						PostMessage( (*eit)->hWnd, WM_INITMENU, (WPARAM) systemMenu, 0 );

						// display the menu
						POINT pt;
						GetCursorPos( &pt );

						int command = TrackPopupMenu( systemMenu,
													  TPM_RETURNCMD | TPM_RIGHTBUTTON,
													  pt.x, pt.y, 0,
													  hWnd,
													  NULL );

						if ( command )
						{
							if ( command != SC_MINIMIZE && command != SC_CLOSE )
							{
								SetForegroundWindow( (*eit)->hWnd );
								
								if ( IsIconic((*eit)->hWnd) )
								{
									//if ( pSwitchToThisWindow )
									//	pSwitchToThisWindow( (*eit)->hWnd, true );
									//else
										ShowWindowAsync( (*eit)->hWnd, SW_RESTORE );
										BringWindowToTop( (*eit)->hWnd );
								}
								else
								{
									//if ( pSwitchToThisWindow )
									//	pSwitchToThisWindow( (*eit)->hWnd, false );
									//else
										ShowWindowAsync( (*eit)->hWnd, SW_SHOW );
										BringWindowToTop( (*eit)->hWnd );
								}

								SendMessage(GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM)(*eit)->hWnd);
							}

							PostMessage( (*eit)->hWnd, WM_SYSCOMMAND, (WPARAM) command, MAKELPARAM( pt.x, pt.y ) );
						}
					}
				}
				else if (!execute && !context && filepath != 0 && !(*eit)->action.empty() && (*eit)->m_pidl == NULL && (*eit)->hWnd == NULL)
				{	
					if ( filepath == 1 )
					{
						dragndropSource.append(	TrimAndStripQuote((*eit)->action, true) );
						if ( !PathFileExists( dragndropSource.c_str() ) )
							dragndropSource = "-1";
					}
					else if ( filepath == 2 )
					{
						dragndropDestination.append( TrimAndStripQuote((*eit)->action, true) );
						
						if ( !PathFileExists( dragndropDestination.c_str() ) || !PathIsDirectory( dragndropDestination.c_str() ) )
							dragndropDestination = "";
						
						if ( dragndropDestination.empty() && !rootScanPath.empty() )
						{
							dragndropDestination.append( TrimAndStripQuote(rootScanPath, true) );
							if ( !PathFileExists( dragndropDestination.c_str() ) || !PathIsDirectory( dragndropDestination.c_str() ) )
								dragndropDestination = "";
						}
					}

					return *eit;
				}
	
				break;
				//-------------------------------------------------------------------
			}
		}
	}

	return NULL;
}

void Popup::HoverActions( int xP, int yP )
{
	if (hoverentry != NULL)
	{
		if ( yP < (hoverentry->y + hoverentry->height) && yP >= hoverentry->y
				&&  xP < (hoverentry->x + hoverentry->width) && xP >= hoverentry->x 
				&& hoverentry->hiddenPart == currentPart )
		{
			return;
		}
	}

	//Check TopBorder and BottomBorder form MultiPartScrolling
	if ( maxPart != 0 )
	{
		if ( yP < topBorder )
		{
			if ( isMultiScrollTimerRunning != 1 )
			{
				if ( isMultiScrollTimerRunning == 2 )
					KillTimer(hWnd, TIMER_MULTISCROLLDOWN);

				isMultiScrollTimerRunning = 1;
				SetTimer(hWnd, TIMER_MULTISCROLLUP, multiscrollDelay, NULL);
			}
		}
		else
		{
			KillTimer(hWnd, TIMER_MULTISCROLLUP);
			
			if ( yP >= actualheight-bottomBorder )
			{
				if ( isMultiScrollTimerRunning != 2 )
				{
					isMultiScrollTimerRunning = 2;
					SetTimer(hWnd, TIMER_MULTISCROLLDOWN, multiscrollDelay, NULL);
				}
			}
			else
			{
				isMultiScrollTimerRunning = 0;
				KillTimer(hWnd, TIMER_MULTISCROLLDOWN);
			}
		}
	}

	bool quickskip = false;
	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if (currentPart != (*eit)->hiddenPart)
			continue;

		if ( yP < ((*eit)->y + (*eit)->height) && yP >= (*eit)->y && !quickskip )
		{
			if ( xP < ((*eit)->x + (*eit)->width) && xP >= (*eit)->x )
			{
				//Separator and InfoEntry cannot be "Active" (Skip)
				if ( (*eit)->entryType == 1 || (*eit)->entryType == 2 )
					continue;

				lastactiveentry = NULL;

				if ( ToolTips && (quicklaunchMode != 0 || tooltipMode != 0) )
				{
					if ( (*eit)->tooltip.empty() )
					{
						if ( quicklaunchMode != 0 || tooltipMode == 1 )
						{
							CString worker = "";
							worker = TrimAndStripQuote((*eit)->action).c_str();
							worker = worker.Right(worker.GetLength()-worker.ReverseFind('\\')-1);
							if (_stricmp(worker.Right(4), ".lnk") == NULL)
								worker = worker.Left(worker.GetLength()-4);

							(*eit)->tooltip = worker;
						}
					}

					updateHint(hWnd, (char*) (*eit)->tooltip.c_str(), true);
				}
				
				if ( !subSound.empty() )
					ParseBangCommand(hWnd, "!playsound", subSound.c_str());
				
				hoverentry = (*eit);

				if ( (*eit)->entryType >= 0 && !(*eit)->action.empty() )
				{
					KillTimer(hWnd, TIMER_HOVER);

					for(PopupListIterator plit = subpopupList.begin(); plit != subpopupList.end(); plit++)
					{
						if ( !(*plit)->PinnedMode && (*plit)->isVisible() )
							(*plit)->hide();
					}
				}
				else if ( (*eit)->entryType < 0 && !folderOpenOnClick )
				{
					//Hack to avoid AutoOpen on Manual Open (Keyboard, Click)
					if ( xP < (*eit)->x+1 || xP > (*eit)->x+3 || yP < (*eit)->y+1 || yP > (*eit)->y+3 )
					{
						KillTimer(hWnd, TIMER_HOVER);

						lastactiveentry = (*eit);

						SetTimer(hWnd, TIMER_HOVER, openDelay, NULL);
					}
					else
					{
						hoverentry = NULL;
					}
				}

				quickskip = true;

				break;
				//-------------------------------------------------------------------
			}
		}
	}

	if (hoverentry != NULL)
	{
		if (!quickskip && hoverentry->entryType >= 0)
			hoverentry = NULL;
	}
	
	repaint();
}

int Popup::isCloseTo(POINT &pos, const POINT size, const int x, const int y, const int cx, const int cy, const int span)
{
	bool internalX = false;
	bool internalY = false;

	if ((pos.x > x-span) && (pos.x < x+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x;
			internalX = true;
		}
	}
	else if ((pos.x > x+cx-span ) && (pos.x < x+cx+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x+cx;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x+cx;
			internalX = true;
		}
	}
	else if ((pos.x+size.x > x-span ) && (pos.x+size.x < x+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x-size.x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x-size.x;
			internalX = true;
		}
	}
	else if ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span))
	{
		if ( ((pos.y > y-span) && (pos.y < y+span)) 
			|| ((pos.y+size.y > y-span) && (pos.y+size.y < y+span)) 
			|| ((pos.y > y+cy-span ) && (pos.y < y+cy+span)) 
			|| ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span)) )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
		else if ( (pos.y > y) && (pos.y < y+cy) 
				&& (pos.y+size.y > y) && (pos.y+size.y < y+cy) )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
	}

	if ((pos.y > y-span) && (pos.y < y+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y;
			internalY = true;
		}
	}
	else if ((pos.y > y+cy-span ) && (pos.y < y+cy+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y+cy;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y+cy;
			internalY = true;
		}
	}
	else if ((pos.y+size.y > y-span ) && (pos.y+size.y < y+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y-size.y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y-size.y;
			internalY = true;
		}
	}
	else if ((pos.y+size.y > y+cy-span ) && (pos.y+size.y < y+cy+span))
	{
		if ( ((pos.x > x-span) && (pos.x < x+span)) 
			|| ((pos.x+size.x > x-span) && (pos.x+size.x < x+span)) 
			|| ((pos.x > x+cx-span ) && (pos.x < x+cx+span)) 
			|| ((pos.x+size.x > x+cx-span ) && (pos.x+size.x < x+cx+span)) )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
		else if ( (pos.x > x) && (pos.x < x+cx) 
				&& (pos.x+size.x > x) && (pos.x+size.x < x+cx) )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
	}

	if (!internalX && !internalY)
		return 0;
	else if (internalX && !internalY)
		return 1;
	else if (!internalX && internalY)
		return 2;
	else
		return 3;
}

void Popup::Sort(vector<string> &inputList)
{
	std::sort( inputList.begin(), inputList.end(), Compare );
}

void Popup::SortTasks(hWndTaskList &inputList)
{
	std::sort( inputList.begin(), inputList.end(), CompareTasks );
}

void Popup::SortShellFolder(vector<LPITEMIDLIST> &inputList)
{
	std::sort( inputList.begin(), inputList.end(), CompareShellFolders );
}

string Popup::InfoUpdate(string infocfg)
{
	if ( strstr(infocfg.c_str(), "[") != NULL && strstr(infocfg.c_str(), "]") != NULL )
	{
		if ( infoExport == 21 )
			infoExport = 1;
		else 
			infoExport++;

		string what = ("specialinfoxpopup" + IntToString( infoExport ));

		ParseBangCommand(hWnd, "!labelinfoexport", ("%#" + what + "%# " + infocfg).c_str());

		return GetxRCString(what.c_str(), "", "");
	}
	else
	{
		return TrimAndStripQuote(infocfg);
	}
}

bool Popup::SelectNextEntry(bool upwards, bool noCursorMove)
{
	if ( entrylist.size() == 1 )
		return false;

	bool truerepaint = false;
	bool activeOneFound = false;
	for (EntryListIterator eit = entrylist.begin(); eit != entrylist.end(); ++eit)
	{
		if ( (*eit) == hoverentry )
		{
			if (upwards)
			{
				if (eit != entrylist.begin())
				{
					eit--;
				}
				else
				{
					eit = entrylist.end();
					eit--;
				}
			}
			else
			{
				if (eit != entrylist.end()-1)
					eit++;
				else
					eit = entrylist.begin();
			}

			//Separator and InfoEntry cannot be "Active" (Skip)
			while ((*eit)->entryType == 1 || (*eit)->entryType == 2)
			{
				if (upwards)
				{
					if (eit != entrylist.begin())
					{
						eit--;
					}
					else
					{
						eit = entrylist.end();
						eit--;
					}
				}
				else
				{
					if (eit != entrylist.end()-1)
						eit++;
					else
						eit = entrylist.begin();
				}
			}

			hoverentry = *eit;

			if (!noCursorMove)
				SetCursorPos(actualx+(*eit)->x+2, actualy+(*eit)->y+2);	

			if ( currentPart != (*eit)->hiddenPart)
			{
				currentPart--;
				if (currentPart < 0)
					currentPart = maxPart;

				truerepaint = true;
			}

			activeOneFound = true;

			break;
		}
	}

	if (!activeOneFound)
	{
		if ( upwards )
		{
			if ( startentry != NULL )
				hoverentry = startentry;

			truerepaint = SelectNextEntry(true);
		}
		else
		{
			if ( startentry != NULL )
			{
				hoverentry = startentry;

				if (!noCursorMove)
					SetCursorPos(actualx+startentry->x+2, actualy+startentry->y+2);	

				if ( currentPart != startentry->hiddenPart)
				{
					currentPart++;
					if (currentPart > maxPart)
						currentPart = 0;

					truerepaint = true;
				}
			}
		}
	}

	return truerepaint;
}

//-----------------------------------------------------------------------
// ShellFolder Support
//-----------------------------------------------------------------------

//Normal Action
void Popup::Invoke(LPITEMIDLIST m_pidlFull)
{
	SHELLEXECUTEINFO sei;

	sei.cbSize = sizeof(SHELLEXECUTEINFO);
	sei.fMask = SEE_MASK_INVOKEIDLIST | SEE_MASK_IDLIST | SEE_MASK_FLAG_NO_UI;
	sei.hwnd = NULL;
	sei.lpVerb = NULL;
	sei.lpFile = NULL;
	sei.lpParameters = NULL;
	sei.lpDirectory = NULL;
	sei.nShow = SW_SHOW;
	sei.lpIDList = m_pidlFull;

	ShellExecuteEx(&sei);
}

//Context Menu
void Popup::RInvoke(LPITEMIDLIST m_pidlFull)
{
	LPMALLOC pMalloc;
	LPSHELLFOLDER psfFolder, psfNextFolder;
	LPITEMIDLIST pidlItem, pidlNextItem, *ppidl;
	LPCONTEXTMENU pContextMenu;
	CMINVOKECOMMANDINFO ici;
	UINT nCount, nCmd;
	HMENU hMenu;

	// Get current cursor position & window
	POINT point;
	HWND hwnd;

	GetCursorPos(&point);
	hwnd = WindowFromPoint(point);

	//
	// Get pointers to the shell's IMalloc interface and the desktop's
	// IShellFolder interface.
	//

	if (SUCCEEDED (SHGetMalloc (&pMalloc)))
	{

		if (!SUCCEEDED (SHGetDesktopFolder (&psfFolder)))
		{
			pMalloc->Release();
		}
		else
		{
			//
			// Convert the path name into a pointer to an item ID list (pidl).
			//
			if (m_pidlFull != NULL)
			{

				if (nCount = GetItemCount (m_pidlFull))
				{ // nCount must be > 0
					//
					// Initialize psfFolder with a pointer to the IShellFolder
					// interface of the folder that contains the item whose context
					// menu we're after, and initialize pidlItem with a pointer to
					// the item's item ID. If nCount > 1, this requires us to walk
					// the list of item IDs stored in m_pidlFull and bind to each
					// subfolder referenced in the list.
					//
					pidlItem = m_pidlFull;

					while (--nCount)
					{
						//
						// Create a 1-item item ID list for the next item in m_pidlFull.
						//

						pidlNextItem = DuplicateItem (pMalloc, pidlItem);

						if (pidlNextItem == NULL)
						{
							psfFolder->Release();
							pMalloc->Release();
							return ;
						}

						//
						// Bind to the folder specified in the new item ID list.
						//
						if (!SUCCEEDED (psfFolder->BindToObject (pidlNextItem, NULL, IID_IShellFolder, (void**)&psfNextFolder)))
						{
							pMalloc->Free(pidlNextItem);
							psfFolder->Release();
							pMalloc->Release();
							return ;
						}

						//
						// Release the IShellFolder pointer to the parent folder
						// and set psfFolder equal to the IShellFolder pointer for
						// the current folder.
						//
						psfFolder->Release();
						psfFolder = psfNextFolder;

						//
						// Release the storage for the 1-item item ID list we created
						// just a moment ago and initialize pidlItem so that it points
						// to the next item in m_pidlFull.
						//
						pMalloc->Free(pidlNextItem);
						pidlItem = GetNextItem (pidlItem);
					}

					//
					// Get a pointer to the item's IContextMenu interface and call
					// IContextMenu::QueryContextMenu to initialize a context menu.
					//
					ppidl = &pidlItem;
					if (SUCCEEDED (psfFolder->GetUIObjectOf (hwnd, 1, (const ITEMIDLIST**)ppidl, (const GUID)IID_IContextMenu, NULL, (void**)&pContextMenu)))
					{

						hMenu = CreatePopupMenu ();
						if (SUCCEEDED (pContextMenu->QueryContextMenu (
						                   hMenu, 0, 1, 0x7FFF, CMF_NORMAL)))
						{

							//
							// Display the context menu.
							//
							nCmd = TrackPopupMenu (hMenu, TPM_LEFTALIGN |
							                       TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD,
							                       point.x, point.y, 0, hwnd, NULL);

							//
							// If a command was selected from the menu, execute it.
							//
							if (nCmd)
							{
								ici.cbSize = sizeof (CMINVOKECOMMANDINFO);
								ici.fMask = SEE_MASK_FLAG_NO_UI;
								ici.hwnd = hwnd;
								ici.lpVerb = MAKEINTRESOURCE (nCmd - 1);
								ici.lpParameters = NULL;
								ici.lpDirectory = NULL;
								ici.nShow = SW_SHOWNORMAL;
								ici.dwHotKey = 0;
								ici.hIcon = NULL;

								pContextMenu->InvokeCommand(&ici);


							}
							DestroyMenu (hMenu);
							pContextMenu->Release();
						}
					}
				}
			}
			//
			// Clean up and return.
			//

			psfFolder->Release();
			pMalloc->Release();
		}
	}
}

//Helpers
LPITEMIDLIST CreateIDList(int cb)
{
	IMalloc *pMalloc;
	LPITEMIDLIST pidl = NULL;

	if (cb <= 0)
		return NULL;

	if (SUCCEEDED(SHGetMalloc(&pMalloc)))
	{
		pidl = (LPITEMIDLIST) pMalloc->Alloc(cb);
		pMalloc->Release();
	}

	if (pidl)
		memset(pidl, 0, cb);

	return pidl;
}

LPITEMIDLIST DuplicateIDList(LPCITEMIDLIST pidl)
{
	LPITEMIDLIST pidlNew;
	int cb = GetIDListSize(pidl);

	if (!pidl)
		return NULL;

	pidlNew = CreateIDList(cb);

	if (pidlNew)
		memcpy(pidlNew, pidl, cb);

	return pidlNew;
}

LPITEMIDLIST JoinIDLists(LPCITEMIDLIST pidlA, LPCITEMIDLIST pidlB)
{
	LPITEMIDLIST pidl;
	int cbA, cbB;

	cbA = pidlA ? GetIDListSize(pidlA) - sizeof(pidlA->mkid.cb) : 0;
	cbB = GetIDListSize(pidlB);

	pidl = CreateIDList(cbA + cbB);

	if (pidl)
	{
		if (pidlA)
			memcpy(pidl, pidlA, cbA);

		memcpy((LPBYTE) pidl + cbA, pidlB, cbB);
	}

	return pidl;
}

int GetIDListSize(LPCITEMIDLIST pidl)
{
	if (!pidl)
		return 0;

	int cb = sizeof(pidl->mkid.cb);

	while (pidl->mkid.cb)
	{
		cb = cb + pidl->mkid.cb;
		pidl = NextID(pidl);
	}

	return cb;
}

LPCITEMIDLIST NextID(LPCITEMIDLIST pidl)
{
	return (LPCITEMIDLIST) ((CONST BYTE *) pidl + pidl->mkid.cb);
}

void FreeIDList(LPITEMIDLIST pidl)
{
	IMalloc *pMalloc;

	if (SUCCEEDED(SHGetMalloc(&pMalloc)))
	{
		pMalloc->Free(pidl);
		pMalloc->Release();
	}
}

// Tooltips for Quicklaunch Mode
void Popup::updateHint(HWND hWnd, LPSTR caption, bool updatesize, bool remove)
{
	if (ToolTips == 0)
		return;	
	
	if (!remove)
	{
		TOOLINFO ti;
		RECT clientRect;

		GetClientRect(hWnd, &clientRect);
		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = TTF_SUBCLASS;
		ti.hwnd = hWnd;
		ti.uId = 0;
		ti.rect = clientRect;
		ti.hinst = hInstance;
		ti.lpszText = caption;
		ti.lParam = 0;

		SendMessage(ToolTips, TTM_UPDATETIPTEXT, 0, LPARAM(&ti));

		if (updatesize)
			SendMessage(ToolTips, TTM_NEWTOOLRECT, 0, LPARAM(&ti));
	}
	else
	{
		TOOLINFO ti;
		RECT emptyRect = {0, 0, 0, 0};

		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = 0;
		ti.hwnd = hWnd;
		ti.uId = 0;
		ti.rect = emptyRect;
		ti.hinst = hInstance;
		ti.lpszText = NULL;
		ti.lParam = 0;

		SendMessage(ToolTips, TTM_DELTOOL, 0, LPARAM(&ti));
	}
}


